<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	defined( '_JEXEC' ) or die( 'Restricted access' );

	jimport('joomla.application.component.controller');

	/**
	 * Mailster Component Digest Controller
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterControllerDigests extends MailsterController
	{
		function __construct(){
			parent::__construct();
		}
								
		/**
		 * logic for cancel an action
		 *
		 * @access public
		 * @return void
		 */
		function cancel()
		{
			// Check for request forgeries
			JRequest::checkToken() or die( 'Invalid Token' );			
			/*$group = JTable::getInstance('mailster_users', '');
			$group->bind(JRequest::get('post'));*/
			$this->setRedirect( 'index.php?option=com_mailster&view=digests' );
		}

		/**
		 * logic to create the new digest screen
		 *
		 * @access public
		 * @return void
		 */
		function add( )
		{
			$this->setRedirect( 'index.php?option=com_mailster&view=digest' );
		}

		/**
		 * logic to create the edit digest screen
		 *
		 * @access public
		 * @return void
		 */
		function edit( )
		{
			$id = JRequest::getInt('id', 0);
			if($id){
				$digestModel = MstFactory::getModel('digest');
				$row = $digestModel->getDigest($id);
				if(!MstFactory::getAuthorization()->isAllowed('core.edit', $row->list_id)){
					return MstFactory::getAuthorization()->noPermissionForAction('digests->edit (core.edit)');
				}
			}
			
			JRequest::setVar( 'view', 'digest' );
			JRequest::setVar( 'hidemainmenu', 1 );

			$model 	= $this->getModel('digest');
			$task 	= JRequest::getVar('task');
			$user = JFactory::getUser();
		
			parent::display();
		}
		
		
		function apply()
		{
			$this->save();
		}

		/**
		 * logic to save a digest
		 *
		 * @access public
		 * @return void
		 */
		function save()
		{
			// Check for request forgeries
			JRequest::checkToken() or die( 'Invalid Token' );
			$log = MstFactory::getLogger();
			$cbUtils = MstFactory::getCBUtils();
			$mstRecipients = MstFactory::getRecipients();
			$subscrUtils = MstFactory::getSubscribeUtils();
				
			$task = JRequest::getVar('task');
			$digestId = JRequest::getInt( 'id', 0);
			
			$post = JRequest::get( 'post' );
			$model = $this->getModel('digest');
			
			if($digestId){
				$row = $model->getDigest($digestId);
				if(!MstFactory::getAuthorization()->isAllowed('core.edit', $row->list_id)){
					return MstFactory::getAuthorization()->noPermissionForAction('digests->save (core.edit)');
				}
			}
											
			$digestId = $model->store($post);
			$msg	= JText::_( 'COM_MAILSTER_DIGEST_SAVED');
						
			switch ($task)
			{
				case 'apply' :
					$link = 'index.php?option=com_mailster&view=digest&task=edit&id='.$digestId;
					break;
			
				default :
					$link = 'index.php?option=com_mailster&view=digests';
				break;
			}
			$this->setRedirect( $link, $msg );
		}

		/**
		 * logic to remove digests
		 *
		 * @access public
		 * @return void
		 */
		function remove()
		{
			$digestModel = MstFactory::getModel('digest');
			$cid = JRequest::getVar( 'cid', array(0), 'post', 'array' );			

			if (!is_array( $cid ) || count( $cid ) < 1) {
				JError::raiseError(500, JText::_( 'COM_MAILSTER_SELECT_AN_ITEM_TO_DELETE' ) );
			}
			
			$digestIds2Remove = array();
			for($i=0; $i < count($cid); $i++){
				$row = $digestModel->getDigest($cid[$i]);
				if(MstFactory::getAuthorization()->isAllowed('core.edit', $row->list_id)){
					$digestIds2Remove[] = $cid[$i];					
				}else{
					MstFactory::getAuthorization()->noPermissionForAction('digests->remove (core.edit)');
				}
			}

			$model = $this->getModel('digest');
			if(!$model->delete($digestIds2Remove)) {
				echo "<script> alert('".$model->getError()."'); window.history.go(-1); </script>\n";
			}

			$msg = count($digestIds2Remove).' '.JText::_( 'COM_MAILSTER_DIGEST_DELETED');

			$cache = JFactory::getCache('com_mailster');
			$cache->clean();

			$this->setRedirect( 'index.php?option=com_mailster&view=digests', $msg );
		}
	}
?>
