<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	defined( '_JEXEC' ) or die( 'Restricted access' );

	jimport('joomla.application.component.controller');

	/**
	 * Mailster Component List Members Controller
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterControllerListMembers extends MailsterController
	{
		function __construct(){
			parent::__construct();
		}
		
		function listMembers()
		{			
			$list_id = JRequest::getInt('listID');
			if(!MstFactory::getAuthorization()->isAllowed('core.edit', $list_id)){
				return MstFactory::getAuthorization()->noPermissionForAction('listmembers->listMembers (core.edit)');
			}
			$model = $this->getModel ( 'listmembers' );
			$view  = $this->getView  ( 'listmembers', 'html'  );
			$view->setModel( $model, true );  // true is for the default model;			
			$listGroupsModel = $this->getModel ( 'listgroups' );			
			$listModel = $this->getModel ( 'list' );			
			$groupModel = $this->getModel ( 'groupusers' );	
			$view->setModel( $listGroupsModel );	
			$view->setModel( $listModel );		
			$view->setModel( $groupModel );
			
			$mstRecipients = MstFactory::getRecipients();
			$repCount = $mstRecipients->getTotalRecipientsCount($list_id);		
			
			if($repCount > MstFactory::getV()->getFtSetting(MstVersionMgmt::MST_FT_ID_REC)){
                JFactory::getApplication()->enqueueMessage(JText::_( 'COM_MAILSTER_TOO_MANY_RECIPIENTS' ), 'warning');
			}
			$view->display();			
		}
		
		function newListGroups()
		{
			$list_id = JRequest::getInt('listID');
			if(!MstFactory::getAuthorization()->isAllowed('core.edit', $list_id)){
				return MstFactory::getAuthorization()->noPermissionForAction('listmembers->newListGroups (core.edit)');
			}
			$app = JFactory::getApplication();			
			$recipCountChanged = false;
			$newGroupIds = JRequest::getVar('groups_selectRight', array());
			$groupCount = count($newGroupIds);
			$mstApp = MstFactory::getApplication();
			$mstRecipients = MstFactory::getRecipients();
			$oldRecipCount = $mstRecipients->getTotalRecipientsCount($list_id); // get old, cached number
			for($i=0; $i < $groupCount; $i++)
			{
				$group_id = $newGroupIds[$i];
				$groupModel = $this->getModel('groupusers');
				$groupUserCount = count($groupModel->getData($group_id));
				if($oldRecipCount + $groupUserCount <=  MstFactory::getV()->getFtSetting(MstVersionMgmt::MST_FT_ID_REC)){
					$recipCountChanged = true;
					$model = $this->getModel('listgroups');
					$listGroup = new stdClass();
					$listGroup->group_id	= $group_id;
					$listGroup->list_id		= $list_id;
					$returnid = $model->store($listGroup);			
					$oldRecipCount = $oldRecipCount + $groupUserCount; 
					$app->enqueueMessage( JText::_( 'COM_MAILSTER_ADDED_GROUP' ) . ' ( ID=' . $group_id . ' )');
				}else{
					$app->enqueueMessage( JText::_( 'COM_MAILSTER_COULD_NOT_ADD_GROUP_X' ) . ' (ID = ' . $group_id . ') -  ' . JText::_( 'COM_MAILSTER_TOO_MANY_RECIPIENTS' )  , 'notice');
				}		
				
			}
			if($recipCountChanged){
				$mstRecipients->recipientsUpdated($list_id);  // update cache state
			}
			$this->setRedirect( 'index.php?option=com_mailster&controller=listmembers&task=listmembers&listID='.$list_id );			
		}
				
		function newListMembers()
		{
			$list_id = JRequest::getInt('listID');
			if(!MstFactory::getAuthorization()->isAllowed('core.edit', $list_id)){
				return MstFactory::getAuthorization()->noPermissionForAction('listmembers->newListGroups (core.edit)');
			}
			$log = MstFactory::getLogger();
			$app = JFactory::getApplication();
			$listUtils = MstFactory::getMailingListUtils();
			$recips = MstFactory::getRecipients();
			$pairlist_precodes = JRequest::getString('pairlist_precodes', '');
			$plist_pcodes = explode(';', $pairlist_precodes);
			$newUserCr = 0;		
			$mstApp = MstFactory::getApplication();
			$mstRecipients = MstFactory::getRecipients();
			$repCount = $mstRecipients->getTotalRecipientsCount($list_id);
			$mList = $listUtils->getMailingList($list_id);
			$userModel = MstFactory::getModel('user');
			$subscrUtils = MstFactory::getSubscribeUtils();
			
			for($i=0; $i < count($plist_pcodes); $i++){
				
				if($plist_pcodes[$i] == 'users')
				{
					$newUserIds = JRequest::getVar('users_selectRight', array());
					$userCount = count($newUserIds);
					for($j=0; $j < $userCount; $j++)
					{
						$userIdsStr	= $newUserIds[$j];
						if($userIdsStr != '')
						{
							if($repCount < MstFactory::getV()->getFtSetting(MstVersionMgmt::MST_FT_ID_REC))
							{
								$userIdsArr = explode(';', $userIdsStr);
								$user_id = $userIdsArr[0];
								$is_joomla_user = $userIdsArr[1];
								$userRow = $userModel->getUserData($user_id, $is_joomla_user);
								$userName =  '[ID='.$user_id.'] '.$userRow->name.' ('.$userRow->email.')';
								$isRecip = $recips->isRecipient($list_id, $userRow->email);
								
								$model = $this->getModel('listmembers');
								$mstRecipients->recipientsUpdated($list_id); // update cache state
								$listMember = new stdClass();
								$listMember->user_id		= $user_id;
								$listMember->list_id		= $list_id;					
								$listMember->is_joomla_user	= $is_joomla_user;									
								$success = $model->store($listMember);
								if($success){
									$newUserCr++;
									$repCount++;
									$listMember = new stdClass();
									$model = $this->getModel('listmembers');
								}
								if($mList->welcome_msg > 0 && $mList->welcome_msg_admin > 0 && !$isRecip){ // if user was not a recipient before send out welcome message if admin-msgs are active
									$log->debug('User '.$userName.' was no recipient before, admin-welcome-msg are active, therefore send out welcome msg');
									$subscrUtils->sendWelcomeOrGoodbyeSubscriberMsg($userRow->name, $userRow->email, $list_id, MstConsts::SUB_TYPE_SUBSCRIBE);
								}elseif($mList->welcome_msg <= 0 || $mList->welcome_msg_admin <= 0){
									$log->debug('Do not send out welcome msg to user '.$userName);
								}elseif($isRecip){
									$log->debug('User '.$userName.' already recipient of list '.$list_id.', therefore skip welcome msg');
								}
							}else{	
								$app->enqueueMessage( JText::_( 'COM_MAILSTER_COULD_NOT_ADD_MORE_USERS' ) . ' - ' . JText::_( 'COM_MAILSTER_TOO_MANY_RECIPIENTS' ) , 'notice');
								break;
							}		
						}
					}
				}
			}	
					
			$app->enqueueMessage( $newUserCr . ' ' . JText::_( 'COM_MAILSTER_X_USERS_ADDED' ));								
			$this->listMembers();			
		}

		function exportUsers(){
			$list_id = JRequest::getInt('listID', 0);
			$this->setRedirect( 'index.php?option=com_mailster&controller=csv&task=export&export_source=list&sourcelist='.$list_id);
		}
			
		
		function cancel()
		{
			// Check for request forgeries
			JRequest::checkToken() or die( 'Invalid Token' );
			
			$listmembers = JTable::getInstance('mailster_list_members', '');
			$listmembers->bind(JRequest::get('post'));

			$this->setRedirect( 'index.php?option=com_mailster&view=listmembers' );
		}
		
		function removeUsers()
		{
			$list_id = JRequest::getInt('listID');
			if(!MstFactory::getAuthorization()->isAllowed('core.edit', $list_id)){
				return MstFactory::getAuthorization()->noPermissionForAction('listmembers->newListGroups (core.edit)');
			}
			$log = MstFactory::getLogger();
			$app = JFactory::getApplication();	
			$listUtils = MstFactory::getMailingListUtils();
			$recips = MstFactory::getRecipients();
			$subscrUtils = MstFactory::getSubscribeUtils();
			$userModel = MstFactory::getModel('user');
			
			$user_ids = JRequest::getVar( 'cid', array(0), 'post', 'array' );
			$log->debug('Remove users called with ' . count($user_ids) . ' user ids');
			
			$mList = $listUtils->getMailingList($list_id);
			
			$is_joomla_user_flags_str = JRequest::getString('is_j_us_str', "");
			$is_joomla_user_flags = explode(";", $is_joomla_user_flags_str);
			$log->debug('is joomla user flags str: ' . $is_joomla_user_flags_str . ' -> ' . count($is_joomla_user_flags) . ' users');
			$total = 0;
			$mstRecipients = MstFactory::getRecipients();
			$mstRecipients->recipientsUpdated($list_id); // update cache state

			if (!is_array( $user_ids ) || count( $user_ids ) < 1 || ($user_ids[0] == 0)){
				$app->enqueueMessage(JText::_( 'COM_MAILSTER_SELECT_AN_ITEM_TO_DELETE'), 'warning' );
			}

            $log->debug('Deleting user(s) from list ID '.$list_id.': '.print_r($user_ids, true).', is_joomla_user_flags: '.print_r($is_joomla_user_flags, true));
			$model = $this->getModel('listmembers');
			if(!$model->delete($list_id, $user_ids, $is_joomla_user_flags)) {
                $app->enqueueMessage(JText::_( 'COM_MAILSTER_ERROR'.': '.$model->getError()), 'error' );
                $log->error('Error deleting user(s) from list ID '.$list_id.': '.print_r($user_ids, true).', is_joomla_user_flags: '.print_r($is_joomla_user_flags, true));
			}
			
			for($i = 0; $i < count($user_ids); $i++){
				if($user_ids[$i] > 0){
					$total++;
					$userRow = $userModel->getUserData($user_ids[$i], $is_joomla_user_flags[$i]);
					$userName =  '[ID='.$user_ids[$i].'] '.$userRow->name.' ('.$userRow->email.')';
					$isRecip = $recips->isRecipient($list_id, $userRow->email);
					if($mList->goodbye_msg > 0 && $mList->goodbye_msg_admin > 0 && !$isRecip){
						// goodbye messages are activated for admin actions, user is now not a recipient, therefore goodbye message shall be sent
						$log->debug('User '.$userName.' is not anymore a recipient of list '.$mList->name.' [ID='.$list_id.'], send out goodbye message');
						$subscrUtils->sendWelcomeOrGoodbyeSubscriberMsg($userRow->name, $userRow->email, $list_id, MstConsts::SUB_TYPE_UNSUBSCRIBE);
					}elseif($mList->goodbye_msg <= 0 || $mList->goodbye_msg_admin <= 0){
						$log->debug('Do not send out goodbye msg to user '.$userName);
					}elseif($isRecip){
						$log->debug('User '.$userName.' still recipient of list '.$list_id.', therefore skip goodbye welcome msg');
					}
				}
			}

			if($total > 0){
				$msg = $total.' '.JText::_( 'COM_MAILSTER_MEMBERS_REMOVED_FROM_LIST');
			}
			$app->enqueueMessage($msg );

			$this->listMembers();
		}	

		function removeGroups()
		{
		$list_id = JRequest::getInt('listID');
			if(!MstFactory::getAuthorization()->isAllowed('core.edit', $list_id)){
				return MstFactory::getAuthorization()->noPermissionForAction('listmembers->newListGroups (core.edit)');
			}
			$app = JFactory::getApplication();		
			$group_ids = JRequest::getVar( 'cid', array(0), 'post', 'array' );
			$total = count( $group_ids );
			if($total && $group_ids[0] == 0){
				$total = 0;
			}
			$mstRecipients = MstFactory::getRecipients();
			$mstRecipients->recipientsUpdated($list_id); // update cache state

			if ( !is_array( $group_ids ) || count( $group_ids ) < 1 || ($group_ids[0] == 0)){
				$app->enqueueMessage(JText::_( 'COM_MAILSTER_SELECT_AN_ITEM_TO_DELETE'), 'warning' );
			}

			$model = $this->getModel('listgroups');
			if(!$model->delete($list_id, $group_ids)) {
				echo "<script> alert('".$model->getError()."'); window.history.go(-1); </script>\n";
			}
			if($total > 0){
				$msg = $total.' '.JText::_( 'COM_MAILSTER_GROUPS_REMOVED_FROM_LIST');
			}
			$app->enqueueMessage($msg );
			
			$this->listMembers();
		
		}
		
	}
?>
