<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	defined( '_JEXEC' ) or die( 'Restricted access' );

	jimport('joomla.application.component.controller');

	/**
	 * Mailster Component List Controller
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterControllerLists extends MailsterController
	{
		function __construct(){			
			parent::__construct();
		}
		
		/**
		 * logic for cancel an action
		 */
		function cancel()
		{
			// Check for request forgeries
			JRequest::checkToken() or die( 'Invalid Token' );
			
			$group = JTable::getInstance('mailster_lists', '');
			$group->bind(JRequest::get('post'));

			$this->setRedirect( 'index.php?option=com_mailster&view=lists' );
		}

		/**
		 * logic to create the new mailing list screen
		 */
		function add( )
		{
			if($this->canDo->get('core.create')){
				$this->edit(true);
			}else{
				return MstFactory::getAuthorization()->noPermissionForAction('lists->add (core.create)');
			}				
		}
		
		/**
		 * logic to create the edit mailing list screen
		 */
		function edit($isNewList=false)
		{			
			$log = MstFactory::getLogger();
			$cid = JRequest::getVar( 'cid');
            $isAllowed = false;
            if($isNewList){
                $isAllowed = true;
                if(count(MstFactory::getMailingListUtils()->getAllMailingLists(false))+1 > MstFactory::getV()->getFtSetting(MstVersionMgmt::MST_FT_ID_MLT)){
                    JFactory::getApplication()->enqueueMessage(JText::_( 'COM_MAILSTER_TOO_MANY_MAILING_LISTS_PRODUCT_LIMIT' ), 'warning');
                    return; // do not go to add/edit screen
                }
            }
			if($cid && is_array($cid) && $cid[0] > 0){
				$isAllowed = MstFactory::getAuthorization()->isAllowed('core.edit', $cid[0]);
				$log->debug('lists->edit: isAllowed '.($isAllowed?'yes':'no'));
			}
			if(!$isAllowed){
				return MstFactory::getAuthorization()->noPermissionForAction('lists->edit (core.edit) - 4acR');
			}
			
			JRequest::setVar( 'view', 'list' );
			JRequest::setVar( 'hidemainmenu', 1 );
			
			$model = $this->getModel ( 'list' );
			$view  = $this->getView  ( 'list', 'html'  );
			$view->setModel( $model, true );  // default model		
			$groupModel = $this->getModel ( 'groups' );
			$view->setModel( $groupModel );	
			$view->display();		
		}
	
		/**
		 * To be called from Ajax call, logic to toggle active/deactivate mailing lists
		 */
		function toggleActive( )
		{
      		$app = JFactory::getApplication();
			$mstUtils = MstFactory::getUtils();
			$resultArray = array();
			$debug = 'Debug';
			$resultArray['res'] = JText::_( 'COM_MAILSTER_TOGGLE_ACTIVE_CALLED' );	
			$cid = JRequest::getVar( 'cid', array(), 'get', 'array' );
			$model = $this->getModel('list');
			$debug = $debug . print_r($cid,  true);
			for($i=0;$i<count($cid);$i++){
				$model->setId($cid[$i]);
				$row = $model->getData();
				if($row && $row->id > 0){
					$debug = $debug . ' ' . $row->id;
					if($row->active > 0){
						$row->active = 0;	
					}else{
						$row->active = 1;
					}
					if($model->store($row)){
						$resultArray['res'] = 'true';	
					}else{
						$resultArray['res'] = 'false';	
					}
				}
			}	
			$resultArray['debug'] = $debug;
			$jsonStr  = $mstUtils->jsonEncode($resultArray);
			echo "[" . $jsonStr . "]";	
        	$app->close(); // Close the application.
		}
		
		
		/**
		 * logic to copy mailing lists
		 */
		function copy( )
		{
			if(!$this->canDo->get('core.create')){
				return MstFactory::getAuthorization()->noPermissionForAction('lists->copy (core.create)');
			}
			$nrCopies = 0;
			$cid = JRequest::getVar( 'cid', array(), 'post', 'array' );
			$model = $this->getModel('list');
			for($i=0;$i<count($cid);$i++){
                if(count(MstFactory::getMailingListUtils()->getAllMailingLists(false))+1 > MstFactory::getV()->getFtSetting(MstVersionMgmt::MST_FT_ID_MLT)){
                    JFactory::getApplication()->enqueueMessage(JText::_( 'COM_MAILSTER_TOO_MANY_MAILING_LISTS_PRODUCT_LIMIT' ), 'warning');
                    break; // skip further loop
                }
				$model->setId($cid[$i]);
				$row = $model->getData();
				if($row && $row->id > 0){
					$row->id = null;
					$row->active = 0;
					$row->name = $row->name . ' - ' . JText::_( 'COM_MAILSTER_COPY');
					if($model->store($row)){
						$nrCopies++;
					}
				}
			}
			$link = 'index.php?option=com_mailster&view=lists';
			$msg = JText::sprintf( 'COM_MAILSTER_COPIED_X_MAILING_LISTS', $nrCopies);
			$this->setRedirect( $link, $msg );
		}
		
		function apply()
		{
			$this->save();
		}
			

		/**
		 * logic to save a mailing list
		 */
		function save()
		{
			// Check for request forgeries
			JRequest::checkToken() or die( 'Invalid Token' );
            $log = MstFactory::getLogger();
            $mailUtils = MstFactory::getMailUtils();
            $convUtils = MstFactory::getConverterUtils();
            /** @var MailsterModelServer $serverModel */
            $serverModel = MstFactory::getModel('server');
			if(!$this->canDo->get('core.edit')){
                $log->error('MailsterControllerLists->save() Authorization Failure');
				return MstFactory::getAuthorization()->noPermissionForAction('lists->save (core.edit)');
			}
			$task = JRequest::getVar('task');

			$post = JRequest::get( 'post' );
			$post['custom_header_html'] = JRequest::getVar( 'custom_header_html', '', 'post', 'string', JREQUEST_ALLOWHTML );
            $log->debug('MailsterControllerLists->save() custom_header_html before: '.$post['custom_header_html']);
            $post['custom_header_html'] = $mailUtils->convertRelativeToAbsoluteSiteURLsInHTMLText($post['custom_header_html']);
            $log->debug('MailsterControllerLists->save() custom_header_html after relative->absolute URL: '.$post['custom_header_html']);
            $post['custom_header_html'] = $mailUtils->convertRelativeToAbsoluteImageSourcesInHTMLText($post['custom_header_html']);
            $log->debug('MailsterControllerLists->save() custom_header_html after relative->absolute IMG SRC: '.$post['custom_header_html']);

			$post['custom_footer_html'] = JRequest::getVar( 'custom_footer_html', '', 'post', 'string', JREQUEST_ALLOWHTML );
            $log->debug('MailsterControllerLists->save() custom_footer_html before: '.$post['custom_footer_html']);
            $post['custom_footer_html'] = $mailUtils->convertRelativeToAbsoluteSiteURLsInHTMLText($post['custom_footer_html']);
            $log->debug('MailsterControllerLists->save() custom_footer_html after relative->absolute URL: '.$post['custom_footer_html']);
            $post['custom_footer_html'] = $mailUtils->convertRelativeToAbsoluteImageSourcesInHTMLText($post['custom_footer_html']);
            $log->debug('MailsterControllerLists->save() custom_footer_html after relative->absolute IMG SRC: '.$post['custom_footer_html']);

			$post['datdescription'] = JRequest::getVar( 'datdescription', '', 'post','string', JREQUEST_ALLOWRAW );
			$post['datdescription']	= str_replace( '<br>', '<br />', $post['datdescription'] );
			$tab = JRequest::getString('tab', '#first');

            $log->debug('lists->save() post: '.print_r($post, true));

            $server_inb_id = $post['server_inb_id'];
            $server_out_id = $post['server_out_id'];
            if($server_inb_id <= 0){
                // need to create new inbound server with given info
                $server = new stdClass();
                $server->id = 0;
                $server->published = 1;
                $server->provider_type = MstConsts::SERVER_PROVIDER_TYPE_USER_SPECIFIC;
                $server->user_edited = 1; // created means edited by user
                $server->name = $post['mail_in_host'] . ' [' . strtoupper($post['mail_in_protocol']) . ']';
                $server->server_type = MstConsts::SERVER_TYPE_MAIL_INBOX;
                $server->server_host = $post['mail_in_host'];
                $server->server_port = $post['mail_in_port'];
                $server->secure_protocol = $post['mail_in_use_secure'];
                $server->secure_authentication = $post['mail_in_use_sec_auth'];
                $server->protocol = $post['mail_in_protocol'];
                $server->connection_parameter = $post['mail_in_params'];
                $server->api_key1 = null;
                $server->api_key2 = null;
                $server->api_endpoint = null;

                $post['server_inb_id'] = $serverModel->store($convUtils->object2Array($server));
                $log->debug('MailsterControllerLists->save() Stored new inbound server with ID '.$post['server_inb_id'].': '.print_r($server, true));
            }
            if($post['use_cms_mailer'] <= 0 && $server_out_id <= 0){
                // need to create new outbound server with given info
                $server = new stdClass();
                $server->id = 0;
                $server->published = 1;
                $server->provider_type = MstConsts::SERVER_PROVIDER_TYPE_USER_SPECIFIC;
                $server->user_edited = 1; // created means edited by user
                $server->name = $post['mail_out_host'] . ' [SMTP]';
                $server->server_type = MstConsts::SERVER_TYPE_SMTP;
                $server->server_host = $post['mail_out_host'];
                $server->server_port = $post['mail_out_port'];
                $server->secure_protocol = $post['mail_out_use_secure'];
                $server->secure_authentication = $post['mail_out_use_sec_auth'];
                $server->protocol = 'smtp';
                $server->connection_parameter = null;
                $server->api_key1 = null;
                $server->api_key2 = null;
                $server->api_endpoint = null;

                $post['server_out_id'] = $serverModel->store($convUtils->object2Array($server));
                $log->debug('MailsterControllerLists->save() Stored new outbound server with ID '.$post['server_out_id'].': '.print_r($server, true));
            }

            /**
             * @var $model MailsterModelList
             */
            $model = $this->getModel('list');
			if ($returnid = $model->store($post)) {
				switch ($task){
					case 'apply' :
						$link = 'index.php?option=com_mailster&controller=lists&task=edit&cid[]='.$returnid.$tab;
						break;
					default :
						$link = 'index.php?option=com_mailster&view=lists';
						break;
				}
				$msg	= JText::_( 'COM_MAILSTER_MAILING_LIST_SAVED');
				$cache = JFactory::getCache('com_mailster');
				$cache->clean();
			} else {
				$msg 	= '';
				$link = 'index.php?option=com_mailster&view=lists';

			}
			
			// Store notifications
			$notifyUtils = MstFactory::getNotifyUtils();
			$listId = JRequest::getInt('id', -1);
			
			for($i=0; $i<100; $i++){ // FIXME hope for the best that we do not have to deal with more than 100 notifys
				$notifyIdInput = 'notifyId' . $i;
				$notifyId = JRequest::getInt($notifyIdInput, -1);
				if($notifyId >= 0){
					$triggerTypeInput = 'triggerType'.$i;
					$targetTypeInput = 'targetType'.$i;
					$targetIdInput = 'targetId'.$i;
					$triggerType = JRequest::getInt($triggerTypeInput, -1);
					$targetType = JRequest::getInt($targetTypeInput, -1);
					$targetId = JRequest::getInt($targetIdInput, -1);
					
					$notify = $notifyUtils->createNewNotify();
					$notify->id = $notifyId;
					$notify->list_id = $listId;
					$notify->notify_type = MstNotify::NOTIFY_TYPE_LIST_BASED;
					$notify->trigger_type = $triggerType;
					$notify->target_type = $targetType;
					$notify->setTargetId($targetId);
					
					$notifyUtils->storeNotify($notify);
				}
			}
			
			$this->setRedirect( $link, $msg );
		}

		/**
		 * logic to remove mailing list
		 */
		function remove()
		{
			$cid = JRequest::getVar( 'cid', array(0), 'post', 'array' );

			$total = count( $cid );

			if (!is_array( $cid ) || count( $cid ) < 1) {
				JError::raiseError(500, JText::_( 'COM_MAILSTER_SELECT_AN_ITEM_TO_DELETE' ) );
			}
			
			$listIds2Remove = array();
			for($i=0; $i < count($cid); $i++){
				if(MstFactory::getAuthorization()->isAllowed('core.delete', $cid[$i])){
					$listIds2Remove[] = $cid[$i];
				}else{
					MstFactory::getAuthorization()->noPermissionForAction('lists->remove (core.delete) for list ID '.$cid[$i]);
				}
			}

			$model = $this->getModel('lists');
			if(!$model->delete($listIds2Remove)) {
				echo "<script> alert('".$model->getError()."'); window.history.go(-1); </script>\n";
			}

			$msg = $total.' '.JText::_( 'COM_MAILSTER_MAILING_LIST_DELETED');

			$cache = JFactory::getCache('com_mailster');
			$cache->clean();

			$this->setRedirect( 'index.php?option=com_mailster&view=lists', $msg );
		}
		
		/**
		 * To be called by an Ajax call
		 */
		function removeNotify(){
			if(!$this->canDo->get('core.edit')){
				return MstFactory::getAuthorization()->noPermissionForAction('lists->removeNotify (core.edit)');
			}
      		$app = JFactory::getApplication();
			$notifyUtils = MstFactory::getNotifyUtils();
			$mstUtils = MstFactory::getUtils();
			$resultArray = array();
			$res = JText::_( 'COM_MAILSTER_REMOVE_NOTIFY_CALLED' );	
			$ajaxParams = JRequest::getString('mtrAjaxData');
			$ajaxParams = $mstUtils->jsonDecode($ajaxParams);
			$task = $ajaxParams->{'task'};			
			if($task == 'removeNotify'){			
				$notifyId = $ajaxParams->{'notifyId'};
				$rowNr = $ajaxParams->{'rowNr'};
				$res = ($notifyUtils->deleteNotify($notifyId) ? 'true' : 'false');
			}
			$resultArray['res'] = $res;	
			$resultArray['notifyId'] = $notifyId;	
			$resultArray['rowNr'] = $rowNr;				
			$jsonStr  = $mstUtils->jsonEncode($resultArray);
			echo "[" . $jsonStr . "]";	
        	$app->close(); // Close the application.
		}
		
		/**
		 * To be called by an Ajax call
		 */
		function getInboxStatus(){
      		$app = JFactory::getApplication();
			$mstUtils 	= MstFactory::getUtils();
			$listUtils	= MstFactory::getMailingListUtils();
			$mailbox 	= MstFactory::getMailingListMailbox();
			$resultArray = array();
			$res = JText::_( 'COM_MAILSTER_GET_INBOX_STATUS_CALLED' );
			$ajaxParams = JRequest::getString('mtrAjaxData');
			$ajaxParams = $mstUtils->jsonDecode($ajaxParams);
			$res = true;
			
			$mailboxStatus = false;
			$listId = $ajaxParams->{'listId'};
			$mList = $listUtils->getMailingList($listId);
			if($listUtils->lockMailingList($mList->id)){
				if($mailbox->open($mList)){
					$mailboxStatus = $mailbox->getInboxStatus();
					$mailbox->close();
				}else{
					$res = false;
				}
				$listUtils->unlockMailingList($mList->id);
			}else{
				$res = false;
			}
				
			$res = ($res ? 'true' : 'false');
			
			$mailboxStatusText = JText::_( 'COM_MAILSTER_INBOX_STATUS' )."\r\n\r\n".print_r($mailboxStatus, true);
			$resultArray['res'] = $res;
			$resultArray['mailboxStatus'] = $mailboxStatus;
			$resultArray['mailboxStatusText'] = $mailboxStatusText;
			$jsonStr  = $mstUtils->jsonEncode($resultArray);
			echo "[" . $jsonStr . "]";	
        	$app->close(); // Close the application.
		}
		
		/**
		 * To be called by an Ajax call
		 */
		function removeFirstMailFromMailbox(){
			if(!$this->canDo->get('core.edit')){
				return MstFactory::getAuthorization()->noPermissionForAction('lists->removeFirstMailFromMailbox (core.edit)');
			}
      		$app = JFactory::getApplication();
			$mstUtils 	= MstFactory::getUtils();
			$listUtils	= MstFactory::getMailingListUtils();
			$mailbox 	= MstFactory::getMailingListMailbox();
			$resultArray = array();
			$res = JText::_( 'COM_MAILSTER_REMOVE_FIRST_MAIL_FROM_MAILBOX_CALLED' );	
			$ajaxParams = JRequest::getString('mtrAjaxData');
			$ajaxParams = $mstUtils->jsonDecode($ajaxParams);
			$res = true;
			
			$listId = $ajaxParams->{'listId'};
			$mList = $listUtils->getMailingList($listId);
			if($listUtils->lockMailingList($mList->id)){
				if($mailbox->open($mList)){
					$mailbox->removeFirstMailFromMailbox();
					$mailbox->close();
				}else{
					$res = false;
				}
				$listUtils->unlockMailingList($mList->id);
			}else{
				$res = false;
			}
				
			$res = ($res ? 'true' : 'false');
			
			$resultArray['res'] = $res;			
			$jsonStr  = $mstUtils->jsonEncode($resultArray);
			echo "[" . $jsonStr . "]";	
        	$app->close(); // Close the application.
		}
		
		/**
		 * To be called by an Ajax call
		 */
		function removeAllMailsFromMailbox(){
			if(!$this->canDo->get('core.edit')){
				return MstFactory::getAuthorization()->noPermissionForAction('lists->removeAllMailsFromMailbox (core.edit)');
			}
      		$app = JFactory::getApplication();
			$mstUtils 	= MstFactory::getUtils();
			$listUtils	= MstFactory::getMailingListUtils();
			$mailbox 	= MstFactory::getMailingListMailbox();
			$resultArray = array();
			$res = JText::_( 'COM_MAILSTER_REMOVE_ALL_MAILS_FROM_MAILBOX_CALLED' );	
			$ajaxParams = JRequest::getString('mtrAjaxData');
			$ajaxParams = $mstUtils->jsonDecode($ajaxParams);
			$res = true;
			
			$listId = $ajaxParams->{'listId'};
			$mList = $listUtils->getMailingList($listId);
			if($listUtils->lockMailingList($mList->id)){
				if($mailbox->open($mList)){
					$mailbox->removeAllMailsFromMailbox();
					$mailbox->close();
				}else{
					$res = false;
				}
				$listUtils->unlockMailingList($mList->id);
			}else{
				$res = false;
			}
				
			$res = ($res ? 'true' : 'false');
			
			$resultArray['res'] = $res;			
			$jsonStr  = $mstUtils->jsonEncode($resultArray);
			echo "[" . $jsonStr . "]";	
        	$app->close(); // Close the application.
		}
		
		/**
		 * To be called by an Ajax call
		 */
		function removeAllMailsInSendQueue(){
			if(!$this->canDo->get('core.edit')){
				return MstFactory::getAuthorization()->noPermissionForAction('lists->removeAllMailsInSendQueue (core.edit)');
			}
      		$app = JFactory::getApplication();
			$mstUtils 	= MstFactory::getUtils();
			$mstQueue = MstFactory::getMailQueue();
			$resultArray = array();
			$res = JText::_( 'COM_MAILSTER_REMOVE_ALL_MAILS_FROM_QUEUE_CALLED' );	
			$ajaxParams = JRequest::getString('mtrAjaxData');
			$ajaxParams = $mstUtils->jsonDecode($ajaxParams);
			$res = true;
			
			$listId = $ajaxParams->{'listId'};
			$res = $mstQueue->removeAllMailsFromListFromQueue($listId);
				
			$res = ($res ? 'true' : 'false');
			
			$resultArray['res'] = $res;			
			$jsonStr  = $mstUtils->jsonEncode($resultArray);
			echo "[" . $jsonStr . "]";	
        	$app->close(); // Close the application.
		}
		
		/**
		 * To be called by an Ajax call
		 */
		function unlockMailingList(){
			if(!$this->canDo->get('core.edit')){
				return MstFactory::getAuthorization()->noPermissionForAction('lists->unlockMailingList (core.edit)');
			}
      		$app = JFactory::getApplication();
			$mstUtils 	= MstFactory::getUtils();
			$listUtils 	= MstFactory::getMailingListUtils();
			$resultArray = array();
			$res = JText::_( 'COM_MAILSTER_UNLOCK_MAILING_LIST_CALLED' );	
			$ajaxParams = JRequest::getString('mtrAjaxData');
			$ajaxParams = $mstUtils->jsonDecode($ajaxParams);
			$res = true;
			
			$listId = $ajaxParams->{'listId'};
			$res = $listUtils->unlockMailingList($listId);
			$res = ($res ? 'true' : 'false');
			
			$resultArray['res'] = $res;			
			$jsonStr  = $mstUtils->jsonEncode($resultArray);
			echo "[" . $jsonStr . "]";	
        	$app->close(); // Close the application.
		}
	}
?>
