<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	defined( '_JEXEC' ) or die( 'Restricted access' );

	jimport('joomla.application.component.controller');

	/**
	 * Mailster Component Maintenance Controller
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterControllerMaintenance extends MailsterController
	{
		function __construct(){
			parent::__construct();			
		}
		
		function fixDBCollation(){
            $log = MstFactory::getLogger();
            $log->debug('maintenance->fixDBCollation()');
			$dbUtils = MstFactory::getDBUtils();
			$dbCollationOk 	= $dbUtils->userTableCollationOk();
			$app = JFactory::getApplication();
			
			if(!$dbCollationOk){
                $log->error('maintenance->fixDBCollation() collation NOT okay');
			    if(method_exists($dbUtils, 'fixDBCollation')){
                    $dbUtils->fixDBCollation(true);
                }
			}else{
                $log->debug('maintenance->fixDBCollation() collation okay');
            }
			$app->redirect('index.php?option=com_mailster');
		}
		
		function deleteLogFile(){
			$app = JFactory::getApplication();
			$log = MstFactory::getLogger();
			$logFile = $log->getLogFile();
			jimport('joomla.filesystem.file'); // For using JFile

			if(JFile::delete($logFile)){
				$app->enqueueMessage(JText::_( 'COM_MAILSTER_DELETED_LOG_FILE' ));
			}else{
				$app->enqueueMessage(JText::_( 'COM_MAILSTER_FAILED_TO_DELETE_LOG_FILE' ), 'error');
			}
			$app->redirect('index.php?option=com_mailster');
		}
		
		function purgeLogDatabase(){
			$app = JFactory::getApplication();
			$logModel = MstFactory::getModel('log');

			if($logModel->clearLog()){
				$app->enqueueMessage(JText::_( 'COM_MAILSTER_LOG_WAS_CLEARED' ));
			}else{
				$app->enqueueMessage(JText::_( 'COM_MAILSTER_FAILED_TO_CLEAR_LOG' ), 'error');
			}
			$app->redirect('index.php?option=com_mailster');
		}

        function removeNoPlgExecFile(){
            $app = JFactory::getApplication();
            $log = MstFactory::getLogger();
            jimport('joomla.filesystem.file'); // For using JFile

            $basePath = JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster';
            $noPlgExecFileV1 = $basePath.DS.MstConsts::PLUGIN_FLAG_NO_EXECUTION;
            $noPlgExecFileV2 = $basePath.DS.MstConsts::PLUGIN_FLAG_NO_EXECUTION_V2;

            $deletedFiles = 0;
            $log->debug('removeNoPlgExecFile check for: '.$noPlgExecFileV1);
            if(file_exists($noPlgExecFileV1)){
                $log->debug('removeNoPlgExecFile existing: '.$noPlgExecFileV1);
                if(JFile::delete($noPlgExecFileV1)){
                    $deletedFiles++;
                }
            }

            $log->debug('removeNoPlgExecFile check for: '.$noPlgExecFileV2);
            if(file_exists($noPlgExecFileV2)){
                if(JFile::delete($noPlgExecFileV2)){
                    $log->debug('removeNoPlgExecFile existing: '.$noPlgExecFileV2);
                    $deletedFiles++;
                }
            }

            if($deletedFiles > 0){
                $app->enqueueMessage(JText::_( 'COM_MAILSTER_DELETED_FILE' ));
            }else{
                $app->enqueueMessage(JText::_( 'COM_MAILSTER_FAILED_TO_DELETE_FILE' ), 'error');
            }
            $app->redirect('index.php?option=com_mailster');
        }
		
	}
?>
