<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	defined( '_JEXEC' ) or die( 'Restricted access' );

	jimport('joomla.application.component.controller');

	/**
	 * Mailster Component Servers Controller
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterControllerServers extends MailsterController
	{
		
		function __construct(){
			parent::__construct();			
		}

		function cancel()
		{
			// Check for request forgeries
			JRequest::checkToken() or die( 'Invalid Token' );
			
			$server = JTable::getInstance('mailster_servers', '');
            $server->bind(JRequest::get('post'));

			$this->setRedirect( 'index.php?option=com_mailster&view=servers' );
		}

		/**
		 * logic to create the new server screen
		 *
		 * @access public
		 * @return void
		 */
		function add( )
		{
			$this->setRedirect( 'index.php?option=com_mailster&view=server' );
		}
		
		function addInbox(){
            $this->setRedirect( 'index.php?option=com_mailster&view=server&layout_id='.MstConsts::SERVER_TYPE_MAIL_INBOX );
        }

        function addSmtp(){
            $this->setRedirect( 'index.php?option=com_mailster&view=server&layout_id='.MstConsts::SERVER_TYPE_SMTP );
        }

		/**
		 * logic to create the edit server screen
		 *
		 * @access public
		 * @return void
		 */
		function edit( )
		{
			JRequest::setVar( 'view', 'server' );
			JRequest::setVar( 'hidemainmenu', 1 );

			$model 	= $this->getModel('server');
			$task 	= JRequest::getVar('task');

			$user = JFactory::getUser();
			
			parent::display();
		}
		
		
		function apply()
		{
			$this->save();
		}

		/**
		 * logic to save a server
		 *
		 * @access public
		 * @return void
		 */
		function save()
		{
			JRequest::checkToken() or die( 'Invalid Token' );
			
			$task		= JRequest::getVar('task');

			$post = JRequest::get( 'post' );
			$post['datdescription'] = JRequest::getVar( 'datdescription', '', 'post','string', JREQUEST_ALLOWRAW );
			$post['datdescription']	= str_replace( '<br>', '<br />', $post['datdescription'] );

            /** @var MailsterModelServer $model */
			$model = $this->getModel('server');
			if ($returnid = $model->store($post)) {

				switch ($task)
				{
					case 'apply' :
						$link = 'index.php?option=com_mailster&controller=servers&task=edit&cid[]='.$returnid;
						break;

					default :
						$link = 'index.php?option=com_mailster&view=servers';
						break;
				}
				$msg	= JText::_( 'COM_MAILSTER_SERVER_SAVED');
				$cache = JFactory::getCache('com_mailster');
				$cache->clean();

			} else {
				$msg 	= '';
				$link = 'index.php?option=com_mailster&view=servers';
			}

			$this->setRedirect( $link, $msg );
		}

        function unpublish(){
            $this->publish();
        }

        function publish(){
            $log = MstFactory::getLogger();
            // Get items to publish from the request.
            $cid = JRequest::getVar('cid', array(), '', 'array');
            $data = array('publish' => 1, 'unpublish' => 0);
            $task = JRequest::getString('task');
            $value = JArrayHelper::getValue($data, $task, 0, 'int');

            if (empty($cid))
            {
                JError::raiseWarning(500, JText::_('COM_MAILSTER_SELECT_AN_ITEM_TO_PUBLISH'));
            }
            else
            {
                /** @var MailsterModelServers $model */
                $model = $this->getModel('servers');

                // Make sure the item ids are integers
                JArrayHelper::toInteger($cid);

                $log->debug('publish to state: '.$value.' for IDs: '.print_r($cid, true));
                // Publish the items.
                if (!$model->publish($cid, $value))
                {
                    JError::raiseWarning(500, $model->getError());
                }
                else
                {
                    if ($value == 1)
                    {
                        $ntext = $this->text_prefix . '_N_ITEMS_PUBLISHED';
                    }
                    elseif ($value == 0)
                    {
                        $ntext = $this->text_prefix . '_N_ITEMS_UNPUBLISHED';
                    }
                    //$this->setMessage(JText::plural($ntext, count($cid)));
                }
            }

            $cache = JFactory::getCache('com_mailster');
            $cache->clean();

            $this->setRedirect( 'index.php?option=com_mailster&view=servers');
        }


        /**
         * logic to remove servers
         *
         * @access public
         * @return void
         */
        function remove()
        {
            $log = MstFactory::getLogger();
            $cid = JRequest::getVar( 'cid', array(0), 'post', 'array' );
            JArrayHelper::toInteger($cid);

            $total = count( $cid );

            if (!is_array( $cid ) || $total < 1) {
                JError::raiseError(500, JText::_( 'COM_MAILSTER_SELECT_AN_ITEM_TO_DELETE' ) );
            }

            $log->debug('Deleting ' . $total . ' servers...');
            /** @var MailsterModelServers $model */
            $model = $this->getModel('servers');
            if(!$model->delete($cid)) {
                echo "<script> alert('".$model->getError()."'); window.history.go(-1); </script>\n";
            }

            $msg = $total.' '.JText::_( 'COM_MAILSTER_SERVER_DELETED');

            $cache = JFactory::getCache('com_mailster');
            $cache->clean();

            $this->setRedirect( 'index.php?option=com_mailster&view=servers', $msg );
        }
	}
?>
