<?php
/**
 * @package Joomla
 * @subpackage Mailster
 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
 * @license GNU/GPL, see license.txt
 * Mailster is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.
 *
 * Mailster is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mailster; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * or see http://www.gnu.org/licenses/.
 */

defined( '_JEXEC' ) or die( 'Restricted access' );

jimport('joomla.application.component.controller');

/**
 * Mailster Component Sync Controller
 *
 * @package Joomla
 * @subpackage Mailster
 */
class MailsterControllerSync extends MailsterController
{

	function __construct(){
		parent::__construct();
	}

	/**
	 * To be called by an Ajax call
	 */
	function doInitUserStateSync(){
		if(!$this->canDo->get('core.admin')){
		//	return MstFactory::getAuthorization()->noPermissionForAction('sync->doInitUserStateSync (core.admin)');
		}
		$log = MstFactory::getLogger();
		$app = JFactory::getApplication();
		$mstUtils = MstFactory::getUtils();
		$recips = MstFactory::getRecipients();
		$cbUtils = MstFactory::getCBUtils();
		$plgUtils = MstFactory::getPluginUtils();
		$cbPluginParams = $plgUtils->getCBPluginParameter();
		$stateField = $cbPluginParams->def('cb_status_sync_status_field', 'block');
		$syncAction = $cbPluginParams->def( 'cb_status_sync_action', 'subscribe_add' );
		$syncActionBothWays = $cbPluginParams->def( 'cb_status_sync_both_ways', 1 );
		$mailingListId = $cbPluginParams->def( 'cb_status_sync_mailing_list_field', 0 );
		$userGroupId = $cbPluginParams->def( 'cb_status_sync_user_group_field', 0 );
			
		$resultArray = array();
		$res = JText::_( 'COM_MAILSTER_DO_INIT_SYNC_CALLED' );
		$ajaxParams = JRequest::getString('mtrAjaxData');
		$ajaxParams = $mstUtils->jsonDecode($ajaxParams);
		$res = true;
		
		$res = ($res ? 'true' : 'false');
		
		$forceNoDoubleOptIn = $ajaxParams->{'forceNoDoubleOptIn'};
		$forceNoWelcomeGoodbyeMsg = $ajaxParams->{'forceNoWelcomeGoodbyeMsg'};
		$doSyncActionInReverse = $ajaxParams->{'doSyncActionInReverse'};
	
		$initLog = array();
		$initLog[] = JText::_( 'COM_MAILSTER_SYNCHRONIZATION_STARTED' );
	
		if($stateField){
			$initLog[] = JText::sprintf( 'COM_MAILSTER_USE_USER_STATE_X_FOR_SYNCHRONIZATION', $stateField );
			$initLog = array_merge($initLog, $cbUtils->syncCBUserStatesWithStateChangeActionForAllUsers($stateField, $syncAction, $mailingListId, $userGroupId, $forceNoDoubleOptIn, $forceNoWelcomeGoodbyeMsg, $doSyncActionInReverse));

			$recips->recipientsUpdatedInAllLists();
			$initLog[] = JText::_( 'COM_MAILSTER_RECIPIENTS_CACHE_UPDATED' );
			$initLog[] = JText::_( 'COM_MAILSTER_SYNCHRONIZATION_COMPLETED' );
		}else{
			$initLog[] = JText::_( 'COM_MAILSTER_NO_CB_USER_STATE_CONFIGURED' );
		}	
	
		$initLogText = '';
		for($i=0;$i<count($initLog);$i++){
			$initLogText .= ($initLog[$i]."\r\n");
		}
		$initLogText = JText::_( 'COM_MAILSTER_INITIALISATION_LOG' )."\r\n\r\n".$initLogText;
	
		$log->debug('doInitSync result: '.$initLogText);
	
		$resultArray['res'] = $res;
		$resultArray['initLog'] = $initLog;
		$resultArray['initLogText'] = $initLogText;
		$jsonStr  = $mstUtils->jsonEncode($resultArray);
		echo "[" . $jsonStr . "]";
		$app->close(); // Close the application.
	}
	
	
	/**
	* To be called by an Ajax call
	*/
	function doInitMstGroupSync(){
		if(!$this->canDo->get('core.admin')){
		//	return MstFactory::getAuthorization()->noPermissionForAction('sync->doInitMstGroupSync (core.admin)');
		}
		$log = MstFactory::getLogger();
		$app = JFactory::getApplication();
		$mstUtils = MstFactory::getUtils();
		$recips = MstFactory::getRecipients();
		$cbUtils = MstFactory::getCBUtils();
		$plgUtils = MstFactory::getPluginUtils();
		$cbPluginParams = $plgUtils->getCBPluginParameter();
		$fieldId = $cbPluginParams->def('cb_user_group_field', 0);
		if($fieldId){
			$fieldObj = $cbUtils->getCBField($fieldId);
			$fieldName = $fieldObj->name;
		}else{
			$fieldName = false;
		}
		
		$resultArray = array();
		$res = JText::_( 'COM_MAILSTER_DO_INIT_SYNC_CALLED' );
		$ajaxParams = JRequest::getString('mtrAjaxData');
		$ajaxParams = $mstUtils->jsonDecode($ajaxParams);
		$res = true;
		
		$syncDirection = $ajaxParams->{'syncDirection'};
		
		$res = ($res ? 'true' : 'false');
		
		$initLog = array();
		$initLog[] = JText::_( 'COM_MAILSTER_SYNCHRONIZATION_STARTED' );
		
		if($fieldName){
			$initLog[] = JText::sprintf( 'COM_MAILSTER_USE_FIELD_X', $fieldName );
			if($syncDirection === 'cb2mailster'){
				$initLog[] = JText::_( 'COM_MAILSTER_SYNCHRONIZATION_DIRECTION' ).': '.JText::_( 'COM_MAILSTER_FROM_CB_FIELD_TO_MAILSTER_USER_GROUP' );
				$initLog = array_merge($initLog, $cbUtils->syncCBUserGroupsToMailsterGroupsForAllUsers($fieldName));
			}else{
				$initLog[] = JText::_( 'COM_MAILSTER_SYNCHRONIZATION_DIRECTION' ).': '.JText::_( 'COM_MAILSTER_FROM_MAILSTER_USER_GROUP_TO_CB_FIELD' );
				$initLog = array_merge($initLog, $cbUtils->syncMailsterGroupsToCBUserGroupsForAllUsers($fieldName));
			}

			$recips->recipientsUpdatedInAllLists();
			$initLog[] = JText::_( 'COM_MAILSTER_RECIPIENTS_CACHE_UPDATED' );
			$initLog[] = JText::_( 'COM_MAILSTER_SYNCHRONIZATION_COMPLETED' );
		}else{
			$initLog[] = JText::_( 'COM_MAILSTER_NO_CB_FIELD_CONFIGURED' );
		}
		
		
		$initLogText = '';
		for($i=0;$i<count($initLog);$i++){
			$initLogText .= ($initLog[$i]."\r\n");
		}
		$initLogText = JText::_( 'COM_MAILSTER_INITIALISATION_LOG' )."\r\n\r\n".$initLogText;
		
		$log->debug('doInitSync result: '.$initLogText);
		
		$resultArray['res'] = $res;
		$resultArray['initLog'] = $initLog;
		$resultArray['initLogText'] = $initLogText;
		$jsonStr  = $mstUtils->jsonEncode($resultArray);
		echo "[" . $jsonStr . "]";
		$app->close(); // Close the application.
	}

}
?>