<?php

	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	defined( '_JEXEC' ) or die( 'Restricted access' );
	
	class MstApplication
	{

		/**
		 * This method basically works like JComponentHelper::getComponent but without the caching
		 */
		public static function getComponent($option = 'com_mailster'){
			$comp = null;

            $db = JFactory::getDbo();
            $query = $db->getQuery(true);
            $query->select('extension_id AS "id", element AS "option", params, enabled');
            $query->from('#__extensions');
            $query->where('`type` = '.$db->quote('component'));
            $query->where('`element` = '.$db->quote($option));
            $db->setQuery($query);

            $error = false;
            $errorMsg = '';

            try {
                $comp = $db->loadObject();
            } catch (DatabaseException $e) {
                $error = true;
                $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
            }

            if ($error || !$comp){
                JError::raiseWarning(500, JText::sprintf('JLIB_APPLICATION_ERROR_COMPONENT_NOT_LOADING', $option, $errorMsg));
                return false;
            }

            // Convert the params to an object.
            if (is_string($comp->params)){
                $temp = new JRegistry;
                $temp->loadString($comp->params);
                $comp->params = $temp;
            }
						
			return $comp;
		}
				
		public static function detectSystemProblems(){
			$res = new stdClass();
			$res->error = false;
			$res->warningOnly = false;
			$res->errorMsg = "";
			$res->autoFixLinkName = false;
			$res->autoFixAvailable = false;
			$res->hideSysDiagnosisLink = false;
			
			$log		= MstFactory::getLogger();
			$env 		= MstFactory::getEnvironment();
			$dbUtils 	= MstFactory::getDBUtils();
			$mstConf	= MstFactory::getConfig();
			$plgUtils 	= MstFactory::getPluginUtils();
			$fileUtils	= MstFactory::getFileUtils();
			
			$jVersionOk		= version_compare(JVERSION,'2.5.0','ge'); // at least Joomla 2.5.0
			$phpVersionOk 	= (floatval(phpversion()) >= 5.0); // at least PHP 5.0
			$dbCollationOk 	= $dbUtils->userTableCollationOk();
			$imapExtOk 		= $env->imapExtensionInstalled();			
			$logFileTooBig	= $log->isLogFile2Big($mstConf->getLogFileSizeWarningLevel()*1024*1024);
			$logDBTooBig	= $log->isLogDatabase2Big($mstConf->getLogDatabaseWarningEntries());
            $noPlgExecFile  = $plgUtils->isNoExecutionFileExisting();
			$mailPlgInstall	= $plgUtils->isMailPluginInstalled();
			$mailPlgActive	= $plgUtils->isMailPluginActive();
			$parseIniFile	= function_exists('parse_ini_file');

			$gCalVersionOk = false;
			$gCalInstalled 	= $env->isGCalendarInstalled();
			if($gCalInstalled){
				$gCalVersionOk	= version_compare($env->getGCalendarVersion(),'2.6.4','ge'); // GCalendar 2.6.4/J2.5 fixes the jQuery issue, lower versions are no good
			}
			
			if(!$jVersionOk){
				$res->error = true;
				$res->errorMsg = JText::_( 'COM_MAILSTER_JOOMLA_VERSION_NOT_SUFFICIENT_CHECK_MINIMUM_SYSTEM_REQUIREMENTS' );
			}elseif(!$phpVersionOk){
				$res->error = true;
				$res->errorMsg = JText::_( 'COM_MAILSTER_PHP_VERSION_NOT_SUFFICIENT_CHECK_MINIMUM_SYSTEM_REQUIREMENTS' );
			}elseif(!$dbCollationOk){
                // now being pro-active and fix this
                if(method_exists($dbUtils, 'fixDBCollation')){
                    $dbUtils->fixDBCollation();
                }else{
                    $res->error = true;
                    $res->errorMsg = JText::_( 'COM_MAILSTER_DATABASE_COLLATION_ERROR' );
                    $res->autoFixAvailable = true;
                    $res->autoFixLink = 'index.php?option=com_mailster&controller=maintenance&task=fixDBCollation';
                }
			}elseif(!$imapExtOk){
				$res->error = true;
				$res->errorMsg = JText::_( 'COM_MAILSTER_IMAP_EXTENSION_NOT_INSTALLED_PLEASE_CHECK_SYSTEM_REQUIREMENTS' );
			}elseif($gCalInstalled && !$gCalVersionOk){
				$res->error = true;
				$res->errorMsg = JText::_( 'COM_MAILSTER_GCALENDAR_IS_INCOMPATIBLE_WITH_MAILSTER_IN_THIS_VERSION' );
				$res->hideSysDiagnosisLink = true;
			}elseif(!$parseIniFile){
				$res->error = true;
				$res->errorMsg = JText::_( 'COM_MAILSTER_PARSE_INI_FILE_PHP_FUNCTION_DISABLED_PLEASE_CHECK_SYSTEM_REQUIREMENTS' );
			}elseif($logFileTooBig){
				$res->error = true;
				$res->warningOnly = true;
				$res->errorMsg = JText::_( 'COM_MAILSTER_LOG_FILE_IS_VERY_LARGE' ) . ' (' . $fileUtils->getFileSizeOfFile($log->getLogFile()) . ')';
				$res->autoFixAvailable = true;
				$res->autoFixLink = 'index.php?option=com_mailster&controller=maintenance&task=deleteLogFile';
				$res->autoFixLinkName = JText::_( 'COM_MAILSTER_DELETE_LOG_FILE' );
			}elseif($logDBTooBig){				
				$res->error = true;
				$res->warningOnly = true;
				$res->errorMsg = JText::_( 'COM_MAILSTER_LOG_DATABASE_HAS_A_LOT_OF_ENTRIES' ) . ' ('.$log->getLogDatabaseEntriesCount().')';
				$res->autoFixAvailable = true;
				$res->autoFixLink = 'index.php?option=com_mailster&controller=maintenance&task=purgeLogDatabase';
				$res->autoFixLinkName = JText::_( 'COM_MAILSTER_PURGE_LOG_DATABASE' );
			}elseif($noPlgExecFile){
                $res->error = true;
                $res->warningOnly = true;
                $res->errorMsg = JText::_( 'COM_MAILSTER_NO_PLUGIN_EXECUTION_FILE_IS_PRESENT_IN_MAILSTER_FOLDER_REMOVE_IT_TO_START_EMAIL_PROCESSING_AGAIN' );
                $res->autoFixAvailable = true;
                $res->autoFixLink = 'index.php?option=com_mailster&controller=maintenance&task=removeNoPlgExecFile';
                $res->autoFixLinkName = JText::_( 'COM_MAILSTER_REMOVE_NO_PLUGIN_EXECUTION_FILE' );
            }elseif(!$mailPlgInstall){
				$res->error = true;
				$res->errorMsg = JText::_( 'COM_MAILSTER_MAIL_PLUGIN_NOT_INSTALLED_PLEASE_REINSTALL_MAILSTER' );
			}elseif(!$mailPlgActive){	
				$res->error = true;
				$res->warningOnly = true;
				$res->errorMsg = JText::_( 'COM_MAILSTER_MAIL_PLUGIN_NOT_ACTIVE_PLEASE_ACTIVATE_MAIL_PLUGIN_IN_ORDER_TO_RETRIEVE_AND_SEND_EMAILS' );
			}
			
			return $res;
		}
		
		public static function performMaintenance($minDuration=0, $execEnd=0){
			$log 		= MstFactory::getLogger();
            $config = MstFactory::getConfig();
			$sendEvents = MstFactory::getSendEvents();
			$subscrUtils = MstFactory::getSubscribeUtils();
            /** @var MailsterModelMails $mailsModel */
            $mailsModel = MstFactory::getModel('mails');
            $mailQueue = MstFactory::getMailQueue();
			$listUtils	= MstFactory::getMailingListUtils();
            $lists = $listUtils->getAllMailingLists();
            $keepBlockedEmailsForDays = $config->getKeepBlockedEmailsForDays();
            $keepBouncedEmailsForDays = $config->getKeepBouncedEmailsForDays();

			$log->info('Performing maintenance (cleanup)...');
            $log->debug('Time left to run: ' . ($execEnd - time()) . ' for performing maintenance (execEnd: ' . $execEnd . ', minDuration: ' . $minDuration . ')');

            $log->debug('Maintenance Task #1: Delete old blocked (older '.$keepBlockedEmailsForDays.' days) and bounced (older '.$keepBouncedEmailsForDays.' days) mails');
            $oldBlockedMails = $mailsModel->getEmailsOlderThanDays($keepBlockedEmailsForDays, 0, true, false, true, 100); // return max. 100 blocked emails' IDs
            $oldBouncedMails = $mailsModel->getEmailsOlderThanDays($keepBouncedEmailsForDays, 0, false, true, true, 100); // return max. 100 bounced emails' IDs
            $oldBlockedMails = $objects = array_map(function($o) { return $o->id; }, $oldBlockedMails);
            $oldBouncedMails = $objects = array_map(function($o) { return $o->id; }, $oldBouncedMails);
            $log->debug('Found '.count($oldBlockedMails).' blocked mails with IDs: '.implode(',', $oldBlockedMails));
            $log->debug('Found '.count($oldBouncedMails).' bounced mails with IDs: '.implode(',', $oldBouncedMails));
            $oldBlockedAndBouncedMailIds = array_merge($oldBlockedMails, $oldBouncedMails);
            $oldBlockedAndBouncedMailIds = array_unique($oldBlockedAndBouncedMailIds);
            $oldBlockedAndBouncedMailIds = array_merge($oldBlockedAndBouncedMailIds);
            $log->debug('Combined set and will now delete '.count($oldBlockedAndBouncedMailIds).' blocked/bounced mails with IDs: '.implode(',', $oldBlockedAndBouncedMailIds));
            $mailsModel->delete($oldBlockedAndBouncedMailIds);

            if(($execEnd > 0) && (($execEnd - time()) <= $minDuration)){
                $log->debug('Timeout after deleting blocked and bounced emails, skip further maintenance tasks');
                return;
            }else{
                $log->debug('Time left to run: ' . ($execEnd - time()) . ' for performing maintenance (execEnd: ' . $execEnd . ', minDuration: ' . $minDuration . ')');
            }

            $log->debug('Maintenance Task #2: Delete old send reports');
			foreach($lists AS $list){
				$log->debug('Searching mails with old send reports of list '.$list->name . ' (ID '.$list->id.')...');
				$mails = $sendEvents->getMailsWithSendReportOlderThan($list->id, $list->save_send_reports);

                if(($execEnd > 0) && (($execEnd - time()) <= $minDuration)){
                    $log->debug('Timeout, do not work on old send reports');
                    break;
                }

				$log->debug('Found ' . count($mails) . ' mails with a send report to delete (age older than '.$list->save_send_reports.' days)');
				if($mails){
					foreach($mails AS $mail){
						$log->debug('Deleting send report of mail '.$mail->mail_id);
						$sendEvents->deleteSendReportOfMail($mail->mail_id);
						$sendEvents->setHasSendReportFlag($mail->mail_id, false);
					}
				}
			}

            if(($execEnd > 0) && (($execEnd - time()) <= $minDuration)){
                $log->debug('Timeout after/while deleting send reports, skip further maintenance tasks');
                return;
            }else{
                $log->debug('Time left to run: ' . ($execEnd - time()) . ' for performing maintenance (execEnd: ' . $execEnd . ', minDuration: ' . $minDuration . ')');
            }
			
			$log->debug('Maintenance Task #3: Delete old subscription info');
			$subInfos = $subscrUtils->getSubscriptionInfoOlderThan(MstConsts::SUBSCRIPTION_INFO_DEFAULT_STORE_DURATION);
			$log->debug('Found ' . count($subInfos) . ' subscription info to delete');
			if($subInfos){
				foreach($subInfos AS $subInfo){
                    if(($execEnd > 0) && (($execEnd - time()) <= $minDuration)){
                        $log->debug('Timeout, do not work on old subscription info');
                        break;
                    }
					$log->debug('Deleting subscription info: '.$subInfo->id . ' (created: '.$subInfo->sub_date.' -> older than '.MstConsts::SUBSCRIPTION_INFO_DEFAULT_STORE_DURATION.' days)');
					$subscrUtils->deleteSubscriptionInfo($subInfo->id);
				}
			}

            if(($execEnd > 0) && (($execEnd - time()) <= $minDuration)){
                $log->debug('Timeout after/while deleting old subscription info, skip further maintenance tasks');
                return;
            }else{
                $log->debug('Time left to run: ' . ($execEnd - time()) . ' for performing maintenance (execEnd: ' . $execEnd . ', minDuration: ' . $minDuration . ')');
            }

            $log->debug('Maintenance Task #4: Do offline archiving');
            foreach($lists AS $list){
                if($list->archive_offline > 0){
                    if(($execEnd > 0) && (($execEnd - time()) <= $minDuration)){
                        $log->debug('Timeout, do not work on offline archiving');
                        break;
                    }
                    $log->debug('Offline archiving enabled for list '.$list->name . ' (ID '.$list->id.'), search for live emails older than '.$list->archive_offline.' days...');

                    $mails = $mailsModel->getEmailsOlderThanDays($list->archive_offline, $list->id);
                    $log->debug('Found ' . count($mails) . ' mails older than '.$list->archive_offline.' days');
                    if($mails){
                        $mailIds = array();
                        $log->debug('Deleting send queue entries for '.count($mails) . ' mails');
                        foreach($mails AS $mail){
                            $mailQueue->removeAllRecipientsOfMailFromQueue($mail->id);
                            $mailIds[] = $mail->id;
                        }
                        $mailsModel->moveEmailsToOfflineArchive($mailIds);
                    }
                }else{
                    $log->debug('Offline archiving disabled for list '.$list->name . ' (ID '.$list->id.')...');
                }
            }

            if(($execEnd > 0) && (($execEnd - time()) <= $minDuration)){
                $log->debug('Timeout after/while doing offline archiving, skip further maintenance tasks');
                return;
            }else{
                $log->debug('Time left to run: ' . ($execEnd - time()) . ' for performing maintenance (execEnd: ' . $execEnd . ', minDuration: ' . $minDuration . ')');
                $log->debug('Completed maintenance tasks');
            }

		}
		
		public static function getInstallInformation(){
			$env = MstFactory::getEnvironment();
			$path = JPATH_ADMINISTRATOR .DS. 'components'.DS.'com_mailster'.DS;
            $xmlFile = $path.'mailster.xml';
            if(!file_exists($xmlFile)){
                $xmlFile = $path.'_manifest.xml'; // fall back to old style XML file
            }
			return $env->getInstallInformationOfComponent($xmlFile);
		}

		public static function getTriggerSourceName($triggerSrc){
			switch($triggerSrc){
				case MstConsts::TRIGGER_SOURCE_PLUGIN_BACKEND:
					return JText::_( 'TRIGGER_SOURCE_PLUGIN_BACKEND' );
					break;
				case MstConsts::TRIGGER_SOURCE_PLUGIN_FRONTEND:
					return JText::_( 'TRIGGER_SOURCE_PLUGIN_FRONTEND' );
					break;
				case MstConsts::TRIGGER_SOURCE_CRONJOB_TASK_FETCH_ALL_LISTS:
					return JText::_( 'TRIGGER_SOURCE_CRONJOB_TASK_FETCH_ALL_LISTS' );
					break;
				case MstConsts::TRIGGER_SOURCE_CRONJOB_TASK_FETCH_LIST:
					return JText::_( 'TRIGGER_SOURCE_CRONJOB_TASK_FETCH_LIST' );
					break;
				case MstConsts::TRIGGER_SOURCE_CRONJOB_TASK_SEND:
					return JText::_( 'TRIGGER_SOURCE_CRONJOB_TASK_SEND' );
					break;
				case MstConsts::TRIGGER_SOURCE_CRONJOB_TASK_ALL:
					return JText::_( 'TRIGGER_SOURCE_CRONJOB_TASK_ALL' );
					break;
					
			}
		}
	}

?>
