<?php

	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	defined( '_JEXEC' ) or die( 'Restricted access' );
	
	class MstAuthorization
	{


		/**
		 *  Get the (ACL) actions         */
		public static function getActions($assetId=null) {
            jimport('joomla.access.access');
            $user   = JFactory::getUser();
            $result = new JObject;
            if(is_null($assetId)){
                $assetName = 'com_mailster';
            }else{
                $assetName = 'com_mailster.list.'.(int) $assetId;
            }
            $actions = JAccess::getActions('com_mailster', 'component');
            foreach ($actions as $action) {
                $result->set($action->name, $user->authorise($action->name, $assetName));
            }
			return $result;
		}
		
		public static function isAllowed($action, $listId=null){
			$log = MstFactory::getLogger();
            jimport('joomla.access.access');
            $user   = JFactory::getUser();
            $result = new JObject;
            if(is_null($listId)){
                $assetName = 'com_mailster';
            }else{
                $assetName = 'com_mailster.list.'.(int) $listId;
            }
            //$log->debug('isAllowed for action: '.$action.', assetName: '.$assetName);
            $result = $user->authorise($action, $assetName);
            if(is_null($result) && !is_null($listId)){
                $result = $user->authorise($action, 'com_mailster');
                $log->debug('isAllowed - 1st result was null, recheck result: is_null: '.is_null($result)?'yes':'no'.', boolean: '.$result?'true':'false');
            }
			return $result;
		}
		
		public static function noPermissionForAction($action){
			$log = MstFactory::getLogger();
			$log->error(JText::sprintf('COM_MAILSTER_NO_PERMISSION_TO_DO_ACTION_X', $action));
			return JError::raiseWarning(403, JText::sprintf('COM_MAILSTER_NO_PERMISSION_TO_DO_ACTION_X', $action));
		}
		
		public static function frontendMenuViewAccessOK(){
			$user = JFactory::getUser();
			$menu = JSite::getMenu();
			$mailingListUtils = MstFactory::getMailingListUtils();		
			$menuParams = $menu->getParams( JRequest::getInt( 'Itemid' ) );			
			$listId 			= $menuParams->get('list_choice');
		//	$authorizedUsers 	= $menuParams->get('authorized_users');
		
			if($listId > 0){
				// specific mailing list
				$mList = $mailingListUtils->getMailingList($listId);
				if($mList->front_archive_access == MstConsts::FRONT_ARCHIVE_ACCESS_ALL_USERS){
					return true; // everyone can access
				}elseif($mList->front_archive_access == MstConsts::FRONT_ARCHIVE_ACCESS_LOGGED_IN_USERS){
					if($user->id > 0){
						return true; // user is logged in, everything good
					}else{
						JFactory::getApplication()->enqueueMessage(JText::_('COM_MAILSTER_YOU_NEED_TO_LOGIN_TO_ACCESS_THIS_SECTION'), 'error');
						return false;
					}
				}elseif($mList->front_archive_access == MstConsts::FRONT_ARCHIVE_ACCESS_LOGGED_IN_SUBSCRIBERS_OF_MAILING_LIST){
					// specific list -> we can directly check whether user is subscriber...
					$recip = MstFactory::getRecipients();
					$subscriberLists = $recip->getListsUserIsMemberOf($user->id, 1);
					foreach($subscriberLists AS $subscrListId){
						if($subscrListId == $listId){
							return true; // user is subscriber, everything good
						}
					}
					JFactory::getApplication()->enqueueMessage(JText::_('COM_MAILSTER_YOU_NEED_TO_BE_A_SUBSCRIBER_OF_THIS_MAILING_LIST_TO_ACCESS_THIS_SECTION'), 'error');
					return false;
                }elseif($mList->front_archive_access == MstConsts::FRONT_ARCHIVE_ACCESS_NOBODY){
                    return false; // nobody is allowed to see this lists mails in the frontend
                }
			}
			
			/*
			if($authorizedUsers === 'registered' || $authorizedUsers === 'subscribers'){
				if($user->id > 0){
					if($authorizedUsers === 'subscribers' && $listId > 0){
						// specific list -> we can directly check whether user is subscriber...
						$recip = MstFactory::getRecipients();
						$subscriberLists = $recip->getListsUserIsMemberOf($user->id, 1);
						foreach($subscriberLists AS $subscrListId){
							if($subscrListId == $listId){
								return true; // user is subscriber, everything good
							}
						}
						JFactory::getApplication()->enqueueMessage(JText::_('COM_MAILSTER_YOU_NEED_TO_BE_A_SUBSCRIBER_OF_THIS_MAILING_LIST_TO_ACCESS_THIS_SECTION'), 'error');
						return false;
					}
					return true;
				}else{
					JFactory::getApplication()->enqueueMessage(JText::_('COM_MAILSTER_YOU_NEED_TO_LOGIN_TO_ACCESS_THIS_SECTION'), 'error');
					return false;
				}
			}
			*/
			return true;			
		}
		
		public static function userGroupAuthorizedInherited($gid){
			
			$user = JFactory::getUser();

			$parent = false;
			$ugid = $user->gid;
					
			if(($gid == 29) && ($ugid == 0)){
				return true;
			}
		
			if($gid == 29){
				if($ugid == 29){
					return true;
				}
				$parent = true;
			}
			if($gid == 18 || ($parent == true)){
				if($ugid == 18){
					return true;
				}
				$parent = true;
			}
			if($gid == 19 || ($parent == true)){
				if($ugid == 19){
					return true;
				}
				$parent = true;
			}
			if($gid == 20 || ($parent == true)){
				if($ugid == 20){
					return true;
				}
				$parent = true;
			}
			if($gid == 21 || ($parent == true)){
				if($ugid == 21){
					return true;
				}
				$parent = true;
			}
			if($gid == 30 || ($parent == true)){
				if($ugid == 30){
					return true;
				}
				$parent = true;
			}
			if($gid == 23 || ($parent == true)){
				if($ugid == 23){
					return true;
				}
				$parent = true;
			}
			if($gid == 24 || ($parent == true)){
				if($ugid == 24){
					return true;
				}
				$parent = true;
			}	
			if($gid == 25 || ($parent == true)){
				if($ugid == 25){
					return true;
				}
				$parent = true;
			}		
			
			return false;
		}
		
		public static function printUnauthorizedMsg(){
			echo '<span class="mailster_unauthorized" style="color:red;">'.JText::_( 'COM_MAILSTER_NOT_AUTHORIZED').'</span>';
		}
		
	}

?>
