<?php

	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	defined( '_JEXEC' ) or die( 'Restricted access' );
	
	class MstEvents
	{
	
		public static function sendError($mailId, $errorMsg){
			$log = MstFactory::getLogger();
			$log->debug('MstEvents::sendError');
			$dateUtils 	= MstFactory::getDateUtils();
			$mailUtils = MstFactory::getMailUtils();
			$mailingListUtils = MstFactory::getMailingListUtils();
			$mail = $mailUtils->getMail($mailId);
			$mList = $mailingListUtils->getMailingList($mail->list_id);
			$subject = JText::sprintf( 'COM_MAILSTER_SEND_ERROR_NOTIFICATION_SUBJECT_WITH_MAILING_LIST', $mList->name);
			$body = JText::sprintf('COM_MAILSTER_SEND_ERROR_NOTIFICATION_BODY_WITH_MAIL_SUBJECT_AND_ERROR_MESSAGE', $mail->subject, $errorMsg);
						
			$sendEvents = MstFactory::getSendEvents();
			$sendEventsLimit = 100; // limit to 100 send events
			$sendReport = $sendEvents->getSendEventsForMail($mailId, $sendEventsLimit); 
			if(count($sendReport)>0){
				$sendReportHtml = ('
				<table>
					<tr>
						<th width="130px">'.JText::_( 'COM_MAILSTER_DATE' ).'</th>
						<th width="20px">&nbsp;</th>
						<th width="200px">'. JText::_( 'COM_MAILSTER_EVENT' ) .'</th>
						<th width="650px">'. JText::_( 'COM_MAILSTER_DESCRIPTION' ) . '</th>
					</tr>');
				foreach($sendReport AS $event){
					$sendReportHtml .= ('<tr>
											<td width="130px">'. $dateUtils->formatDate($event->event_time). '</td>
											<td width="20px"><img src="'. JURI::root() . 'administrator/' . $event->imgPath.'" /></td>
											<td width="200px">'. $event->name . '</td>
											<td width="650px">'. $event->desc . '</td>
										</tr>');
				}
				if(count($sendReport) == $sendEventsLimit){
					$sendReportHtml .= ('<tr>
											<td width="130px"> </td>
											<td width="20px"> </td>
											<td width="200px"> </td>
											<td width="650px"> - snip - </td>
										</tr>');
				}
				$sendReportHtml .= '</table>';
			}else{
				$sendReportHtml = '';
			}

			$body = '<html><header></header><body>'.$body.$sendReportHtml.'</body></html>';
			
			$triggerType = MstEventTypes::SEND_ERROR;
			self::newMailingListEvent($mail->list_id, $triggerType, $subject, $body, true);
			$log->debug('MstEvents::sendError Sent notification');
		}
		
		public static function getSenderString($fromName, $fromEmail){
			$senderStr = (!is_null($fromName) && (strlen($fromName)>0)) ? ($fromName . ' ') : '';
			$senderStr .= '<'.$fromEmail.'>';
			return $senderStr;
		}
		
		public static function newMailingListMail($mailId){
			$log = MstFactory::getLogger();
			$log->debug('MstEvents::newMailingListMail');
			$mailUtils = MstFactory::getMailUtils();
			$mailingListUtils = MstFactory::getMailingListUtils();
			$mail = $mailUtils->getMail($mailId);
			$mList = $mailingListUtils->getMailingList($mail->list_id);
			$subject = JText::sprintf( 'COM_MAILSTER_NEW_MAILING_LIST_MAIL_NOTIFICATION_SUBJECT_WITH_MAILING_LIST', $mList->name);
			$body = JText::sprintf( 'COM_MAILSTER_NOTIFICATION_BODY_SENDER_ADDRESS_WITH_EMAIL', self::getSenderString($mail->from_name, $mail->from_email));
			$body .= "\r\n".JText::sprintf('COM_MAILSTER_NEW_MAILING_LIST_MAIL_NOTIFICATION_BODY_WITH_MAIL_SUBJECT', $mail->subject);
			$triggerType = MstEventTypes::NEW_LIST_MAIL;
			self::newMailingListEvent($mail->list_id, $triggerType, $subject, $body);
			$log->debug('MstEvents::newMailingListMail Sent notification');
		}
		
		public static function newBouncedMail($mailId){	
			$log = MstFactory::getLogger();
			$log->debug('MstEvents::newBouncedMail');
			$config = MstFactory::getConfig();
			$mailUtils = MstFactory::getMailUtils();
			$mailingListUtils = MstFactory::getMailingListUtils();
			$mail = $mailUtils->getMail($mailId);
			$mList = $mailingListUtils->getMailingList($mail->list_id);
			$subject = JText::sprintf( 'COM_MAILSTER_NEW_BOUNCED_MAIL_NOTIFICATION_SUBJECT_WITH_MAILING_LIST', $mList->name);
			$htmlMail = false;
			if($config->includeBodyInBouncedBlockedNotifications()){
				$log->debug('MstEvents::newBouncedMail Include text in notification');
				$body = JText::sprintf( 'COM_MAILSTER_NEW_BOUNCED_MAIL_NOTIFICATION_BODY_WITH_MAIL_SUBJECT', $mail->subject);
				if(!is_null($mail->html) && (strlen(trim($mail->html)) > 0)){
					$htmlMail = true;
					$body = htmlentities($body);
					$body .= '<br/><hr/><br/>';
					$body .= $mail->html;
					$body .= '<br/><hr/>';
				}else{
					$htmlMail = false;
					$body .= "\r\n\r\n";
					$body .= $mail->body;
				}				
			}else{
				$log->debug('MstEvents::newBouncedMail Do NOT include text in notification');
				$body = JText::sprintf( 'COM_MAILSTER_NEW_BOUNCED_MAIL_NOTIFICATION_BODY_WITH_MAIL_SUBJECT', $mail->subject);
			}
			$triggerType = MstEventTypes::NEW_BOUNCED_MAIL;
			self::newMailingListEvent($mail->list_id, $triggerType, $subject, $body, $htmlMail);
			$log->debug('MstEvents::newBouncedMail Sent notification');
		}
		
		public static function newBlockedMail($mailId){	
			$log = MstFactory::getLogger();
			$log->debug('MstEvents::newBlockedMail');
			$config = MstFactory::getConfig();
			$mailUtils = MstFactory::getMailUtils();
			$mailingListUtils = MstFactory::getMailingListUtils();
			$mail = $mailUtils->getMail($mailId);
			$mList = $mailingListUtils->getMailingList($mail->list_id);
			$subject = JText::sprintf( 'COM_MAILSTER_NEW_BLOCKED_MAIL_NOTIFICATION_SUBJECT_WITH_MAILING_LIST', $mList->name);
			$htmlMail = false;
			if($config->includeBodyInBouncedBlockedNotifications()){
				$log->debug('MstEvents::newBlockedMail Include text in notification');
				$body = JText::sprintf( 'COM_MAILSTER_NOTIFICATION_BODY_SENDER_ADDRESS_WITH_EMAIL', self::getSenderString($mail->from_name, $mail->from_email));
				if(!is_null($mail->html) && (strlen(trim($mail->html)) > 0)){
					$htmlMail = true;
					$body = htmlentities($body);
					$body .= '<br/>'.htmlentities(JText::sprintf( 'COM_MAILSTER_NEW_BLOCKED_MAIL_NOTIFICATION_BODY_WITH_MAIL_SUBJECT', $mail->subject));
					$body .= '<br/><hr/><br/>';
					$body .= $mail->html;
					$body .= '<br/><hr/>';
				}else{
					$htmlMail = false;
					$body .= "\r\n".JText::sprintf( 'COM_MAILSTER_NEW_BLOCKED_MAIL_NOTIFICATION_BODY_WITH_MAIL_SUBJECT', $mail->subject);
					$body .= "\r\n\r\n";
					$body .= $mail->body;
				}				
			}else{
				$log->debug('MstEvents::newBlockedMail Do NOT include text in notification');
				$body = JText::sprintf( 'COM_MAILSTER_NOTIFICATION_BODY_SENDER_ADDRESS_WITH_EMAIL', self::getSenderString($mail->from_name, $mail->from_email));
				$body .= "\r\n".JText::sprintf( 'COM_MAILSTER_NEW_BLOCKED_MAIL_NOTIFICATION_BODY_WITH_MAIL_SUBJECT', $mail->subject);
			}
			$triggerType = MstEventTypes::NEW_BLOCKED_MAIL;
			self::newMailingListEvent($mail->list_id, $triggerType, $subject, $body, $htmlMail);
			$log->debug('MstEvents::newBlockedMail Sent notification');
		}
		
		public static function newFilteredMail($mailId){
			$log = MstFactory::getLogger();
			$log->debug('MstEvents::newFilteredMail');
			$mailUtils = MstFactory::getMailUtils();
			$mailingListUtils = MstFactory::getMailingListUtils();
			$mail = $mailUtils->getMail($mailId);	
			$mList = $mailingListUtils->getMailingList($mail->list_id);
			$subject = JText::sprintf( 'COM_MAILSTER_NEW_FILTERED_MAIL_NOTIFICATION_SUBJECT_WITH_MAILING_LIST', $mList->name);
			$body = JText::sprintf( 'COM_MAILSTER_NOTIFICATION_BODY_SENDER_ADDRESS_WITH_EMAIL', self::getSenderString($mail->from_name, $mail->from_email));
			$body .= "\r\n".JText::sprintf( 'COM_MAILSTER_NEW_FILTERED_MAIL_NOTIFICATION_BODY_WITH_MAIL_SUBJECT', $mail->subject);
			$triggerType = MstEventTypes::NEW_FILTERED_MAIL;
			self::newMailingListEvent($mail->list_id, $triggerType, $subject, $body);
			$log->debug('MstEvents::newFilteredMail Sent notification');
		}
		
		public static function userSubscribedOnWebsite($name, $email, $listId){	
			$log = MstFactory::getLogger();
			$log->debug('MstEvents::userSubscribedOnWebsite');
			$mailingListUtils = MstFactory::getMailingListUtils();
			$mList = $mailingListUtils->getMailingList($listId);
			$subject = JText::sprintf( 'COM_MAILSTER_USER_SUBSCRIBED_ON_WEBSITE_NOTIFICATION_SUBJECT_WITH_MAILING_LIST', $mList->name);
			$body = JText::sprintf('COM_MAILSTER_USER_SUBSCRIBED_ON_WEBSITE_NOTIFICATION_BODY_WITH_LIST_NAME_AND_USER_NAME_AND_EMAIL', $name, $email, $mList->name);
			$triggerType = MstEventTypes::USER_SUBSCRIBED_ON_WEBSITE;
			self::newMailingListEvent($listId, $triggerType, $subject, $body);
			$log->debug('MstEvents::userSubscribedOnWebsite Sent notification');
		}
		
		public static function userUnsubscribedOnWebsite($email, $listId){
			$log = MstFactory::getLogger();
			$log->debug('MstEvents::userUnsubscribedOnWebsite');
			$mailingListUtils = MstFactory::getMailingListUtils();
			$mList = $mailingListUtils->getMailingList($listId);
			$subject = JText::sprintf( 'COM_MAILSTER_USER_UNSUBSCRIBED_ON_WEBSITE_NOTIFICATION_SUBJECT_WITH_MAILING_LIST', $mList->name);
			$body = JText::sprintf('COM_MAILSTER_USER_UNSUBSCRIBED_ON_WEBSITE_NOTIFICATION_BODY_WITH_LIST_NAME_AND_EMAIL', $email, $mList->name);
			$triggerType = MstEventTypes::USER_UNSUBSCRIBED_ON_WEBSITE;
			self::newMailingListEvent($listId, $triggerType, $subject, $body);
			$log->debug('MstEvents::userUnsubscribedOnWebsite Sent notification');
		}
		
		private static function newMailingListEvent($listId, $triggerType, $subject, $body, $htmlMail=false){
			$log = MstFactory::getLogger();
			$log->debug('MstEvents::newMailingListEvent');
			$mailUtils = MstFactory::getMailUtils();
			$notifyUtils = MstFactory::getNotifyUtils();
			$mailingListUtils = MstFactory::getMailingListUtils();

			if(MstFactory::getV()->getFtSetting(MstVersionMgmt::MST_FT_ID_NOTIFY)){
				$mailingListNotifies = $notifyUtils->getNotifiesOfMailingList($listId);
				$mList = $mailingListUtils->getMailingList($listId);
				$replyTo = array($mList->admin_mail, '');
				$log->debug('Mailing list notifications in place for list ID '.$listId.' '.print_r($mailingListNotifies, true));			
				for($i=0; $i<count($mailingListNotifies); $i++){
					$notify = $mailingListNotifies[$i];
					if($notifyUtils->notifyMatches($notify, MstNotify::NOTIFY_TYPE_LIST_BASED, $triggerType)){
						$log->debug('Notify will be sent out: ' . $notifyUtils->getTriggerTypeStr($triggerType));
						$mail = $notifyUtils->getNotifyMailTmpl($notify, $subject, $body, $replyTo, $listId, $htmlMail);
                        try {
                            $mail->Send();  // send notificaton
                        } catch (RuntimeException $e) {
                            $exceptionErrorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                            $log->error('Events::newMailingListEvent error: '.$exceptionErrorMsg);
                        }
						$error =  $mail->IsError();
						if($error == true) { // send errors?
							$log->error('Sending of notify failed! Last error: ' . $mail->ErrorInfo);
						}	
					}
				}
			}else{
				$log->warning('MstEvents::newMailingListEvent Did not send notification due to check procedure');
			}
		}
		
		public static function mailIsNotForwarded($listId, $subject, $senderName, $senderEmail, $senderBlocked=false, $emailFilteredByWords=false, $emailTooLarge=false){	
			$log = MstFactory::getLogger();
			$log->debug('MstEvents::mailIsNotForwarded');
			$mailUtils = MstFactory::getMailUtils();
			$notifyUtils = MstFactory::getNotifyUtils();
			$mailingListUtils = MstFactory::getMailingListUtils();
						
			$mList = $mailingListUtils->getMailingList($listId);
			
			if($mList->notify_not_fwd_sender == 1){
				$log->debug('Notification to sender of not fowarded email will be sent out');
				$replyTo = array($mList->admin_mail, '');
				
				if($senderBlocked){	
					$cause = JText::sprintf( 'COM_MAILSTER_SENDER_BLOCKED_AS_NOT_ALLOWED_TO_SEND_TO_LIST_WITH_SENDER_X', $senderEmail );
					
					if(strtolower(trim($senderEmail)) === strtolower(trim($mList->list_mail))){
						$log->info('Event "Sender blocked" will not be send out as blocked sender (and receiver of notification) is the mailing list address');
						return; // exit now
					}
					
				}elseif($emailFilteredByWords){
					$cause = JText::_( 'COM_MAILSTER_FILTERED_MAIL_BECAUSE_OF_CONTENT' );					
				}elseif($emailTooLarge){
					$maxEmailSize = $mList->mail_size_limit;
					$cause = JText::sprintf( 'COM_MAILSTER_EMAIL_SIZE_LIMIT_OF_X_KB_WAS_EXCEEDED', $maxEmailSize );					
				}else{ // Unknown cause?! Whatever, notify sender anyway 					
					$cause = JText::_( 'COM_MAILSTER_MAIL_NOT_FORWARDED' );
				}
				
				$log->debug('Cause for not forwarded email: ' . $cause);
				
				$notificationSubject = JText::sprintf( 'COM_MAILSTER_MAIL_NOT_FORWARDED_NOTIFICATION_FOR_SENDER_SUBJECT_WITH_MAILING_LIST', $mList->name);
				$notificationBody = JText::sprintf('COM_MAILSTER_MAIL_NOT_FORWARDED_NOTIFICATION_FOR_SENDER_BODY_WITH_MAIL_SUBJECT_AND_CAUSE', $subject, $cause);

				$mail = $notifyUtils->getSenderNotifyMailTmpl($senderName, $senderEmail, $notificationSubject, $notificationBody, $replyTo, $listId);
				$log->debug('Prepared notify mail: ' . print_r($mail, true));
                try {
                    $mail->Send();  // send notificaton
                } catch (RuntimeException $e) {
                    $exceptionErrorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                    $log->error('Events::newMailingListEvent error: '.$exceptionErrorMsg);
                }
				$error =  $mail->IsError();
				if($error == true) { // send errors?
					$log->error('Sending of notify failed! Last error: ' . $mail->ErrorInfo);
				}else{
					$log->debug('Sender was successfully notified');
				}
			}
		}		
		
	}

?>
