<?php

	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	defined( '_JEXEC' ) or die( 'Restricted access' );
	
	class MstConfiguration
	{

		protected static $mainConfigFile = 'config.xml';

		protected static function _getConfigFilePath(){
			return JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster'.DS.self::$mainConfigFile;	
		}
		
		public static function getProperty($property, $default){
			$propPath = self::_getConfigFilePath();
			$confIO = MstFactory::getConfIO();
			return $confIO->getProperty($propPath, $property, $default);
		}	
		
		public static function getComponentParams($option='com_mailster'){
			$mstApp = MstFactory::getApplication();
			$comp = JComponentHelper::getComponent($option, true); // strict mode to get false when not enabled/existing
			if($comp->enabled == true){
				$comp = $mstApp->getComponent($option);
				return $comp->params;
			}
			return false;
		}
		
		public static function updateComponentParams($params, $option='com_mailster'){
			$comp = JComponentHelper::getComponent($option, true); // strict mode to get false when not enabled/existing
			if($comp){
				$log = MstFactory::getLogger();
				$db = JFactory::getDBO();
				$query = 'UPDATE #__extensions SET params = ' . $db->quote($params) . ' WHERE element=\'' . $option . '\'';
				$db->setQuery( $query );
				
				$result = false;
				$errorMsg = '';
                try {
                    $result = $db->query(); // update cache version/state
                } catch (DatabaseException $e) {
                    $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                }
				if(!$result){
					$log->error('Updating plugin parameters of plugin ' . $option . ' failed: ' . $errorMsg. ', query was: '.$query);
					return false;
				}
				return true;
			}
		}
		
		public static function getComponentExtensionTblEntry($ext = 'com_mailster'){
            $tbl = 'extension';
            $table = JTable::getInstance($tbl);
            $optArray = array();
            $optArray['element'] = $ext;
            $loadOk = $table->load($table->find($optArray));
			if(!$loadOk){
			    JError::raiseWarning(500, 'Not a valid component');
			    return false;
			}
			return $table;
		}
		
		public static function getAllParameters(){		
			$table = self::getComponentExtensionTblEntry();
            jimport('joomla.form.form');
            // Joomla! 1.6 and 1.7

            //$registry = new JRegistry;
            //$registry->loadJSON($table->params);
            //$params = $registry;
            //$params = JForm::getInstance('com_mailster.config',  $registry);

            // TODO MAKE THIS BETTER TO GET RID OF JPARAMETER AND FOR CLEAN RENDERING
            if(version_compare(JVERSION,'3.0','ge')) {
                $params = new JRegistry($table->params, self::_getConfigFilePath());
            }else{
                $params = new JParameter($table->params, self::_getConfigFilePath());
            }
			return $params;
		}

		public static function showUserDescription(){
			$params = self::getComponentParams();
			if($params){
		    	$showDesc = $params->get('show_user_description'); 
		    	if($showDesc > 0){ 
			    	return true;
		    	}
	            return false;
			}else{
				return false;
			}
		}

        public static function getKeepBlockedEmailsForDays(){
            $params = self::getComponentParams();
            if($params){
                return $params->get('keep_blocked_mails_for_days', 30);
            }else{
                return 30;
            }
        }
        public static function getKeepBouncedEmailsForDays(){
            $params = self::getComponentParams();
            if($params){
                return $params->get('keep_bounced_mails_for_days', 30);
            }else{
                return 30;
            }
        }
		
		public static function getLoggingLevel(){		
			$params = self::getComponentParams();
			if($params){
	    		return $params->get('logging_level', MstConsts::LOG_LEVEL_INFO); 
			}else{
				return MstConsts::LOG_LEVEL_INFO;
			}
		}
		
		public static function getLogFileSizeWarningLevel(){		
			$params = self::getComponentParams();
			if($params){
	    		return $params->get('log_file_warning_size_mb', 25); 
			}else{
				return 25;
			}
		}
		
		public static function getLogDatabaseWarningEntries(){		
			$params = self::getComponentParams();
			if($params){
	    		return $params->get('log_db_warning_entries_nr', 1000); 
			}else{
				return 1000;
			}
		}
		
		public static function isLog2File(){		
			$params = self::getComponentParams();
			if($params){
				$logDest = $params->get('log_entry_destination');
		    	if($logDest == MstConsts::LOG_DEST_FILE || $logDest == MstConsts::LOG_DEST_DB_AND_FILE){
		    		return true;
		    	}else{
		    		return false;
		    	}
			}else{
				return true;
			}
		}
		
		public static function isLog2Database(){		
			$params = self::getComponentParams();
			if($params){
				$logDest = $params->get('log_entry_destination');
		    	if($logDest == MstConsts::LOG_DEST_DB || $logDest == MstConsts::LOG_DEST_DB_AND_FILE){
		    		return true;
		    	}else{
		    		return false;
		    	}
			}else{
				return false;
			}
		}
		
		public static function isUseAlternativeTextVars(){			
			$params = self::getComponentParams();
			if($params){
	    		return ($params->get('use_alt_txt_vars') > 0); 
			}else{
				return false;
			}
		}
		
		public static function isLoggingForced(){			
			$params = self::getComponentParams();
			if($params){
	    		return ($params->get('force_logging') > 0); 
			}else{
				return false;
			}
		}
		
		public static function useMailingListAddressAsFromField(){			
			$params = self::getComponentParams();
			if($params){
	    		return ($params->get('mail_from_field') > 0); 
			}else{
				return false;
			}
		}
		
		public static function useMailingListNameAsFromField(){			
			$params = self::getComponentParams();
			if($params){
	    		return ($params->get('name_from_field') > 0); 
			}else{
				return false;
			}
		}

        public static function getMailingListNameGlobalSetting(){
            $params = self::getComponentParams();
            if($params){
                /*
                 * earlier global format:
                 *      <option value="0">COM_MAILSTER_SENDER_NAME</option>
                 *      <option value="1">COM_MAILSTER_MAILING_LIST_NAME</option>
                 *
                 */
                $nameFromField = $params->get('name_from_field', MstConsts::NAME_FROM_MODE_SENDER_NAME_VIA_LIST_NAME);
                switch($nameFromField){
                    case "0": // old format for sender name
                        return MstConsts::NAME_FROM_MODE_SENDER_NAME;
                        break;
                    case "1": // old format for list name
                        return MstConsts::NAME_FROM_MODE_MAILING_LIST_NAME;
                        break;
                    case "sender_name": // new format for sender name
                        return MstConsts::NAME_FROM_MODE_SENDER_NAME;
                        break;
                    case "list_name": // new format for list name
                        return MstConsts::NAME_FROM_MODE_MAILING_LIST_NAME;
                    break;
                    case "sender_name_via_list_name": // format for sender name via list name
                        return MstConsts::NAME_FROM_MODE_SENDER_NAME_VIA_LIST_NAME;
                        break;
                    default:
                        return  MstConsts::NAME_FROM_MODE_SENDER_NAME_VIA_LIST_NAME;
                }
            }else{
                return MstConsts::NAME_FROM_MODE_SENDER_NAME_VIA_LIST_NAME;
            }
        }
		
		public static function insertSenderAddressForEmptySenderName(){			
			$params = self::getComponentParams();
			if($params){
	    		return ($params->get('mail_from_email_for_from_name_field') > 0); 
			}else{
				return false;
			}
		}

        public static function getBlockedEmailAddresses(){
            $params = self::getComponentParams();
            $words = array();
            if($params){
                $wordsStr = $params->get('blocked_email_addresses');
                $words = explode(',', $wordsStr);
                if($words){
                    $nrWords = count($words);
                    for($i=0;$i<$nrWords;$i++){
                        $word = $words[$i];
                        $word = trim($word);
                        if(($word === '') || ($word === MstConsts::NO_PARAMETER_SUPPLIED_FLAG)){
                            unset($words[$i]); // remove empty element
                        }else{
                            $words[$i] = $word; // take trimmed version
                        }
                    }
                    $words = array_values($words); // re-index
                }else{
                    $words = array();
                }
            }
            return $words;
        }
		
		public static function getWordsToFilter(){
			$params = self::getComponentParams();
			$words = array();
			if($params){
		    	$wordsStr = $params->get('words_to_filter');
		    	$words = explode(',', $wordsStr);
		    	if($words){
		    		$nrWords = count($words);
			    	for($i=0;$i<$nrWords;$i++){
			    		$word = $words[$i];
			    		$word = trim($word);
			    		if(($word === '') || ($word === MstConsts::NO_PARAMETER_SUPPLIED_FLAG)){
			    			unset($words[$i]); // remove empty element
			    		}
			    	}
			    	$words = array_values($words); // re-index
		    	}else{
		    		$words = array();
		    	}
			}
	    	return $words;
		}
		
		public static function getDigestMailFormat(){
			$params = self::getComponentParams();
			if($params){
		    	$dateFormat = trim($params->get('digest_format_html_or_plain'));
		    	if(strlen($dateFormat) < 1){
		    		$dateFormat = 'plain';
		    	}
			}else{
				$dateFormat = 'plain';
			}
	    	return $dateFormat;
		}
		
		public static function getDateFormat(){		
			$params = self::getComponentParams();
			if($params){
		    	$dateFormat = trim($params->get('date_format'));
		    	if(strlen($dateFormat) < 1){
		    		$dateFormat = '%d.%m.%Y %H:%M:%S';
		    	}
			}else{
				$dateFormat = '%d.%m.%Y %H:%M:%S';
			}
	    	return $dateFormat;
		}
		
		public static function getDateFormatWithoutTime(){		
			$params = self::getComponentParams();
			if($params){
		    	$dateFormat = trim($params->get('date_format_without_time'));
		    	if(strlen($dateFormat) < 1){
		    		$dateFormat = '%d.%m.%Y';
		    	}
			}else{
				$dateFormat = '%d.%m.%Y';
			}
	    	return $dateFormat;
		}
		
		public static function addMailsterMailHeaderTag(){	
			$params = self::getComponentParams();
			if($params){
		    	if ($params->get('tag_mailster_mails') > 0){
		    		return true;
		    	}
		    	return false;
			}else{
				return true;
			}
		}
		
		public static function includeBodyInBouncedBlockedNotifications(){	
			$params = self::getComponentParams();
			if($params){
		    	if ($params->get('include_body_in_blocked_bounced_notifies', 1) > 0){
		    		return true;
		    	}
		    	return false;
			}else{
				return true;
			}
		}
		
		public static function isUndoLineWrapping(){	
			$params = self::getComponentParams();
			if($params){
		    	if ($params->get('undo_line_wrapping') > 0){
		    		return true;
		    	}
		    	return false;
			}else{
				return false;
			}
		}

        public static function getMaxMailsPerMinute(){
            $default = 0;
            $params = self::getComponentParams();
            if(MstFactory::getV()->getFtSetting(MstVersionMgmt::MST_FT_ID_THROTTLE) && $params){
                return $params->get('max_mails_per_minute', $default);
            }else{
                return $default;
            }
        }

        public static function getMaxMailsPerHour(){
            $default = 0;
            $params = self::getComponentParams();
            if(MstFactory::getV()->getFtSetting(MstVersionMgmt::MST_FT_ID_THROTTLE) && $params){
                return $params->get('max_mails_per_hour', $default);
            }else{
                return $default;
            }
        }

		public static function getWaitBetweenTwoMails(){			
			$default = 0;	
			$params = self::getComponentParams();
			if(MstFactory::getV()->getFtSetting(MstVersionMgmt::MST_FT_ID_THROTTLE) && $params){
				return $params->get('wait_between_two_mails', $default);
			}else{
				return $default;
			}
		}
		
		public static function getLastMailSentAt(){
			$default = -1;	
			$params = self::getComponentParams();
			if($params){
				return $params->get('last_mail_sent_at', $default);
			}else{
				return $default;
			}
		}
		
		public static function getLastHourMailSentIn(){
			$default = -1;	
			$params = self::getComponentParams();
			if($params){
				return $params->get('last_hour_mail_sent_in', $default);
			}else{
				return $default;
			}
		}
		
		public static function getNrOfMailsSentInLastHour(){
			$default = -1;	
			$params = self::getComponentParams();
			if($params){
				return $params->get('nr_of_mails_sent_in_last_hour', $default);
			}else{
				return $default;
			}
		}
		
		public static function getLastDayMailSentIn(){
			$default = -1;	
			$params = self::getComponentParams();
			if($params){
				return $params->get('last_day_mail_sent_in', $default);
			}else{
				return $default;
			}
		}
		
		public static function getNrOfMailsSentInLastDay(){
			$default = -1;	
			$params = self::getComponentParams();
			if($params){
				return $params->get('nr_of_mails_sent_in_last_day', $default);
			}else{
				return $default;
			}
		}
		


		public static function setLastMailSentAt($lastMailSentAtTime){
			$params = self::getComponentParams();
			if($params){
				$params->set('last_mail_sent_at', $lastMailSentAtTime);
				return self::updateComponentParams($params);
			}
		}
		
		public static function setLastHourMailSentIn($lastHourMailSentIn){
			$params = self::getComponentParams();
			if($params){
				$params->set('last_hour_mail_sent_in', $lastHourMailSentIn);
				return self::updateComponentParams($params);
			}
		}
		
		public static function setNrOfMailsSentInLastHour($nrOfMailsSentInLastHour){
			$params = self::getComponentParams();
			if($params){
				$params->set('nr_of_mails_sent_in_last_hour', $nrOfMailsSentInLastHour);
				return self::updateComponentParams($params);
			}
		}
		
		public static function setLastDayMailSentIn($lastDayMailSentIn){
			$params = self::getComponentParams();
			if($params){
				$params->set('last_day_mail_sent_in', $lastDayMailSentIn);
				return self::updateComponentParams($params);
			}
		}
		
		public static function setNrOfMailsSentInLastDay($nrOfMailsSentInLastDay){
			$params = self::getComponentParams();
			if($params){
				$params->set('nr_of_mails_sent_in_last_day', $nrOfMailsSentInLastDay);
				return self::updateComponentParams($params);
			}
		}
		
		
		
		public static function loadLocalJSFramework(){	
			$params = self::getComponentParams();
			if($params){
		    	if ($params->get('local_js_framework') > 0){
		    		return true;
		    	}
			}
			return false;
			
		}
		
		public static function isJSFrameworkDisabled(){
			$params = self::getComponentParams();
			if($params){
		    	if ($params->get('disable_js_framework') > 0){
		    		return true;
		    	}
			}
			return false;
		}
		
		public static function addSubjectPrefixToReplies(){	
			$params = self::getComponentParams();
			if($params){
		    	if ($params->get('add_reply_prefix') > 0){
		    		return true;
		    	}
		    	return false;
			}else{
				return true;
			}
		}			
		
		public static function getMailboxOpenTimeout(){		
			$params = self::getComponentParams();
			if($params){
				if ($params->get('imap_opentimeout') > 0){
					return $params->get('imap_opentimeout');
				}
			}
			return 20;
		}	
		
		public static function getReplyPrefix(){		
			$params = self::getComponentParams();
			if($params){
	    		return $params->get('reply_prefix');
			}else{
				return "Re:";
			}
		}
		
		public static function getRecaptchaKeys(){			
			$params = self::getComponentParams();
	    	$keys = array();
	    	$keys['public'] = '';
	    	$keys['private'] = '';
			if($params){
		    	$keys['public'] = $params->get('recaptcha_public_key');
		    	$keys['private'] = $params->get('recaptcha_private_key');
			}
		    return $keys;
		}
		
		public static function getRecaptchaParamString(){			
			$params = self::getComponentParams();
			if($params){
				$theme = $params->get('recaptcha_theme');
				$lang = $params->get('recaptcha_lang');
			}else{
				$theme = 'red';
				$lang = 'en';
			}
			$paramStr =  "theme : '".$theme . "', lang :'" . $lang . "'";			
			return $paramStr;
		}	
		
		public static function getAttachmentBaseDir(){	
			$params = self::getComponentParams();
			if($params){
	    		$attachDir = $params->get('attachment_dir');
	    		if($attachDir && strlen($attachDir)>0){
	    			return $attachDir;
	    		}
			}
			return 'tmp';
		}
		
	}

?>