<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

defined('_JEXEC') or die('Restricted access');

class MstRecipients
{		

	public function updateRecipientInLists($userId, $isJUser){
        $log = MstFactory::getLogger();
		$lists = $this->getListsUserIsMemberOf($userId, $isJUser);
        if(count($lists)>0){
		    $this->recipientsUpdatedInLists($lists);
        }else{
            $log->debug('updateRecipientInLists there are not lists to be updated for user ID '.$userId.', isJUser: '.$isJUser);
        }
	}
	
	public function getListsUserIsMemberOf($userId, $isJUser){
		$model = MstFactory::getModel('user');
		$memberInfo = $model->getMemberInfo($userId, $isJUser);
		$usersLists = array();
		$lists = $memberInfo['lists'];
		for($i=0; $i<count($lists); $i++){
			$list = &$lists[$i];
			if(in_array($list->id, $usersLists) == false){
				$usersLists[] = $list->id; 
			}
		}
		$lists = $memberInfo['listGroups'];
		for($i=0; $i<count($lists); $i++){
			$list = &$lists[$i];
			if(in_array($list->id, $usersLists) == false){
				$usersLists[] = $list->id; 
			}
		}
		array_unique($usersLists);
		return $usersLists;
	}

    public function recipientsUpdatedInAllListsWithJoomlaGroups(){
        $log = MstFactory::getLogger();
        $mstConvUtils = MstFactory::getConverterUtils();
        /** @var MailsterModelGroups $groupsModel  */
        $groupsModel = MstFactory::getModel('groups');
        /**  @var MailsterModelListGroups $listgroupsModel  */
        $listgroupsModel = MstFactory::getModel('listgroups');
        $mstGroupsUsingJGroups = $groupsModel->getMailsterGroupsUsingJoomlaGroups(); // get the Mailster groups that are linked to Joomla groups

        $usedMstGroupIds = array_map(function($o){ return $o->id; }, $mstGroupsUsingJGroups);
        $log->debug('recipientsUpdatedInAllListsWithJoomlaGroups have to take care of all lists that have this groups in recipients: '.print_r($usedMstGroupIds, true));

        $listsToBeUpdated = $listgroupsModel->getListsWithGroups($usedMstGroupIds);
        $log->debug('recipientsUpdatedInAllListsWithJoomlaGroups take care of this lists: '.print_r(array_map(function($o){ return 'ID '.$o->id . ' '.$o->name; }, $listsToBeUpdated), true));

        foreach($listsToBeUpdated AS $list){
            $this->recipientsUpdated($list->id);
        }

        return true;
    }
	
	public function recipientsUpdatedInAllLists(){	
		$mListUtils = MstFactory::getMailingListUtils();
		$lists = $mListUtils->getAllMailingLists();
		foreach($lists AS $list){
			$this->recipientsUpdated($list->id);
		}
		return true;
	}
	
	public function recipientsUpdatedInLists($listIds){
        for($i=0;$i<count($listIds);$i++){
            $this->recipientsUpdated($listIds[$i]);
        }
	}
	
	public function recipientsUpdated($listId){		
		$log = MstFactory::getLogger();
        $log->debug('recipientsUpdated list ID '.$listId.', will update cache version');
		if($listId){
			if($listId != '' && $listId > 0){
				$cacheUtils = MstFactory::getCacheUtils();
				$cacheUtils->newRecipientState($listId);
				return;
			}else{
                $log->warning('recipientsUpdated list ID '.$listId.' not possible');
            }
		}
		$log->error('Cannot update Cache State for Recipients, listId not set');
		$log->error('listId: ' . $listId);
	}
	
	public function getRecipients($listId, $includeDigestInfo = false, $excludeInvalidAddresses = false, $markInvalidAddresses = false){
		$log = MstFactory::getLogger();
		$cache = MstFactory::getCache();
		$digestModel = MstFactory::getModel('digest');
		$cacheUtils = MstFactory::getCacheUtils();
		$version = $cacheUtils->getRecipientState($listId);	
		$countOnly = 0;
        $log->debug('getRecipients list ID: '.$listId.', includeDigestInfo: '.($includeDigestInfo ? 'yes':'no'));
        $log->debug('getRecipients cache call recipients() with version '. $version);
		$res = $cache->call( array( $this, 'recipients' ), $listId, $version);
        $log->debug('getRecipients result size: '.count($res));

		if($includeDigestInfo){
			for($i=0, $n=count( $res ); $i < $n; $i++) {
				$recipient = &$res[$i];
				$digests = $digestModel->getData(true, $recipient->user_id, $recipient->is_joomla_user, $listId);
				if(count($digests)>0){
					$recipient->digest = $digests[0];
				}else{
					$recipient->digest = false;
				}
			}
		}

        if($excludeInvalidAddresses || $markInvalidAddresses){
            /** @var JMail $mailer */
            $mailer = JFactory::getMailer();
            $mailUtils = MstFactory::getMailUtils();
            for($i=0, $n=count( $res ); $i < $n; $i++) {
                $validEmail = ($mailer->validateAddress($res[$i]->email) && $mailUtils->isValidEmail($res[$i]->email));
                if(!$validEmail){
                    if($excludeInvalidAddresses){
                        $log->warning('getRecipients list ID '.$listId.' has invalid email: '.$res[$i]->email.', will be EXCLUDED from result');
                        unset($res[$i]); // remove recipient object
                    }elseif($markInvalidAddresses){
                        $log->warning('getRecipients list ID '.$listId.' has invalid email: '.$res[$i]->email.', will only be marked in result');
                        $res[$i]->validEmail = false;
                    }
                }else{
                    if($markInvalidAddresses){
                        $res[$i]->validEmail = true;
                        $log->debug('validEmail: '.$res[$i]->email);
                    }
                }
            }

            $res = array_values($res); // re-index
        }

		return $res;	
	}
		
	public function getTotalRecipientsCount($listId, $excludeInvalidAddresses = false){
        $log = MstFactory::getLogger();
		$recips = $this->getRecipients($listId, false, $excludeInvalidAddresses);
        $count = count($recips);
        $log->debug('getTotalRecipientsCount result: '.$count);
		return $count;
	}

	function recipients($listId, $cacheVersion){
		$log = MstFactory::getLogger();
		
		$toSelect = ' SELECT name, email';
				
		$query = ' SELECT name, email, user_id, is_joomla_user FROM ('
				. $toSelect . ', id AS user_id, \'0\' AS is_joomla_user' // all Mst users directly linked to list
				. ' FROM #__mailster_users'
				. ' WHERE id in ('
					. ' SELECT user_id'
					. ' FROM #__mailster_list_members'
					. ' WHERE list_id = \'' . $listId . '\''
					. ' AND is_joomla_user=\'0\''
				. ' )'
				. ' UNION'
				. $toSelect . ', id AS user_id, \'1\' AS is_joomla_user' // all Joomla users directly linked to list
				. ' FROM #__users'
				. ' WHERE id in ('
					. ' SELECT user_id'
					. ' FROM #__mailster_list_members'
					. ' WHERE list_id = \'' . $listId . '\''
					. ' AND is_joomla_user=\'1\''
				. ' )'
				. ' UNION'
				. $toSelect . ', id AS user_id, \'0\' AS is_joomla_user' // all Mst users linked through a Mst group
				. ' FROM #__mailster_users'
				. ' WHERE id in ('
					. ' SELECT user_id'
					. ' FROM #__mailster_group_users'
					. ' WHERE group_id in ('
						. ' SELECT group_id'
						. ' FROM #__mailster_list_groups'
						. ' LEFT JOIN #__mailster_groups gr ON (group_id=gr.id)'
						. ' WHERE list_id = \'' . $listId . '\''
						. ' AND gr.is_joomla_group=\'0\''
					. ' )'				
					. ' AND is_joomla_user=\'0\''
				. ' )'				
				. ' UNION'
				. $toSelect . ', id AS user_id, \'1\' AS is_joomla_user' // all Joomla users linked through a Mst group
				. ' FROM #__users'
				. ' WHERE id in ('
					. ' SELECT user_id'
					. ' FROM #__mailster_group_users'
					. ' WHERE group_id in ('
						. ' SELECT group_id'
						. ' FROM #__mailster_list_groups'
						. ' LEFT JOIN #__mailster_groups gr ON (group_id=gr.id)'
						. ' WHERE list_id = \'' . $listId . '\''
						. ' AND gr.is_joomla_group=\'0\''
					. ' )'				
					. ' AND is_joomla_user=\'1\''
				. ' ) '
				. '	UNION '
					. $toSelect . ', ussr.id AS user_id, \'1\' AS is_joomla_user' // all Joomla users linked through a Joomla group
					. ' FROM #__user_usergroup_map umap'
                    . ' LEFT JOIN #__users ussr ON (umap.user_id=ussr.id)'
                    . ' WHERE umap.group_id in ('
                        . ' SELECT gr.joomla_group_id'
                        . ' FROM #__mailster_list_groups ggo'
                        . ' LEFT JOIN #__mailster_groups gr ON (ggo.group_id=gr.id)'
                        . ' WHERE ggo.list_id = \'' . $listId . '\''
                        . ' AND gr.is_joomla_group=\'1\''
                    . ' )'
			 	. ' ORDER BY name, email';
		$query .= ') AS recips';
		
		$log->debug('recipients_query (cache version now: '.$cacheVersion.'): '.$query);
		
		$db = JFactory::getDBO();
		$db->setQuery( $query );
		$errorMsg = false;
        try {
            $result = $db->loadObjectList();
        } catch (DatabaseException $e) {
            $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
        }
		if($errorMsg){
			$log->error('Recipients::recipients -> '.$errorMsg);
			$log->error('Recipients::recipients -> recipients_query: '.$query);
		}
		return $result;	
	}
		
	function isRecipient($listId, $email){	
		$email = strtolower(trim($email));	
		$recipients = $this->getRecipients($listId);
		$recipCount = count($recipients);
		for($j = 0; $j < $recipCount; $j++) {
			$recipient = &$recipients[$j];
			$recipMail = strtolower(trim($recipient->email));
			if($recipMail === $email){
				return true;
			}
		}
		return false;		
	}	
	
}
?>
