<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

defined('_JEXEC') or die('Restricted access');

class MstSendEvent
{		
	
	public static function newQueueMail($mailId, $recipCount, $emailSize){
		self::storeEvent($mailId, MstConsts::SEND_EVENT_NEW_QUEUE_MAIL, null, $recipCount, $emailSize, null, null, null);
	}
	public static function sendingRunStarted($mailId, $triggerType, $sendSessionId){
		self::storeEvent($mailId, MstConsts::SEND_EVENT_SENDING_RUN_STARTED, null, $triggerType, $sendSessionId, null, null, null);
	}
	public static function mailPrepared($mailId, $remainingRecipCount, $mailSendErrorCount){
		self::storeEvent($mailId, MstConsts::SEND_EVENT_MAIL_PREPARED, null, $remainingRecipCount, $mailSendErrorCount, null, null, null);
	}
	public static function mailPreparedForRecips($mailId, $recips, $remainingRecipCount, $mailSendErrorCount){
		self::storeEvent($mailId, MstConsts::SEND_EVENT_MAIL_PREPARED_FOR_RECIPS, $recips, $remainingRecipCount, $mailSendErrorCount, null, null, null);
	}
	public static function mailSendOk($mailId){
		self::storeEvent($mailId, MstConsts::SEND_EVENT_MAIL_SEND_OK, null, null, null, null, null, null);
	}
	public static function mailSendError($mailId, $recips, $errorMsg, $mailSendErrorCount, $maxSendErrorCount){
		self::storeEvent($mailId, MstConsts::SEND_EVENT_MAIL_SEND_ERROR, $recips, $mailSendErrorCount, $maxSendErrorCount, null, null, $errorMsg);
	}
	public static function recipientQueueRemovalDueErrors($mailId, $recips, $singleRecipErrorCount, $mailSendErrorCount, $maxSendErrorCount){
		self::storeEvent($mailId, MstConsts::SEND_EVENT_RECIP_ERROR_QUEUE_REMOVAL, $recips, $singleRecipErrorCount, $mailSendErrorCount, $maxSendErrorCount, null, null);
	}
	public static function sendingAbortedDueErrors($mailId, $recips, $mailSendErrorCount, $maxSendErrorCount, $remainingRecipCount){
		self::storeEvent($mailId, MstConsts::SEND_EVENT_SENDING_ABORTED, null, $mailSendErrorCount, $maxSendErrorCount, $remainingRecipCount, null, null);
	}
	public static function sendingFinished($mailId, $mailSendErrorCount){
		self::storeEvent($mailId, MstConsts::SEND_EVENT_SENDING_FINISHED, null, $mailSendErrorCount, null, null, null, null);
	}
	public static function sendingRunStopped($mailId, $triggerType, $sendSessionId){
		self::storeEvent($mailId, MstConsts::SEND_EVENT_SENDING_RUN_STOPPED, null, $triggerType, $sendSessionId, null, null, null);
	}
	public static function mailResend($mailId, $recipCount, $oldListId, $newListId, $userId){
		self::storeEvent($mailId, MstConsts::SEND_EVENT_MAIL_RESEND, null, $recipCount, $oldListId, $newListId, $userId, null);
	}
	
	private static function storeEvent($mailId, $eventId, $recips, $int1, $int2, $int3, $int4, $msg){
		$log = MstFactory::getLogger();
		$log->debug('Attempting to store '.self::getEventName($eventId). ' send event...');
		$mailUtils	= MstFactory::getMailUtils();
		$listUtils 	= MstFactory::getMailingListUtils();
		$mstUtils	= MstFactory::getUtils();
		$mail = $mailUtils->getMail($mailId);
		$mList	= $listUtils->getMailingList($mail->list_id);
		
		if($mList->save_send_reports <= 0){
			$log->debug('Do not need to store send report event, is turned off for list');
			return; // do not save send report
		}
		
		if($mail->has_send_report <= 0){
			$log->debug('Updating has send report flag of email ' . $mailId);
			self::setHasSendReportFlag($mailId, true); // set flag that we have a send report for this email
		}
		
		$db = JFactory::getDBO();
				
		if(!is_null($recips) && is_array($recips)){
			$recips = $mstUtils->jsonEncode($recips);
		}

		$recips = is_null($recips) ? 'NULL' : ($db->quote($recips));
		$msg 	= is_null($msg) ? 'NULL' : ($db->quote($msg));
		$int1 	= is_null($int1) ? -1 : $int1;
		$int2 	= is_null($int2) ? -1 : $int2;
		$int3 	= is_null($int3) ? -1 : $int3;
		$int4	= is_null($int4) ? -1 : $int4;
		
		$query =	'INSERT INTO'
		. ' #__mailster_send_reports'
		. ' ('
		. ' id,'
		. ' mail_id,'
		. ' event_time,'
		. ' event_type,'
		. ' recips,'
		. ' int_val1,'
		. ' int_val2,'
		. ' int_val3,'
		. ' int_val4,'
		. ' msg'
		. ' ) VALUES ('
		. ' NULL,'
		. ' \'' . $mailId . '\','
		. ' NOW(),'
		. ' \'' . $eventId . '\','
		. ' ' . $recips . ','
		. ' \'' . $int1 . '\','
		. ' \'' . $int2 . '\','
		. ' \'' . $int3 . '\','
		. ' \'' . $int4 . '\','
		. ' ' . $msg . ''
		. ' )';
		$db->setQuery($query);
		$eventStr = self::getEventName($eventId);
		
		$errorMsg = '';
        try {
            $result = $db->query();
        } catch (DatabaseException $e) {
            $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
        }
		if($result){
			$log->debug('Successfully stored send event: ' . $eventStr);
		}else{
			$log->error('Failed to store send event: ' . $eventStr . ', query was: "'.$query.'" , error was: ' . $errorMsg);
		}
				
		return $result;
	}
	
	public static function getEventName($eventId){
		$eventStr = '';
		switch($eventId){
			case MstConsts::SEND_EVENT_NEW_QUEUE_MAIL:
				$eventStr = JText::_( 'COM_MAILSTER_SEND_EVENT_NEW_QUEUE_MAIL' );
				break;
			case MstConsts::SEND_EVENT_SENDING_RUN_STARTED:
				$eventStr = JText::_( 'COM_MAILSTER_SEND_EVENT_SENDING_RUN_STARTED' );
				break;
			case MstConsts::SEND_EVENT_MAIL_PREPARED:
				$eventStr = JText::_( 'COM_MAILSTER_SEND_EVENT_MAIL_PREPARED_FOR_SENDING' );
				break;
			case MstConsts::SEND_EVENT_MAIL_PREPARED_FOR_RECIPS:
				$eventStr = JText::_( 'COM_MAILSTER_SEND_EVENT_MAIL_PREPARED_FOR_RECIPIENTS' );
				break;
			case MstConsts::SEND_EVENT_MAIL_SEND_OK:
				$eventStr = JText::_( 'COM_MAILSTER_SEND_EVENT_SUCCESSFULLY_SENT_MAIL' );
				break;
			case MstConsts::SEND_EVENT_MAIL_SEND_ERROR:
				$eventStr = JText::_( 'COM_MAILSTER_SEND_EVENT_SEND_ERROR_WHILE_SENDING' );
				break;
			case MstConsts::SEND_EVENT_RECIP_ERROR_QUEUE_REMOVAL:
				$eventStr = JText::_( 'COM_MAILSTER_SEND_EVENT_RECIPIENT_REMOVED_FROM_QUEUE_DUE_TOO_MANY_SEND_ERRORS' );
				break;
			case MstConsts::SEND_EVENT_SENDING_ABORTED:
				$eventStr = JText::_( 'COM_MAILSTER_SEND_EVENT_ABORTED_SENDING_MAIL_DUE_TOO_MANY_SEND_ERRORS' );
				break;
			case MstConsts::SEND_EVENT_SENDING_FINISHED:
				$eventStr = JText::_( 'COM_MAILSTER_SEND_EVENT_SENDING_FINISHED_FORWARD_COMPLETE' );
				break;
			case MstConsts::SEND_EVENT_SENDING_RUN_STOPPED:
				$eventStr = JText::_( 'COM_MAILSTER_SEND_EVENT_SENDING_RUN_STOPPED' );
				break;
			case MstConsts::SEND_EVENT_MAIL_RESEND:
				$eventStr = JText::_( 'COM_MAILSTER_SEND_EVENT_MAIL_RESENT' );
				break;
			default:
				$eventStr = JText::_( 'COM_MAILSTER_SEND_EVENT_UNKNOWN_SEND_EVENT' );
			break;
		}
		return $eventStr;
	}
	
	private static function getRecipInfoStr($recips){		
		$recipStr = '';
		if(count($recips) > 0){
			for($i=0; $i < count($recips); $i++){
				$recipStr .=  ($recips[$i]->name ?  ($recips[$i]->name.' (') : ''). '' . $recips[$i]->email . '' . ($recips[$i]->name ?  ')' : '') . "<br/>\r\n";
			}
		}
		return $recipStr;
	}
	
	public static function getEventExtraInformation(&$entry){
		$log = MstFactory::getLogger();
		$eventImg = 'components/com_mailster/assets/images/';
		$mstUtils	= MstFactory::getUtils();
		if($entry->recips && $entry->recips != null){
			$entry->recips = $mstUtils->jsonDecode($entry->recips);
			if(!isset($entry->recips->to)){
				return;
			}
			$entry->recips->toImg = $eventImg.'16-user-to.png';
			$entry->recips->ccImg = $eventImg.'16-user-cc.png';
			$entry->recips->bccImg = $eventImg.'16-user-bcc.png';
			$entry->recips->toStr = self::getRecipInfoStr($entry->recips->to);
			$entry->recips->ccStr = self::getRecipInfoStr($entry->recips->cc);
			$entry->recips->bccStr = self::getRecipInfoStr($entry->recips->bcc);
		}
	}
	
	public static function getEventDescription($event){
		$mstApp = MstFactory::getApplication();
		$eventDesc = '';
		$int1 = $event->int_val1;
		$int2 = $event->int_val2;
		$int3 = $event->int_val3;
		$int4 = $event->int_val4;
		switch($event->event_type){
			case MstConsts::SEND_EVENT_NEW_QUEUE_MAIL:
				$int2 = $int2/1024;
				$eventDesc = JText::sprintf( 'COM_MAILSTER_SEND_EVENT_NEW_QUEUE_MAIL_FOR_X_RECIPIENTS_WITH_SIZE_X_KBYTE', $int1, $int2 );
				break;
			case MstConsts::SEND_EVENT_SENDING_RUN_STARTED:
				$triggerType = $mstApp->getTriggerSourceName($int1);
				$eventDesc = JText::sprintf( 'COM_MAILSTER_SEND_EVENT_SENDING_RUN_STARTED_TRIGGER_WAS_X_AND_SESSION_ID_IS_X', $triggerType, $int2 );
				break;
			case MstConsts::SEND_EVENT_MAIL_PREPARED:
				$eventDesc = JText::sprintf( 'COM_MAILSTER_SEND_EVENT_MAIL_PREPARED_FOR_SENDING_REMAINING_RECIPIENT_COUNT_OF_X_AND_X_SEND_ERRORS_YET', $int1, $int2 );
				break;
			case MstConsts::SEND_EVENT_MAIL_PREPARED_FOR_RECIPS:
				$eventDesc = JText::sprintf( 'COM_MAILSTER_SEND_EVENT_MAIL_PREPARED_FOR_RECIPIENTS_REMAINING_RECIPIENT_COUNT_OF_X_AND_X_SEND_ERRORS_YET', $int1, $int2 );
				break;
			case MstConsts::SEND_EVENT_MAIL_SEND_OK:
				$eventDesc = JText::sprintf( 'COM_MAILSTER_SEND_EVENT_SUCCESSFULLY_SENT_MAIL_NO_SEND_ERRORS' );
				break;
			case MstConsts::SEND_EVENT_MAIL_SEND_ERROR:
				$eventDesc = JText::sprintf( 'COM_MAILSTER_SEND_EVENT_SEND_ERROR_WHILE_SENDING_MAIL_HAS_X_SEND_ERRORS_YET_MAX_ALLOWED_ARE_X_ERRORS', $int1, $int2 ) . '<br/>' . JText::_('ERROR')  . ': ' . $event->msg;
				break;
			case MstConsts::SEND_EVENT_RECIP_ERROR_QUEUE_REMOVAL:
				$eventDesc = JText::sprintf( 'COM_MAILSTER_SEND_EVENT_RECIPIENT_REMOVED_FROM_QUEUE_DUE_TOO_MANY_SEND_ERRORS_RECIPIENT_HAS_X_ERRORS_MAIL_HAS_X_ERRORS_MAX_ALLOWED_ARE_X_ERRORS', $int1, $int2, $int3 );
				break;
			case MstConsts::SEND_EVENT_SENDING_ABORTED:
				$eventDesc = JText::sprintf( 'COM_MAILSTER_SEND_EVENT_ABORTED_SENDING_MAIL_DUE_TOO_MANY_SEND_ERRORS_MAIL_HAS_X_ERRORS_MAX_ALLOWED_ARE_X_ERRORS_REMAINING_RECIPIENT_COUNT_IS_X', $int1, $int2, $int3 );
				break;
			case MstConsts::SEND_EVENT_SENDING_FINISHED:
				$eventDesc = JText::sprintf( 'COM_MAILSTER_SEND_EVENT_SENDING_FINISHED_FORWARD_COMPLETE_THERE_WERE_X_SEND_ERRORS_WHILE_SENDING', $int1 );
				break;
			case MstConsts::SEND_EVENT_SENDING_RUN_STOPPED:
				$triggerType = $mstApp->getTriggerSourceName($int1);
				$eventDesc = JText::sprintf( 'COM_MAILSTER_SEND_EVENT_SENDING_RUN_STOPPED_TRIGGER_WAS_X_AND_SESSION_ID_IS_X', $triggerType, $int2 );
				break;
			case MstConsts::SEND_EVENT_MAIL_RESEND:
				$user =& JFactory::getUser($int4);
				$username = $user->name . ' ('.$user->username.')';
				$eventDesc = JText::sprintf( 'COM_MAILSTER_SEND_EVENT_MAIL_RESENT_USER_X_RESEND_MAIL_FROM_LIST_X_AS_MAIL_OF_LIST_X_RESULTING_IN_X_RECIPIENTS', $username, $int1, $int2, $int3);
				break;
			default:
				$eventDesc = JText::sprintf( 'COM_MAILSTER_SEND_EVENT_UNKNOWN_SEND_EVENT_SEND_EVENT_HAS_ID_OF_X', $event->event_type );
			break;
		}
		return $eventDesc;
	}
	public static function getEventImg($eventId){
		$eventImg = 'components/com_mailster/assets/images/';
		switch($eventId){
			case MstConsts::SEND_EVENT_NEW_QUEUE_MAIL:
				$eventImg .= '16-send-event-new-queue-mail.png';
				break;
			case MstConsts::SEND_EVENT_SENDING_RUN_STARTED:
				$eventImg .= '16-send-event-send-run-started.png';
				break;
			case MstConsts::SEND_EVENT_MAIL_PREPARED:
				$eventImg .= '16-send-event-mail-prepared.png';
				break;
			case MstConsts::SEND_EVENT_MAIL_PREPARED_FOR_RECIPS:
				$eventImg .= '16-send-event-recips-prepared.png';
				break;
			case MstConsts::SEND_EVENT_MAIL_SEND_OK:
				$eventImg .= '16-send-event-send-ok.png';
				break;
			case MstConsts::SEND_EVENT_MAIL_SEND_ERROR:
				$eventImg .= '16-send-event-send-error.png';
				break;
			case MstConsts::SEND_EVENT_RECIP_ERROR_QUEUE_REMOVAL:
				$eventImg .= '16-send-event-recip-queue-removal.png';
				break;
			case MstConsts::SEND_EVENT_SENDING_ABORTED:
				$eventImg .= '16-send-event-send-aborted.png';
				break;
			case MstConsts::SEND_EVENT_SENDING_FINISHED:
				$eventImg .= '16-send-event-send-finished.png';
				break;
			case MstConsts::SEND_EVENT_SENDING_RUN_STOPPED:
				$eventImg .= '16-send-event-send-run-stopped.png';
				break;
			case MstConsts::SEND_EVENT_MAIL_RESEND:
				$eventImg .= '16-send-event-resend.png';
				break;
			default:
				$eventImg .= '';
			break;
		}
		return $eventImg;
	}
	
	public static function getSendEventsForMail($mailId, $limit=0){
		$db = JFactory::getDBO();
		$query = 	'SELECT * '
		. ' FROM #__mailster_send_reports'
		. ' WHERE mail_id = \'' . $mailId . '\''
		. ' ORDER BY event_time ASC, id ASC';
		if($limit > 0){
			$query .= (' LIMIT '.$limit);
		}
		$db->setQuery($query);
		$sendEvents = $db->loadObjectList();
		foreach($sendEvents AS $entry){
			$entry->name = self::getEventName($entry->event_type);
			$entry->desc = self::getEventDescription($entry);
			$entry->extras = self::getEventExtraInformation($entry);
			$entry->imgPath = self::getEventImg($entry->event_type);
		}
		return $sendEvents;
	}
	
	public static function getMailsWithSendReportOlderThan($listId, $ageInDays){
		$db = JFactory::getDBO();
		$query = ' SELECT mail_id'
					 . ' FROM #__mailster_send_reports'
					 . ' WHERE mail_id IN (SELECT id FROM #__mailster_mails '
					 					. ' WHERE list_id = \'' . $listId . '\''
					 					. ' AND DATEDIFF(NOW(), fwd_completed_timestamp) > ' . $ageInDays.')'
					 . ' GROUP by mail_id ';
		$db->setQuery($query);
		$mails = $db->loadObjectList();
		return $mails;
	}

    public static function deleteSendReportOfMails($mailIds){
        $log = MstFactory::getLogger();
        $db = JFactory::getDBO();

        if (count($mailIds)){
            $mailIdsStr = implode( ',', $mailIds);
            $query = 'DELETE FROM #__mailster_send_reports'
                . ' WHERE mail_id IN ('. $mailIdsStr .')';
            $db->setQuery($query);
            try {
                $result = $db->query();
            } catch (DatabaseException $e) {
                $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                $log->error('deleteSendReportOfMails Failed to delete send reports of these mails: '.$mailIdsStr.', error: ' . $errorMsg);
                return false;
            }
        }
        return true;

    }
	
	public static function deleteSendReportOfMail($mailId){
        $log = MstFactory::getLogger();
		$db = JFactory::getDBO();
		$query = 	'DELETE FROM #__mailster_send_reports'
		. ' WHERE mail_id = \'' . $mailId . '\'';
		$db->setQuery($query);
        try {
		    $result = $db->query();
        } catch (DatabaseException $e) {
            $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
            $log->error('deleteSendReportOfMail Failed to delete send reports of mail ID: '.$mailId.', error: ' . $errorMsg);
            return false;
        }
		return $result;
	}
	
	public static function setHasSendReportFlag($mailId, $hasSendReport){
		$log = MstFactory::getLogger();
		$has_send_report = $hasSendReport ? '1' : '0';
		$log->debug('Updating mail ' . $mailId . ': ' . ($hasSendReport ? 'has send report' : 'does not have send report'));
		
		// Update Mail
		$query = 'UPDATE #__mailster_mails SET has_send_report = \'' . $has_send_report . '\' WHERE id=\'' . $mailId . '\'';		
		$db = JFactory::getDBO();
		$db->setQuery( $query );
		$errorMsg = '';
        try {
            $result = $db->query();
        } catch (DatabaseException $e) {
            $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
        }
		if(!$result){
			$log->error('Updating of send report flag failed, ' . $errorMsg);
		}
	}
		
}
