<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

defined( '_JEXEC' ) or die( 'Restricted access' );
		
class MstSubscribeUtils
{
	
	public function getUnsubscribeURL($mail){
		$log		= MstFactory::getLogger();
		$env 		= MstFactory::getEnvironment();
		$hashUtils	= MstFactory::getHashUtils();
		if(!empty($mail->id) && ($mail->id > 0)){
			$hKey = $mail->hashkey;
			$salt = rand();
			$saltedKeyHash = $hashUtils->getUnsubscribeKey($salt, $hKey);
			$query = 'option=com_mailster&view=unsubscribe&m=' . ($mail->id) . '&h=' . $saltedKeyHash . '&s=' . $salt;
			$baseUrl = $env->getSiteBaseUrl();
			$query = $baseUrl.'index.php?'.$query;
			$log->debug('getUnsubscribeURL() query: ' . $query);		
			return $query;
		}
		return false;
	}

	public function subscribeUser($name, $email, $listId, $digestChoice=false){
		$name = trim($name);
		$email = trim($email);
		$success = false;
		if($digestChoice === false){
			$digestChoice = MstConsts::DIGEST_NO_DIGEST;
		}
		$log = MstFactory::getLogger();
		$log->debug('subscribeUser for '.$name.' (email: ' . $email . ', list id: ' . $listId.')');
		$mstRecipients = MstFactory::getRecipients();
		$cr = $mstRecipients->getTotalRecipientsCount($listId);
		$db = JFactory::getDBO();
		if($cr < MstFactory::getV()->getFtSetting(MstVersionMgmt::MST_FT_ID_REC)){
			$user = $this->getUserByEmail($email);
			if(!$user['user_found']){
				$query = ' INSERT INTO'
					. ' #__mailster_users'
					. ' (id, name, email)'
					. ' VALUES ('
					. ' NULL, ' . $db->quote( $name ) . ', ' . $db->quote( $email ) . ')';
				$db->setQuery( $query );
				$db->query();
				$user['user_id'] = $db->insertid();
				$user['is_joomla_user'] = '0';
				$log->debug('subscribeUser - '.$name.' needs to be stored as Mailster user, stored under ID: '.$user['user_id']);
			}
			$success = $this->subscribeUserId($user['user_id'], $user['is_joomla_user'], $listId, $digestChoice);
		}
		return $success;
	}
	
	public function sendWelcomeOrGoodbyeSubscriberMsg($name, $email, $listId, $subType){
		$log = MstFactory::getLogger();
		$mailingListUtils = MstFactory::getMailingListUtils();
		$mList = $mailingListUtils->getMailingList($listId);
		
		$body = '';
		$altBody = '';
		
		if($name && ($name !== '')){
			$salutation = JText::sprintf( 'COM_MAILSTER_HELLO_X', $name);
		}else{
			$salutation = JText::sprintf( 'COM_MAILSTER_HELLO_GENERIC');
		}
		
		switch($subType){
			case MstConsts::SUB_TYPE_SUBSCRIBE:
				$log->debug('Build welcome email for subscribe');
				$subject = JText::sprintf( 'COM_MAILSTER_WELCOME_TO_MAILING_LIST_X', $mList->name );
				$desc = JText::sprintf( 'COM_MAILSTER_YOU_ARE_NOW_SUBSCRIBED_TO_MAILING_LIST_X_WITH_EMAIL_ADDRESS_X', $mList->name, $email);
				if($mList->welcome_msg <= 0){
					$log->debug('Sending of welcome email is disabled, return without sending');
					return;
				}
				break;
			case MstConsts::SUB_TYPE_UNSUBSCRIBE:
				$log->debug('Build goodbye email for unsubscribe');
				$subject = JText::sprintf( 'COM_MAILSTER_YOU_WERE_REMOVED_FROM_MAILING_LIST_X', $mList->name );
				$desc = JText::sprintf( 'COM_MAILSTER_YOU_UNSUBSCRIBED_FROM_MAILING_LIST_X_WITH_EMAIL_ADDRESS_X', $mList->name, $email);
				if($mList->goodbye_msg <= 0){
					$log->debug('Sending of goodbye email is disabled, return without sending');
					return;
				}
				break;
		}
		
		$body .= "<html><head></head>";
		$body .= "<body>";
		$body .= "<p>";
		$body .= $salutation;
		$body .= "</p>";
		$body .= "<p>";
		$body .= $desc;
		$body .= "</p>";
		$body .= "</body>";
		$body .= "</html>";
				
		$altBody .= $salutation;
		$altBody .= "\n\n";
		$altBody .= $desc;
		$altBody .= "\n\n";
		
		$log->debug('Email text: '.$altBody);
		
		$mailSender = MstFactory::getMailSender();
		$mail = $mailSender->getListMailTmpl($mList);
        if(is_null($mail)){
            $log->error('sendWelcomeOrGoodbyeSubscriberMsg No Mailer object - exit method');
            return false;
        }
		
		$replyTo = array($mList->admin_mail, '');
        try {
            if(version_compare(JVERSION, '3.0', 'ge')) {
                $mail->addReplyTo($replyTo[0], $replyTo[1]);
            } else {
                $mail->addReplyTo($replyTo);
            }
        } catch (Exception $e) {
            $exceptionErrorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
            $log->error('sendWelcomeOrGoodbyeSubscriberMsg addReplyTo for '.print_r($replyTo, true).' caused exception: '.$exceptionErrorMsg);
        }
		$mail->FromName = $mList->name;
		$mail->addCustomHeader(MstConsts::MAIL_HEADER_RETURN_PATH . ': <>'); // try to set return path to NULL
		$mail->addCustomHeader(MstConsts::MAIL_HEADER_AUTO_SUBMITTED . ': auto-generated'); // indicate this was generated and we do not want a response
		try {
            $mail->AddAddress($email, $name);
        } catch (Exception $e) {
            $exceptionErrorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
            $log->error('sendWelcomeOrGoodbyeSubscriberMsg AddAdress for recipient '.$email.' (name: '.$name.') caused exception: '.$exceptionErrorMsg);
        }
		$mail->setSubject($subject);
		$mail->setBody($body);
		$mail->AltBody = $altBody;
		$mail->IsHTML(true);

        try {
            $mail->Send();  // send notificaton
        } catch (RuntimeException $e) {
            $exceptionErrorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
            $log->error('sendWelcomeOrGoodbyeSubscriberMsg error: '.$exceptionErrorMsg);
        }
		$error =  $mail->IsError();
		if($error == true) {
			// send errors?
			$log->error('Sending of email to subscriber failed! Last error: ' . $mail->ErrorInfo);
			return false;
		}else{
			$log->debug('Successfully sent subscriber email');
		}
		return true;
	}
	
	public function getUserByEmail($email, $returnAsObj=false, $enforceDataSource=false){
		
		$email = trim($email);
		$user = array();
		$user['user_found'] = false;
		$user['email'] = $email;
		$user['name'] = '';
        $user['description'] = '';
		$db = JFactory::getDBO();
		
		// if not data source preference given, start with looking at Joomla users
		// if data source preference is given, then go when it is 1 (= this means is_joomla_user = 1)
		if(($enforceDataSource === false) || ($enforceDataSource !== false) && ($enforceDataSource == 1)){
			$query = ' SELECT *'
					. ' FROM #__users'
					. ' WHERE email =' . $db->quote( $email );
			$db->setQuery( $query );
			$users = $db->loadObjectList();
		}else{
			$users = array();
		}
		
		if(count($users) > 0){
			$user['user_found'] = true;
			$user['user_id'] = $users[0]->id;
			$user['is_joomla_user'] = '1';
			$user['name'] = $users[0]->name;
            $user['description'] = '';
		}else{ // look at second data source (is_joomla_user = 0)
			$query = ' SELECT *'
					. ' FROM #__mailster_users'
					. ' WHERE email =' . $db->quote( $email );
			$db->setQuery( $query );
			$users = $db->loadObjectList();
			if(count($users) > 0){
				$user['user_found'] = true;
				$user['user_id'] = $users[0]->id;
				$user['is_joomla_user'] = '0';
				$user['name'] = $users[0]->name;
                $user['description'] = $users[0]->notes;
			}
		}
		if($returnAsObj){
			$convUtils = MstFactory::getConverterUtils();
			$user = $convUtils->array2Object($user);
		}
		return $user;
	}
	
	
	public function subscribeUserId($userId, $isJoomlaUser, $listId, $digestChoice=false){
		$log = MstFactory::getLogger();
		$success = false;		
		if($digestChoice === false){
			$digestChoice = MstConsts::DIGEST_NO_DIGEST;
		}
		$db = JFactory::getDBO();
		$query = ' SELECT *'
					. ' FROM #__mailster_list_members'
					. ' WHERE list_id =' . $db->quote( $listId )
					. ' AND user_id =' . $db->quote( $userId )
					. ' AND is_joomla_user =\'' . $isJoomlaUser . '\'';
		$db->setQuery( $query );
		$members = $db->loadObjectList();
		if(count($members) > 0){
			// no need to insert, the user is already in the list
			$log->debug('subscribeUserId - user ID '.$userId.', isJoomlaUser: '.$isJoomlaUser. ' does not need to be added to list ID '.$listId.', is already among list members');
		}else{
			$log->debug('subscribeUserId - user ID '.$userId.', isJoomlaUser: '.$isJoomlaUser. ' needs to be added to list ID '.$listId);
			$query = ' INSERT INTO'
				. ' #__mailster_list_members'
				. ' (list_id, user_id, is_joomla_user)'
				. ' VALUES ('
				. ' ' . $db->quote( $listId ) . ', ' . $db->quote( $userId ) . ', \'' . $isJoomlaUser . '\')';
			$db->setQuery( $query );			
			$db->query();
		}
		
		if($digestChoice != MstConsts::DIGEST_NO_DIGEST){
			$log->debug('subscribeUserId - user ID '.$userId.', isJoomlaUser: '.$isJoomlaUser. ' wants to retrieve messages in digests from list ID '.$listId.', digest setting: '.$digestChoice);
			
			$digestModel = MstFactory::getModel('digest');
			$digestObj = new stdClass();
			$digestObj->id = 0;
			$digestObj->list_id = $listId;
			$digestObj->user_id = $userId;
			$digestObj->is_joomla_user = $isJoomlaUser;
			$digestObj->digest_freq = $digestChoice;
			$digestObj->last_send_date = null;
			$digestObj->next_send_date = null;
			$digestArray = MstFactory::getConverterUtils()->object2Array($digestObj);
			$log->debug('subscribeUserId - digest array before saving: '.print_r($digestArray, true));
			// TODO FIXME Inlcude check in digestModel->store --> gibt es schon Digest für list, user (isJoomlaUser)? -> wenn ja wird es überschrieben
			$digestId = $digestModel->store($digestArray);
			$log->debug('subscribeUserId - digest stored with ID '.$digestId);
		}
		
		$mstRecipients = MstFactory::getRecipients();
		$mstRecipients->recipientsUpdated($listId);  // update cache state
		$success = true;
		return $success;
	}
	
	public function unsubscribeUser($email, $listId){
		$log = MstFactory::getLogger();
		$log->debug('unsubscribeUser for email: ' . $email . ', list id: ' . $listId);
		$email = trim($email);
		$success = false;
		
		$dataSourcePreference = array();
		$dataSourcePreference[] = 1; // go with Joomla users
		$dataSourcePreference[] = 0; // go with Mailster users
		
		for($i=0; $i < 2; $i++) // one iteration for Joomla Users and for Mailster Users
		{
			$user = $this->getUserByEmail($email, false, $dataSourcePreference[$i]);
			$log->debug('unsubscribeUser - found user: '.print_r($user, true));
			if($user['user_found']){
				$success = $success || $this->unsubscribeUserId($user['user_id'], $user['is_joomla_user'], $listId);
			}
		}
		return $success;
	}
	
	public function subscribeUserWithDoubleOptIn($email, $name, $listId, $add2GroupId=0, $digestChoice=false){	
		$log = MstFactory::getLogger();
		if($digestChoice === false){
			$digestChoice = MstConsts::DIGEST_NO_DIGEST;
		}
		$subscriptionInfoResult = $this->storeSubscriptionInfo($email, $name, $listId, MstConsts::SUB_TYPE_SUBSCRIBE, $add2GroupId, $digestChoice);
		$this->sendDoubleOptInRelatedEmail($email, $name, $listId, MstConsts::SUB_TYPE_SUBSCRIBE, $subsriptionInfoResult['hashkey'], $subsriptionInfoResult['subscription_id']);
		return $subscriptionInfoResult;
	}	
	
	public function unsubscribeUserWithDoubleOptIn($email, $listId){
		$log = MstFactory::getLogger();
		$user = $this->getUserByEmail($email);
        $subscriptionInfoResult = $this->storeSubscriptionInfo($email, $user['name'], $listId, MstConsts::SUB_TYPE_UNSUBSCRIBE);
		$this->sendDoubleOptInRelatedEmail($email, $user['name'], $listId, MstConsts::SUB_TYPE_UNSUBSCRIBE, $subsriptionInfoResult['hashkey'], $subsriptionInfoResult['subscription_id']);
		return $subscriptionInfoResult;
	}
	
	public function getSubscribeInfo($subscriptionId){
		$log = MstFactory::getLogger();
		$db = JFactory::getDBO();
		$query = ' SELECT *'
					. ' FROM #__mailster_subscriptions'
					. ' WHERE id =\'' . $subscriptionId . '\'';
		$db->setQuery( $query );
		$subscribeInfo = $db->loadObject();
		$log->debug($query);
		return $subscribeInfo;
	}
	
	public function deleteSubscriptionInfo($subscriptionId){
		$db = JFactory::getDBO();
		$query = ' DELETE '
				. ' FROM #__mailster_subscriptions'
				. ' WHERE id =\'' . $subscriptionId . '\'';
		$db->setQuery( $query );
		$result = $db->query();
		$affRows = $db->getAffectedRows();	
		if($affRows > 0){
			return true;
		}
		return false;
	}	
	
	public static function getSubscriptionInfoOlderThan($ageInDays){
		$db = JFactory::getDBO();
		$query = 	'SELECT *'
				. ' FROM #__mailster_subscriptions'
				. ' WHERE DATEDIFF(NOW(), sub_date) > ' . $ageInDays;
		$db->setQuery($query);
		$subInfos = $db->loadObjectList();
		return $subInfos;
	}
	
	public function storeSubscriptionInfo($email, $name, $listId, $subType, $add2GroupId=0, $digestChoice=false){
		if($digestChoice === false){
			$digestChoice = MstConsts::DIGEST_NO_DIGEST;
		}
		$result = array();
		$hashUtils = MstFactory::getHashUtils();
		$log = MstFactory::getLogger();
		$user = $this->getUserByEmail($email);
		if($user['user_found']){
			$userId = $user['user_id'];
		}else{
			$userId = 0;
		}
		
		$log->debug('Store subscription related info for: '.print_r($user, true));
		
		$hashkey = $hashUtils->getSubscriptionHashkey();
		$db = JFactory::getDBO();
		$query = ' INSERT INTO'
					. ' #__mailster_subscriptions'
					. ' ('
						. ' id,'
						. ' list_id,'
						. ' user_id,'
						. ' add2group,'
						. ' name,'
						. ' email,'
						. ' sub_type,'
						. ' sub_date,'
						. ' hashkey,'
						. ' digest_freq'
					. ') VALUES ('
						. ' NULL,'
						. ' \'' . $listId . '\','
						. ' \'' . $userId . '\','
						. ' \'' . $add2GroupId . '\','
						. ' ' . $db->quote( $name ) .','
						. ' ' . $db->quote( $email ) . ','
						. ' \'' . $subType . '\','
						. ' NOW(),'
						. ' ' . $db->quote( $hashkey ). ','
						. ' \'' . $digestChoice . '\''
					. ')';
		$db->setQuery( $query );
		$db->query();
		$subscriptionId = $db->insertid();
		$log->debug('Error msg: '.$db->getErrorMsg());
		$log->debug('Hashkey: '.$hashkey);	
		$result['hashkey'] = $hashkey;
		$result['subscription_id'] = $subscriptionId;
		return $result;
	}
	
	public function sendDoubleOptInRelatedEmail($email, $name, $listId, $subType, $hashkey, $subscriptionId){
		$log = MstFactory::getLogger();
		$mailingListUtils = MstFactory::getMailingListUtils();
		$mList = $mailingListUtils->getMailingList($listId);
		
		$body = '';
		$altBody = '';
		
		if($name){
			$salutation = JText::sprintf( 'COM_MAILSTER_HELLO_X', $name);
		}else{
			$salutation = JText::sprintf( 'COM_MAILSTER_HELLO_GENERIC');
		}
				
		switch($subType){
			case MstConsts::SUB_TYPE_SUBSCRIBE:
				$log->debug('Build subscription email for subscribe');
				$link = $this->getDoubleOptInSubscriptionConfirmationURL($hashkey, $subscriptionId);
				$subject = JText::sprintf( 'COM_MAILSTER_CONFIRM_YOUR_SUBSCRIPTION_TO_MAILING_LIST_X', $mList->name );
				$desc = JText::sprintf( 'COM_MAILSTER_YOU_REQUESTED_TO_SUBSCRIBE_TO_MAILING_LIST_X_WITH_EMAIL_ADDRESS_X', $mList->name, $email);
				$linkDesc = JText::_( 'COM_MAILSTER_PLEASE_CONFIRM_SUBSCRIPTION_BY_CLICKING_ON_THIS_LINK');
				$htmlLink = '<a href="'.$link.'">'. JText::_( 'COM_MAILSTER_CLICK_LINK_FOR_CONFIRMATION').'</a>';
				break;
			case MstConsts::SUB_TYPE_UNSUBSCRIBE:
				$log->debug('Build subscription email for unsubscribe');
				$link = $this->getDoubleOptInUnubscriptionConfirmationURL($hashkey, $subscriptionId);
				$subject = JText::sprintf( 'COM_MAILSTER_CONFIRM_YOUR_UNSUBSCRIPTION_FROM_THE_MAILING_LIST_X', $mList->name );
				$desc = JText::sprintf( 'COM_MAILSTER_YOU_REQUESTED_TO_UNSUBSCRIBE_FROM_MAILING_LIST_X_WITH_EMAIL_ADDRESS_X', $mList->name, $email);
				$linkDesc = JText::_( 'COM_MAILSTER_PLEASE_CONFIRM_UNSUBSCRIPTION_BY_CLICKING_ON_THIS_LINK');
				$htmlLink = '<a href="'.$link.'">'. JText::_( 'COM_MAILSTER_CLICK_LINK_FOR_CONFIRMATION').'</a>';
				break;
		}
		
		$body .= "<html><head></head>";
		$body .= "<body>";
		$body .= "<p>";
		$body .= $salutation;
		$body .= "</p>";
		$body .= "<p>";
		$body .= $desc;
		$body .= "<br/>";
		$body .= ($linkDesc . ': ' . $htmlLink);
		$body .= "</p>";
		$body .= "</body>";
		$body .= "</html>";
		
		
		$altBody .= $salutation;
		$altBody .= "\n\n";
		$altBody .= $desc;
		$altBody .= "\n\n";
		$altBody .= ($linkDesc . ': ' . $link);
		$altBody .= "\n\n";
		
		$log->debug('Subscription email text: '.$altBody);
				
		$mailSender = MstFactory::getMailSender();
		$mail = $mailSender->getListMailTmpl($mList);
        if(is_null($mail)){
            $log->error('sendDoubleOptInRelatedEmail No Mailer object - exit method');
            return false;
        }
		
		$replyTo = array($mList->admin_mail, '');
        try {
            if(version_compare(JVERSION, '3.0', 'ge')) {
                $mail->addReplyTo($replyTo[0], $replyTo[1]);
            } else {
                $mail->addReplyTo($replyTo);
            }
        } catch (Exception $e) {
            $exceptionErrorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
            $log->error('sendDoubleOptInRelatedEmail addReplyTo for '.print_r($replyTo, true).' caused exception: '.$exceptionErrorMsg);
        }
		$mail->FromName = $mList->name;
		$mail->addCustomHeader(MstConsts::MAIL_HEADER_RETURN_PATH . ': <>'); // try to set return path to NULL
		$mail->addCustomHeader(MstConsts::MAIL_HEADER_AUTO_SUBMITTED . ': auto-generated'); // indicate this was generated and we do not want a response
        try {
		    $mail->AddAddress($email, $name);
        } catch (Exception $e) {
            $exceptionErrorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
            $log->error('sendDoubleOptInRelatedEmail AddAdress for recipient '.$email.' (name: '.$name.') caused exception: '.$exceptionErrorMsg);
        }
		$mail->setSubject($subject);
		$mail->setBody($body);
		$mail->AltBody = $altBody;
		$mail->IsHTML(true);

        try {
            $mail->Send();  // send notificaton
        } catch (RuntimeException $e) {
            $exceptionErrorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
            $log->error('sendDoubleOptInRelatedEmail error: '.$exceptionErrorMsg);
        }
		$error =  $mail->IsError();
		if($error == true) { // send errors?
			$log->error('Sending of double opt-in confirmation failed! Last error: ' . $mail->ErrorInfo);
			return false;
		}else{
			$log->debug('Successfully sent subscription action confirmation email');
		}
		return true;
	}
	
	public function getDoubleOptInSubscriptionConfirmationURL($hashkey, $subscriptionId){
		$log 		= MstFactory::getLogger();
		$hashUtils 	= MstFactory::getHashUtils();
		$env 		= MstFactory::getEnvironment();
		$salt = rand();
		$saltedKeyHash = $hashUtils->getSubscribeKey($salt, $hashkey);
		$log->debug('SubscriptionConfirmation salt: ' . $salt);
		$log->debug('SubscriptionConfirmation hashkey: '.$hashkey);
		$log->debug('SubscriptionConfirmation saltedKeyHash: '.$saltedKeyHash);
		$query = 'option=com_mailster&view=subscribe&si='.$subscriptionId.'&sm='.MstConsts::SUBSCRIBE_MODE_DOUBLE_OPT_IN.'&h=' . $saltedKeyHash . '&s=' . $salt;		
		$baseUrl = $env->getSiteBaseUrl();
		$query = $baseUrl.'index.php?'.$query;		
		$log->debug('SubscriptionConfirmation url: ' . $query);	
		return $query;
	}
	
	public function getDoubleOptInUnubscriptionConfirmationURL($hashkey, $subscriptionId){
		$log 		= MstFactory::getLogger();
		$hashUtils 	= MstFactory::getHashUtils();
		$env 		= MstFactory::getEnvironment();
		$salt = rand();
		$saltedKeyHash = $hashUtils->getUnsubscribeKey($salt, $hashkey);
		$log->debug('UnsubscriptionConfirmation salt: ' . $salt);
		$log->debug('UnsubscriptionConfirmation hashkey: '.$hashkey);
		$log->debug('UnsubscriptionConfirmation saltedKeyHash: '.$saltedKeyHash);
		$query = 'option=com_mailster&view=unsubscribe&si='.$subscriptionId.'&sm='.MstConsts::UNSUBSCRIBE_MODE_DOUBLE_OPT_IN.'&h=' . $saltedKeyHash . '&s=' . $salt;
		$baseUrl = $env->getSiteBaseUrl();
		$query = $baseUrl.'index.php?'.$query;
		$log->debug('UnsubscriptionConfirmation url: ' . $query);				
		return $query;
	}
	
	public function unsubscribeUserId($userId, $isJoomlaUser, $listId, $directSubscribersUnsubscribeOnly = false){
		$log = MstFactory::getLogger();
		$log->debug('Unsubscribe user id: '.$userId.' (is joomla user: '.$isJoomlaUser.') from list id: '.$listId);
		$success = false;
		$db = JFactory::getDBO();
        if(!$directSubscribersUnsubscribeOnly){
            $query = ' DELETE '
                    . ' FROM #__mailster_group_users'
                    . ' WHERE user_id =\'' . $userId . '\''
                    . ' AND is_joomla_user =\'' . $isJoomlaUser . '\''
                    . ' AND group_id IN ('
                    . ' 	SELECT group_id'
                    . ' 	FROM #__mailster_list_groups'
                    . '		WHERE list_id = \'' . $listId . '\')';
            $db->setQuery( $query );
            $result = $db->query();
            $affRows = $db->getAffectedRows();
            if($affRows > 0){
                $success = true;
            }
            $log->debug('Affected rows (group users): '.$affRows);
        }
		$query = ' DELETE '
				. ' FROM #__mailster_list_members'
				. ' WHERE user_id =\'' . $userId . '\''
				. ' AND is_joomla_user =\'' . $isJoomlaUser . '\''
				. ' AND list_id = \'' . $listId . '\'';
		$db->setQuery( $query );
		$result = $db->query();
		$affRows = $db->getAffectedRows();
		if($affRows > 0){
			$success = true;
		}
		$log->debug('Affected rows (list members): '.$affRows);
		$mstRecipients = MstFactory::getRecipients();
		$mstRecipients->recipientsUpdated($listId);  // update cache state
		return $success;
	}

	public function isUserLoggedIn(){
		$user = JFactory::getUser();
		if($user->id){
			return true;
		}
		return false;
	} 
	
	public function getMailingLists2RegisterAt($onlyPublicRegistration, $onlyActiveLists=false){		
		$publicRegistration = $onlyPublicRegistration ? '1' : '0'; 
		$db = JFactory::getDBO();
		$query = ' SELECT *'
				. ' FROM #__mailster_lists'
				. ' WHERE allow_subscribe =\'1\'';
		if($onlyPublicRegistration == true){	
			$query = $query	. ' AND public_registration =' . $db->quote( $publicRegistration );
		}
		if($onlyActiveLists){
			$query = $query	. ' AND active =' . $db->quote( '1' );
		}
		$db->setQuery( $query );
		$lists = $db->loadObjectList();
		return $lists;
	} 
	
	public function getMailingLists2UnsubscribeFrom(){	
		$db = JFactory::getDBO();
		$query = ' SELECT *'
				. ' FROM #__mailster_lists'
				. ' WHERE allow_unsubscribe =\'1\'';
		$db->setQuery( $query );
		$lists = $db->loadObjectList();
		return $lists;
	}
	
	public function isUserSubscribedToMailingList($userId, $isJoomlaUser, $listId){
		$mstRecips = MstFactory::getRecipients();
		$recipients = $mstRecips->getRecipients($listId);
		$recipCount = count($recipients);
		for($j = 0; $j < $recipCount; $j++) {
			$recipient = &$recipients[$j];
			if($userId == $recipient->user_id && $isJoomlaUser == $recipient->is_joomla_user){
				return true;
			}
		}
		return false;
	}
	
}
?>