<?php
/**
 * @package Joomla
 * @subpackage Mailster
 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
 * @license GNU/GPL, see license.txt
 * Mailster is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.
 *
 * Mailster is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mailster; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * or see http://www.gnu.org/licenses/.
 */

defined('_JEXEC') or die('Restricted access');

class MstCBUtils
{
	public static function isCBInstalled(){
		$dbUtils = MstFactory::getDBUtils();
		return $dbUtils->isTableExisting('#__comprofiler');
	}

    public static function getCBVersion(){
        /** @var JTableExtension $extension */
        $extension = JTable::getInstance('extension');
        $id = $extension->find(array('element' => 'com_comprofiler'));
        $extension->load($id);
        $componentInfo = json_decode($extension->manifest_cache, true);
        return $componentInfo['version'];
    }

	public static function updateCBProfile($id, $userProfileObj, $fields2Update){
		$log = MstFactory::getLogger();
		$db = JFactory::getDBO();
		$query = ' UPDATE #__comprofiler SET';

		foreach($fields2Update AS $field){
			$query .= (' '.$field.' = '.$db->quote($userProfileObj->$field));
		}

		$query .= ' WHERE user_id =\'' . $id . '\'';
		$log->debug('updateCBProfile query: '.$query);
		$db->setQuery($query);
		$result = $db->query();
	}
	
	public static function updateCBProfileField($id, $fieldName, $fieldValuesArr){
		$log = MstFactory::getLogger();
        $log->debug('updateCBProfileField ID: '.$id.', fieldName: '.$fieldName.', fieldValuesArr: '.print_r($fieldValuesArr, true));
		$fieldValueStr = implode('|*|', $fieldValuesArr); // create String like "Group 1|*|Group 2"
		$log->debug('updateCBProfileField userGroupNameStr: '.$fieldValueStr);
		
		$cbUser = self::getCBUser($id);
		$log->debug('updateCBProfileField Profile to update: '.print_r($cbUser, true));
		if($cbUser){
			$cbUser->$fieldName = $fieldValueStr; // replace with new groups string
			self::updateCBProfile($id, $cbUser, array($fieldName)); // update CB User profile -> only the relevant field...
			return true;
		}
		return false;
	}

	public static function getCBFieldsSuitableForCBUserGroupSync(){
		$db = JFactory::getDBO();
		$query = "SELECT '0' AS value, '- Non selected -' AS cb_user_group_field "
		. " UNION SELECT fieldid AS value, name AS cb_user_group_field"
		. " FROM #__comprofiler_fields"
		. " WHERE `type` = 'multicheckbox' OR `type` = 'multiselect' OR `type` = 'radio' OR `type` = 'select'";
		$db->setQuery($query);
		return $db->loadObjectList();
	}
		
	public static function getCBUser($id){
		$db = JFactory::getDBO();
		$query = 'SELECT c.*, ju.block AS block, ju.email AS email, ju.name AS name'
		. ' FROM #__comprofiler c'
		. ' LEFT JOIN #__users ju ON (c.user_id=ju.id)'
		. ' WHERE c.id = \''.$id.'\'';
		$db->setQuery($query);
		return $db->loadObject();
	}
			
	public static function getCBUserByConfirmcode($confirmcode){
        $log = MstFactory::getLogger();
		$db = JFactory::getDBO();
		$query = 'SELECT id'
		. ' FROM #__comprofiler'
		. ' WHERE cbactivation = '.$db->quote($confirmcode);
		$db->setQuery($query);
		$idObj = $db->loadObject();
        $log->debug('CBUtils::getCBUserByConfirmcode for code '.$confirmcode.', produced query: '.$query.', produced result: '.print_r($idObj, true));
        if($idObj){
		    return self::getCBUser($idObj->id);
        }
        return false;
	}
		
	public static function getCBFieldId($name){
		$db = JFactory::getDBO();
		$query = 'SELECT fieldid'
		. ' FROM #__comprofiler_fields'
		. ' WHERE name = \''.$name.'\'';
			
		$db->setQuery($query);
		$fieldId = $db->loadResult();
		if($fieldId && $fieldId > 0){
			return $fieldId;
		}
		return false;
	}

	public static function getCBFieldByFieldName($name){
		$fieldId = self::getCBFieldId($name);
		if($fieldId){
			return self::getCBField($fieldId);
		}
		return false;
	}
		
	public static function getCBField($fieldId){
		$db = JFactory::getDBO();
		$query = 'SELECT *'
		. ' FROM #__comprofiler_fields'
		. ' WHERE fieldid = '.$fieldId;
			
		$db->setQuery($query);
		$field = $db->loadObject();

		if($field){
			$field->fieldValues = self::getCBFieldValues($fieldId);
			return $field;
		}else{
			return false;
		}
	}

	public static function getCBFieldValuesByFieldName($name){
		$fieldId = self::getCBFieldId($name);
		if($fieldId){
			return self::getCBFieldValues($fieldId);
		}
		return false;
	}

	public static function getCBFieldValues($fieldId){
		$db = JFactory::getDBO();
		$query = 'SELECT *'
		. ' FROM #__comprofiler_field_values'
		. ' WHERE fieldid = '.$fieldId
		. ' ORDER BY ordering, fieldtitle';
		$db->setQuery($query);
		$fieldValues = $db->loadObjectList();
		return $fieldValues;
	}

	public static function getAllUsersCBUserGroups($fieldName){
		$db = JFactory::getDBO();
		$fieldId = self::getCBFieldId($fieldName);
		$field = self::getCBField($fieldId);
		$tbl = $field->table;

		$db = JFactory::getDBO();
		$query = 'SELECT id, '.$fieldName
		. ' FROM ' . $tbl
		. ' WHERE 1';
		$db->setQuery($query);
		$users = $db->loadObjectList();
		return $users;
	}

	public static function getLinkedUserGroupIds($fieldId){
		$allLinkedUserGroupIds = array();
		$log = MstFactory::getLogger();
		$groupModel = MstFactory::getModel('group');
		$allCBUserGroups = self::getCBFieldValues($fieldId);
		// $log->debug('getLinkedUserGroupIds allCBUserGroups: '.print_r($allCBUserGroups, true));
		for($i=0; $i<count($allCBUserGroups);$i++){
			$group = $groupModel->getGroupByName($allCBUserGroups[$i]->fieldtitle);
			if(!is_null($group) && $group->id > 0){
				$allLinkedUserGroupIds[] = $group->id;
				$log->debug('getLinkedUserGroupIds matched user group "'.$allCBUserGroups[$i]->fieldtitle.'", is Mailster group ID '.$group->id);
			}
		}
		return $allLinkedUserGroupIds;
	}

	public static function getNotLinkedUserGroupNames($fieldId){
		$notLinkedUserGroupNames = array();
		$log = MstFactory::getLogger();
		$groupModel = MstFactory::getModel('group');
		$allCBUserGroups = self::getCBFieldValues($fieldId);
		// $log->debug('getNotLinkedUserGroupIds allCBUserGroups: '.print_r($allCBUserGroups, true));
		for($i=0; $i<count($allCBUserGroups);$i++){
			$group = $groupModel->getGroupByName($allCBUserGroups[$i]->fieldtitle);
			if(is_null($group) || $group->id <= 0){
				$notLinkedUserGroupNames[] = $allCBUserGroups[$i]->fieldtitle;
				$log->debug('getNotLinkedUserGroupNames Did not match user group field value '.$allCBUserGroups[$i]->fieldtitle);
			}
		}
		return $notLinkedUserGroupNames;
	}

	public static function syncMailsterGroupsToCBUserGroupsForAllUsers($fieldName){
		$initLog = array();
		$log = MstFactory::getLogger();
		$fieldId = self::getCBFieldId($fieldName);
		$allLinkedUserGroupIds = self::getLinkedUserGroupIds($fieldId);
		$log->debug('syncMailsterGroupsToCBUserGroupsForAllUsers fieldName: '.$fieldName);
		$groupsModel = MstFactory::getModel('groups');
		$groupUsersModel = MstFactory::getModel('groupusers');
		$groupUsers = $groupUsersModel->getAllGroupsUsersOfJoomlaUserData();
		$allGroupsIndexedById = $groupsModel->getData(true);
		$log->debug('syncMailsterGroupsToCBUserGroupsForAllUsers allGroupsIndexedById: '.print_r($allGroupsIndexedById, true));
		$log->debug('syncMailsterGroupsToCBUserGroupsForAllUsers allLinkedUserGroupIds: '.print_r($allLinkedUserGroupIds, true));
		$log->debug('syncMailsterGroupsToCBUserGroupsForAllUsers groupUsers: '.print_r($groupUsers, true));

		$usersProfilesUpdate = 0;
		$lastId = -1;
		$groupsOfUser = array();
		for($k=0;$k<count($groupUsers);$k++){
			$groupUser = $groupUsers[$k];
			if($lastId != $groupUser->user_id && $lastId > 0){
				$log->debug('syncMailsterGroupsToCBUserGroupsForAllUsers() Now process CB user ID '.$lastId);
				$userGroupNameArr = array();
				$userGroupsLinked = array_intersect($allLinkedUserGroupIds, $groupsOfUser); // intersect group IDs
				$log->debug('syncMailsterGroupsToCBUserGroupsForAllUsers groupsOfUser: '.print_r($groupsOfUser, true));
				$log->debug('syncMailsterGroupsToCBUserGroupsForAllUsers userGroupsLinked: '.print_r($userGroupsLinked, true));
                foreach($userGroupsLinked AS $i => $linkedUserGroupId){
                    $userGroupNameArr[] = $allGroupsIndexedById[$linkedUserGroupId]->name;
                }
				if(count($userGroupNameArr)){					
					if(self::updateCBProfileField($lastId, $fieldName, $userGroupNameArr)){
						$initLog[] = JText::sprintf( 'COM_MAILSTER_ADDED_X_GROUPS_TO_USER_PROFILE_X', count($userGroupNameArr), $lastId);
						$usersProfilesUpdate++;
					}
				}else{
					$log->debug('syncMailsterGroupsToCBUserGroupsForAllUsers Skip user profile '.$lastId.', no groups to add to CB');
				}
				$groupsOfUser = array(); // reset tmp array
			}
			$lastId = $groupUser->user_id;
			$groupsOfUser[] = $groupUser->group_id; // add to tmp array
			
			if($k == count($groupUsers)-1){ // do not forget to take last array entry into account / save it
				$log->debug('syncMailsterGroupsToCBUserGroupsForAllUsers() Now process CB user ID '.$lastId);
				$userGroupNameArr = array();
				$userGroupsLinked = array_intersect($allLinkedUserGroupIds, $groupsOfUser); // intersect group IDs
				$log->debug('syncMailsterGroupsToCBUserGroupsForAllUsers groupsOfUser: '.print_r($groupsOfUser, true));
				$log->debug('syncMailsterGroupsToCBUserGroupsForAllUsers userGroupsLinked: '.print_r($userGroupsLinked, true));
				for($i=0;$i<count($userGroupsLinked);$i++){
					$userGroupNameArr[] = $allGroupsIndexedById[$userGroupsLinked[$i]]->name;
				}
				if(count($userGroupNameArr)){
					if(self::updateCBProfileField($lastId, $fieldName, $userGroupNameArr)){
						$initLog[] = JText::sprintf( 'COM_MAILSTER_ADDED_X_GROUPS_TO_USER_PROFILE_X', count($userGroupNameArr), $lastId);
						$usersProfilesUpdate++;
					}
				}else{
					$log->debug('syncMailsterGroupsToCBUserGroupsForAllUsers Skip user profile '.$lastId.', no groups to add to CB');
				}
			}
		}
		$log->debug('syncMailsterGroupsToCBUserGroupsForAllUsers Updated '.$usersProfilesUpdate.' user profiles');
		$initLog[] = JText::sprintf( 'COM_MAILSTER_UPDATED_X_USER_PROFILES', $usersProfilesUpdate );
		return $initLog;
	}
	
	public static function syncCBUserStatesWithStateChangeActionForAllUsers($statusField, $syncAction, $mailingListId, $userGroupId, $forceNoDoubleOptIn, $forceNoWelcomeGoodbyeMsg, $doSyncActionInReverse=false){
		$log = MstFactory::getLogger();
		$logArray = array();
		$db = JFactory::getDBO();
		$query = 'SELECT id'
		. ' FROM #__comprofiler WHERE 1';
		$db->setQuery($query);
		$userIdObjs = $db->loadObjectList();
		$logArray[] = JText::_( 'COM_MAILSTER_INITIAL_SYNCHRONIZATION' );
		$logArray[] = JText::sprintf( 'COM_MAILSTER_X_USERS_WILL_BE_SYNCHRONIZED', count($userIdObjs));
		$log->debug('syncCBUserStatesWithStateChangeActionForAllUsers: '.($forceNoDoubleOptIn ? 'Enforce to not use double opt-in':'Use double opt-in settings of list'));
		$log->debug('syncCBUserStatesWithStateChangeActionForAllUsers: '.($forceNoWelcomeGoodbyeMsg ? 'Enforce to not send welcome/goodbye':'Use welcome/goodbye settings of list'));
		$log->debug('syncCBUserStatesWithStateChangeActionForAllUsers: '.($doSyncActionInReverse ? 'Do also reverse action of defined syncAction: '.$syncAction:'Do NOT do reverse action of defined syncAction: '.$syncAction));
		for($i=0;$i<count($userIdObjs);$i++){
			$logArray = array_merge($logArray, self::syncCBUserStateWithStateChangeAction($userIdObjs[$i]->id, $statusField, $syncAction, $mailingListId, $userGroupId, $forceNoDoubleOptIn, $forceNoWelcomeGoodbyeMsg, $doSyncActionInReverse));
		}
		return $logArray;
	}
	
	public static function syncCBUserStateWithStateChangeAction($userId, $statusField, $syncAction, $mailingListId, $userGroupId, $forceNoDoubleOptIn, $forceNoWelcomeGoodbyeMsg, $doSyncActionInReverse=false){
		$log = MstFactory::getLogger();
		$logArray = array();
		$cbUser = self::getCBUser($userId);
		$listUtils = MstFactory::getMailingListUtils();
		$subscrUtils = MstFactory::getSubscribeUtils();
		$list = $listUtils->getMailingList($mailingListId);
		$recips = MstFactory::getRecipients();
		if(($cbUser->$statusField == 1) || ($doSyncActionInReverse && ($cbUser->$statusField != 1))){
			
			$userName =  '[ID='.$userId.'] '.$cbUser->name.' ('.$cbUser->email.')';
			if($cbUser->$statusField == 1){
				$log->debug('syncCBUserGroupsToMailsterGroups: Status of '.$statusField.' of user '.$userName.' is '.$cbUser->$statusField.', process sync action (syncAction: '.$syncAction.') in normal way...');
			}elseif($doSyncActionInReverse && ($cbUser->$statusField == 0)){
				$log->debug('syncCBUserGroupsToMailsterGroups: Status of '.$statusField.' of user '.$userName.' is '.$cbUser->$statusField.', process sync action (syncAction: '.$syncAction.') in REVERSE...');
			}else{
				$log->error('syncCBUserGroupsToMailsterGroups: Unknown pre-condition for sync!');
			}
			
			if($list){
				$isRecip = $recips->isRecipient($mailingListId, $cbUser->email);
				if($isRecip){
					$log->debug('User '.$userName.' IS already recipient of list '.$list->name .' [ID='.$mailingListId.']');
				}else{
					$log->debug('User '.$userName.' is NOT a recipient of list '.$list->name .' [ID='.$mailingListId.']');
				}
				if((($cbUser->$statusField == 1) && ($syncAction === 'subscribe_add') && !$isRecip)
				|| (($cbUser->$statusField != 1) && ($syncAction === 'unsubscribe_remove') && !$isRecip && $doSyncActionInReverse)){ // SUBSCRIBE TO MAILING LIST
					$logArray[] = JText::sprintf( 'COM_MAILSTER_SUBSCRIBED_USER_X_TO_MAILING_LIST_Y', $userName, $list->name);
					if($forceNoDoubleOptIn || ($list->subscribe_mode != MstConsts::SUBSCRIBE_MODE_DOUBLE_OPT_IN)){
						$log->debug('syncCBUserGroupsToMailsterGroups: Do not use Double Opt-in subscribe');
						$success = $subscrUtils->subscribeUser($cbUser->name, $cbUser->email, $mailingListId); // subscribing user...
						if(!$forceNoWelcomeGoodbyeMsg){
							$subscrUtils->sendWelcomeOrGoodbyeSubscriberMsg($cbUser->name, $cbUser->email, $mailingListId, MstConsts::SUB_TYPE_SUBSCRIBE);
						}
						// ####### TRIGGER NEW EVENT #######
						$mstEvents = MstFactory::getEvents();
						$mstEvents->userSubscribedOnWebsite($cbUser->name, $cbUser->email, $mailingListId);
						// #################################
					}else{
						$log->debug('syncCBUserGroupsToMailsterGroups: Double Opt-in subscribe mode');
						$subscrUtils->subscribeUserWithDoubleOptIn($cbUser->email, $cbUser->name, $mailingListId, 0);
					}
				}elseif((($cbUser->$statusField == 1) && ($syncAction === 'unsubscribe_remove') && $isRecip)
					 || (($cbUser->$statusField != 1) && ($syncAction === 'subscribe_add') && $isRecip && $doSyncActionInReverse)){ // UNSUBSCRIBE FROM MAILING LIST
					$logArray[] = JText::sprintf( 'COM_MAILSTER_UNSBSCRIBED_USER_X_FROM_MAILING_LIST_Y', $userName, $list->name);
					if($forceNoDoubleOptIn || ($list->unsubscribe_mode != MstConsts::UNSUBSCRIBE_MODE_DOUBLE_OPT_IN)){
						$log->debug('syncCBUserGroupsToMailsterGroups: Do not use Double Opt-in unsubscribe');
						$success = $subscrUtils->unsubscribeUser($cbUser->email, $mailingListId); // unsubscribing user...
						$tmpUser = $subscrUtils->getUserByEmail($cbUser->email);
						if(!$forceNoWelcomeGoodbyeMsg){
							$subscrUtils->sendWelcomeOrGoodbyeSubscriberMsg($tmpUser['name'], $cbUser->email, $mailingListId, MstConsts::SUB_TYPE_UNSUBSCRIBE);
						}
						// ####### TRIGGER NEW EVENT #######
						$mstEvents = MstFactory::getEvents();
						$mstEvents->userUnsubscribedOnWebsite($cbUser->email, $mailingListId);
						// #################################
					}else{
						$log->debug('syncCBUserGroupsToMailsterGroups: Double Opt-in unsubscribe mode');
						$subscrUtils->unsubscribeUserWithDoubleOptIn($cbUser->email, $mailingListId);
					}
				}else{
					$log->debug('No subscribe/add or unsubscribe/remove action necessary for '.$userName.' as either already (when adding) or not (when removing) recipient of list');
				}
			}else{
				$log->debug('syncCBUserGroupsToMailsterGroups: Mailing list ID '.$mailingListId.' not existing (anymore), exit');
			}
			
			if($userGroupId){
				$log->debug('syncCBUserGroupsToMailsterGroups: "'.$statusField.'" status is '.$cbUser->$statusField.' execute "'.$syncAction.'" action for user group '.$userGroupId);
				$groupModel = MstFactory::getModel('group');
				$groupModel->setId($userGroupId);
				$group = $groupModel->getData();
				if($group){
					$userGroupsModel = MstFactory::getModel('usergroups');
					$groupIds = array();
					$groupIds[] = $userGroupId;
					$isGroupMember = $userGroupsModel->isUserInGroup($userId, 1, $userGroupId);
					if($isGroupMember){
						$log->debug('User '.$userName.' IS already member of group '.$group->name .' [ID='.$userGroupId.']');
					}else{
						$log->debug('User '.$userName.' is NOT a member of group '.$group->name .' [ID='.$userGroupId.']');
					}
					if((($cbUser->$statusField == 1) && ($syncAction === 'subscribe_add') && !$isGroupMember)
					|| (($cbUser->$statusField != 1) && ($syncAction === 'unsubscribe_remove') && !$isGroupMember && $doSyncActionInReverse)){ // ADD TO USER GROUP
						$logArray[] = JText::sprintf( 'COM_MAILSTER_ADDED_USER_X_TO_GROUP_Y', $userName, $group->name );
						$userGroupsModel->store($groupIds, $userId, 1);
					}elseif((($cbUser->$statusField == 1) && ($syncAction === 'unsubscribe_remove') && $isGroupMember)
						 || (($cbUser->$statusField != 1) && ($syncAction === 'subscribe_add') && $isGroupMember && $doSyncActionInReverse)){ // REMOVE FROM USER GROUP
						$logArray[] = JText::sprintf( 'COM_MAILSTER_REMOVED_USER_X_FROM_GROUP_Y', $userName, $group->name );
						$userGroupsModel->delete($groupIds, $userId, 1);
					}else{
						$log->debug('No subscribe/add or unsubscribe/remove action necessary for '.$userName.' as either already (when adding) or not (when removing) user of group');
					}
				}else{
					$log->debug('syncCBUserGroupsToMailsterGroups: User group ID '.$userGroupId.' not existing (anymore), exit');
				}
			}
		}else{
			$log->debug('syncCBUserGroupsToMailsterGroups: Status of '.$statusField.' of user '.$userId.' is '.$cbUser->$statusField.', skip...');
		}
		return $logArray;
	}

	public static function syncCBUserGroupsToMailsterGroupsForAllUsers($fieldName){
		$log = MstFactory::getLogger();
		$logArray = array();
		$db = JFactory::getDBO();
		$query = 'SELECT id'
		. ' FROM #__comprofiler WHERE 1';
		$db->setQuery($query);
		$userIdObjs = $db->loadObjectList();
		$logArray[] = JText::_( 'COM_MAILSTER_INITIAL_SYNCHRONIZATION' );
		$logArray[] = JText::sprintf( 'COM_MAILSTER_X_USERS_WILL_BE_SYNCHRONIZED', count($userIdObjs));
		for($i=0;$i<count($userIdObjs);$i++){
			$logArray = array_merge($logArray, self::syncCBUserGroupsToMailsterGroups($userIdObjs[$i]->id, $fieldName, null, true));
		}
		return $logArray;
	}

	public static function syncCBUserGroupsToMailsterGroups($userId, $fieldName, $userGroups=null, $retLogArray=false){
		$log = MstFactory::getLogger();
		$groupModel = MstFactory::getModel('group');
		$userGroupsModel = MstFactory::getModel('usergroups');
		$logArray = array();

		$log->debug('syncCBUserGroupsToMailsterGroups called for user ID '.$userId. ', fieldName '.$fieldName.' and userGroups '.print_r($userGroups, true));

		$fieldId = self::getCBFieldId($fieldName);
		$fieldObj = self::getCBField($fieldId);

		if(!$fieldId || !$fieldObj){
			$log->warning('syncCBUserGroupsToMailsterGroups: Field configured does not exist in CB: ' . $fieldName);
		}else{
			$log->debug('syncCBUserGroupsToMailsterGroups: Field "'.$fieldName.'" has CB fieldid: ' . $fieldObj->fieldid);
		}

		$allCBUserGroups = self::getCBFieldValues($fieldId);
		$log->debug('syncCBUserGroupsToMailsterGroups allCBUserGroups: '.print_r($allCBUserGroups, true));

		$allLinkedUserGroupIds = self::getLinkedUserGroupIds($fieldId);

		if(is_null($userGroups)){
			$userGroups = self::getCBUserProfileGroups($userId, $fieldName);
		}

		// Remove user from all groups existing in CB and Mailster
		$log->debug('Remove user from the following groups (all groups existing in CB and Mailster): '.print_r($allLinkedUserGroupIds, true));
		$userGroupsModel->delete($allLinkedUserGroupIds, $userId, 1);
		$log->debug('User groups to add in Mailster: '.print_r($userGroups, true));

		$groupIds = array();
		if(!empty($userGroups)){
			foreach($userGroups as $groupName){
				if($groupName){
					$group = $groupModel->getGroupByName($groupName);
					if(!is_null($group) && $group->id > 0){
						$log->debug('User group to add: ' . $groupName . ', corresponds to: ' . print_r($group, true));
						$groupIds[] = $group->id;
					}else{
						$log->debug('No Mailster user group found for: ' . $groupName);
					}
				}
			}
			if(count($groupIds)){
				// Add user to all Mailster groups he is linked to in CB groups
				$log->debug('Add user to the following groups: '.print_r($groupIds, true));
				ob_start(); // activate output buffering
				$userGroupsModel->store($groupIds, $userId, 1);
				$noticeWarningsOutput = ob_get_contents();
				if (ob_get_level()) {
					ob_end_clean();  // deactivate output buffering
				}
				if($retLogArray){
					$logArray[] = JText::sprintf( 'COM_MAILSTER_ADDED_USER_X_TO_X_GROUPS', $userId, count($groupIds) );
				}
			}else{
				$log->debug('No Mailster Groups to add for user '.$userId);
			}
		}
		if($retLogArray){
			return $logArray;
		}
	}

	public static function getCBUserProfileGroups($userId, $fieldName){
		$log = MstFactory::getLogger();
		$fieldId = self::getCBFieldId($fieldName);
		$field = self::getCBField($fieldId);
		$tbl = $field->table;

		$db = JFactory::getDBO();
		$query = 'SELECT *'
		. ' FROM ' . $tbl
		. ' WHERE id = \''.$userId.'\'';
		$db->setQuery($query);
		$user = $db->loadObject();
		if($user){
			$log->debug('User '.$userId.' found');
			$groupsStr = $user->$fieldName;
			$log->debug('User field '.$fieldName.' has content: '.$groupsStr);
			$userGroups = explode('|*|', $groupsStr);
			return $userGroups;
		}
		$log->debug('User profile of user '.$userId.' not found');
		return null;
	}

	public static function setCBUserProfileGroups($userId, $fieldName, $userGroups){
		$log = MstFactory::getLogger();
		$fieldId = self::getCBFieldId($fieldName);
		$field = self::getCBField($fieldId);
		$tbl = $field->table;
		$groupsStr = implode('|*|', $userGroups);
		$db = JFactory::getDBO();
		$query = 'UPDATE '.$tbl.' SET '.$fieldName.' = ' . $db->quote($groupsStr) . ' WHERE id = \''.$userId.'\'';
		$db->setQuery($query);
		$errorMsg = '';
        try {
            $result = $db->query();
        } catch (DatabaseException $e) {
            $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
        }
		if(!$result){
			$log->error('Updating of CB user groups for user ' . $userId . ' failed: ' . $errorMsg);
			return false; // something went wrong
		}
		$log->debug('User profile of user '.$userId.' updated successfully');
		return true;
	}

	public static function addGroupInCBUser($userId, $groupId){
		$log = MstFactory::getLogger();
		$plgUtils = MstFactory::getPluginUtils();
		if(self::isCBInstalled()){
			$log->debug('CB is installed');
			if($plgUtils->isCBPluginInstalled() && $plgUtils->isCBPluginActive()){
				$log->debug('CB Plugin installed and active');
				if($plgUtils->isCBMstUserGroupSyncActive()){
					$log->debug('CB-Mailster User Group Sync is active');
					$cbPlgParams = $plgUtils->getCBPluginParameter();
					$groupModel = MstFactory::getModel('group');
					$groupModel->setId($groupId);
					$group = $groupModel->getData();
					$cbUserGroupFieldId = $cbPlgParams->def( 'cb_user_group_field', 0 );
					if($cbUserGroupFieldId > 0){
						$cbUserGroupField = self::getCBField($cbUserGroupFieldId);
						self::addGroup2CBUserProfile($userId, $cbUserGroupField->name, $group->name);
					}
				}else{
					$log->debug('CB-Mailster User Group Sync not active');
				}
			}else{
				$log->debug('CB Plugin not installed or not active');
			}
		}else{
			$log->debug('CB NOT is installed, Sync not possible');
		}
	}

	public static function removeGroupFromCBUser($userId, $groupId){
		$log = MstFactory::getLogger();
		$plgUtils = MstFactory::getPluginUtils();
		if(self::isCBInstalled()){
			$log->debug('CB is installed');
			if($plgUtils->isCBPluginInstalled() && $plgUtils->isCBPluginActive()){
				$log->debug('CB Plugin installed and active');
				if($plgUtils->isCBMstUserGroupSyncActive()){
					$log->debug('CB-Mailster User Group Sync is active');
					$cbPlgParams = $plgUtils->getCBPluginParameter();
					$groupModel = MstFactory::getModel('group');
					$groupModel->setId($groupId);
					$group = $groupModel->getData();
					$cbUserGroupFieldId = $cbPlgParams->def( 'cb_user_group_field', 0 );
					if($cbUserGroupFieldId > 0){
						$cbUserGroupField = self::getCBField($cbUserGroupFieldId);
						self::removeGroupFromCBUserProfile($userId, $cbUserGroupField->name, $group->name);
					}
				}else{
					$log->debug('CB-Mailster User Group Sync not active');
				}
			}else{
				$log->debug('CB Plugin not installed or not active');
			}
		}else{
			$log->debug('CB NOT is installed, Sync not possible');
		}
	}

	private static function addGroup2CBUserProfile($userId, $fieldName, $groupName){
		$log = MstFactory::getLogger();

		$userGroups = self::getCBUserProfileGroups($userId, $fieldName);
		if($userGroups){
			$log->debug('User groups in DB: '.print_r($userGroups, true));
			if(!empty($userGroups)){
				foreach($userGroups as $group){
					if(trim(strtolower($groupName)) === trim(strtolower($group))){
						$log->debug('Group '.$groupName.' already in DB stored');
						// found group, no need to add
						return true; // exit without changing user profile
					}
				}
			}else{
				$userGroups = array();
			}
			$log->debug('Group '.$groupName.' not yet in DB, will be added...');
			$userGroups[] = $groupName; // append new Group
			return self::setCBUserProfileGroups($userId, $fieldName, $userGroups);
		}
		return false; // user not found
	}

	private static function removeGroupFromCBUserProfile($userId, $fieldName, $groupName){
		$log = MstFactory::getLogger();

		$userGroups = self::getCBUserProfileGroups($userId, $fieldName);
		if($userGroups){
			$log->debug('User groups in DB: '.print_r($userGroups, true));
			if(!empty($userGroups)){
				for($i=0;$i<count($userGroups);$i++){
					$group = $userGroups[$i];
					if(trim(strtolower($groupName)) === trim(strtolower($group))){
						$log->debug('Group '.$groupName.' found in CB DB, will be deleted');
						unset($userGroups[$i]);
					}
				}
				$userGroups = array_values($userGroups); // re-index
			}
			$log->debug('User groups that will be stored in DB: '.print_r($userGroups, true));
			return self::setCBUserProfileGroups($userId, $fieldName, $userGroups);
		}
		return false; // user not found
	}

}
