<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	 
	defined('_JEXEC') or die('Restricted access');

	jimport('joomla.application.component.model');

	/**
	 * Digest Model
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterModelDigest extends MailsterModel
	{
		var $_data = null;
		var $_total = null;
		var $_pagination = null;

		function __construct(){
			parent::__construct();
			
			$app = JFactory::getApplication();
			$limit = $app->getUserStateFromRequest('global.list.limit', 'limit', $app->getCfg('list_limit'));
			$limitstart = $app->getUserStateFromRequest(JRequest::getCmd('option','com_mailster').'limitstart','limitstart',0);
				
			$limit = $limit + 0; // make it numerical
			$limitstart = $limitstart + 0; // make it numerical
				
			if($limit == 0){
				$limitstart = 0;
			}
	
			$this->setState('limit',$limit);
			$this->setState('limitstart',$limitstart);
		}
		
		function getDigestChoiceHtml($digestFreq = 0, $htmlId='digest_freq'){
			$digestOpts = array (
					JHTML::_('select.option', MstConsts::DIGEST_NO_DIGEST, 	JText::_( 'COM_MAILSTER_DIGEST_NO_DIGEST' )),
					JHTML::_('select.option', MstConsts::DIGEST_DAILY, 		JText::_( 'COM_MAILSTER_DIGEST_DAILY' )),
					JHTML::_('select.option', MstConsts::DIGEST_WEEKLY, 	JText::_( 'COM_MAILSTER_DIGEST_WEEKLY' )),
					JHTML::_('select.option', MstConsts::DIGEST_MONTHLY, 	JText::_( 'COM_MAILSTER_DIGEST_MONTHLY' )) );
			$digestChoice = JHTML::_('select.genericlist', $digestOpts, $htmlId, 'class="inputbox" size="1" style="width:200px;"', 'value', 'text', $digestFreq);
			return $digestChoice;
		}
		
		function getDigestStr($digestFreq = 0){
			switch($digestFreq){
				case MstConsts::DIGEST_NO_DIGEST:
					return JText::_( 'COM_MAILSTER_DIGEST_NO_DIGEST' );
					break;
				case MstConsts::DIGEST_DAILY:
					return JText::_( 'COM_MAILSTER_DIGEST_DAILY' );
					break;
				case MstConsts::DIGEST_WEEKLY:
					return JText::_( 'COM_MAILSTER_DIGEST_WEEKLY' );
					break;
				case MstConsts::DIGEST_MONTHLY:
					return JText::_( 'COM_MAILSTER_DIGEST_MONTHLY' );
					break;
			}
		}
		
		function getDigestSummaryStr($digestFreq = 0){
			switch($digestFreq){
				case MstConsts::DIGEST_NO_DIGEST:
					return JText::_( 'COM_MAILSTER_DIGEST_NO_DIGEST' );
					break;
				case MstConsts::DIGEST_DAILY:
					return JText::_( 'COM_MAILSTER_DIGEST_TODAYS_TOPIC_SUMMARY' );
					break;
				case MstConsts::DIGEST_WEEKLY:
					return JText::_( 'COM_MAILSTER_DIGEST_LAST_WEEKS_TOPIC_SUMMARY' );
					break;
				case MstConsts::DIGEST_MONTHLY:
					return JText::_( 'COM_MAILSTER_DIGEST_LAST_MONTHS_TOPIC_SUMMARY' );
					break;
			}
		}
		
		function isUserDigestRecipientOfList($userId, $isJoomlaUser, $listId){
			$log = MstFactory::getLogger();
			$log->debug('isUserDigestRecipientOfList(): '.$userId.', '.$isJoomlaUser.', '.$listId);
			$digests = $this->getDigestsOfUser($userId, $isJoomlaUser, $listId);
			if($digests && count($digests) > 0){
				for($i=0; $i<count($digests); $i++){
					if($digests[$i]->digest_freq > 0){
						return true;
					}
				}
			}
			return false;
		}
		
		function getDigest($digestId){
			$log = MstFactory::getLogger();
			if($digestId > 0){
				$log->debug('getDigest(): '.$digestId);
				$digests = $this->getData(true, null, null, null, $digestId);
				if(count($digests)>0){
					return $digests[0];
				}else{
					return null;
				}
			}
			return null;
		}
		
		function getDigestsOfUser($userId, $isJoomlaUser, $listId=null){
			$log = MstFactory::getLogger();
			$digests = $this->getData(true, $userId, $isJoomlaUser, $listId, null);
            if(count($digests)){
                $log->debug('getDigestsOfUser() digests found for user ID: '.$userId.', joomla user: '.$isJoomlaUser.', list ID: '.$listId.': '.print_r($digests, true));
            }else{
                $log->debug('getDigestsOfUser() NO digests found for user ID: '.$userId.', joomla user: '.$isJoomlaUser.', list ID: '.$listId);
            }
			return $digests;
		}
		
		function getDataExcludingDigest2ArticleDigests(){
			return $this->getData(false, null, null, null, null, true);
		}
				
		function getData($overrideLimits=false, $userId=null, $isJoomlaUser=null, $listId=null, $digestId=null, $hideDigest2ArticleDigests=false)
		{
			$log = MstFactory::getLogger();
			$log->debug('DigestModel::getData [override: '.$overrideLimits.', userId: '.$userId.', isJoomlaUser: '.$isJoomlaUser.', listId: '.$listId.', digestId: '.$digestId.']');
			$query = $this->_buildQuery($userId, $isJoomlaUser, $listId, $digestId, $hideDigest2ArticleDigests);
			$log = MstFactory::getLogger();
			//$log->debug('Query_in_digest_model: '.$query);
			$limitstart = $this->getState('limitstart');
			$limit = $this->getState('limit');

			if(!$overrideLimits){
				$this->_data = $this->_getList($query, $limitstart, $limit);
			}else{
				$this->_data = $this->_getList($query, 0, 0);
			}			
			for($i=0, $n=count($this->_data); $i<$n; $i++){
				$this->_data[$i]->digestFreqStr = $this->getDigestStr($this->_data[$i]->digest_freq);
				$this->_data[$i]->summaryStr = $this->getDigestSummaryStr($this->_data[$i]->digest_freq);
			}
			return $this->_data;	
		}
			
		function getPagination()
		{
			if (empty($this->_pagination))
			{
				jimport('joomla.html.pagination');
				$this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
			}

			return $this->_pagination;
		}
		
		function getTotal()
		{
			if (empty($this->_total)){
				$query = $this->_buildQuery();
				$this->_total = $this->_getListCount($query);
			}
			return $this->_total;
		}
		
		public function getTable($type = 'mailster_digests', $prefix = '', $config = array()){
            JTable::addIncludePath(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster'.DS.'tables');
			return JTable::getInstance($type, $prefix, $config);
		}

		function _buildQuery($userId=null, $isJoomlaUser=null, $listId=null, $digestId=null, $hideDigest2ArticleDigests=false)
		{	
			$where		= $this->_buildContentWhere($userId, $isJoomlaUser, $listId, $digestId, $hideDigest2ArticleDigests);
			$orderby	= $this->_buildContentOrderBy();
						
			$query = 'SELECT d.*';
			$query = $query	. ' FROM #__mailster_digests d'
							. $where
							. $orderby;	
						
			return $query;
		}

		function _buildContentOrderBy()
		{
			$orderby 	= ' ORDER BY d.user_id, d.is_joomla_user';
			return $orderby;
		}

		function _buildContentWhere($userId, $isJoomlaUser, $listId, $digestId, $hideDigest2ArticleDigests)
		{	
			$where = array();
			if(!is_null($userId)){
				$where[] = 'd.user_id = \''.$userId.'\'';
			}
			if(!is_null($isJoomlaUser)){
				$where[] = 'd.is_joomla_user = \''.$isJoomlaUser.'\'';
			}
			if(!is_null($listId)){
				$where[] = 'd.list_id = \''.$listId.'\'';
			}
			if(!is_null($digestId)){
				$where[] = 'd.id = \''.$digestId.'\'';
			}
			if($hideDigest2ArticleDigests){
				$where[] = 'd.user_id > '.MstConsts::DIGEST_USER_ID_MEANING_DIGEST_TO_ARTICLE;
			}
			$where 		= ( count( $where ) ? ' WHERE ' . implode( ' AND ', $where ) : '' );	
			return $where;
		}
		
		function updateSendingDate($digestId){
			$log = MstFactory::getLogger();
			$log->debug('updateSendingDate for digest ID '.$digestId);
			$dbUtils = MstFactory::getDBUtils();
			$convUtils = MstFactory::getConverterUtils();
			$currDigest = $this->getDigest($digestId);
			if($currDigest){
				$log->debug('updateSendingDate digest before updating: '.print_r($currDigest, true));
				$currDigest->last_send_date = $dbUtils->getDateTimeNow();
				$currDigest->next_send_date = $this->getNextSendDate($digestId, $currDigest->digest_freq, true);
				$log->debug('updateSendingDate digest after updating: '.print_r($currDigest, true));
				$this->store($convUtils->object2Array($currDigest), false);
				return true;
			}
			return false;
		}
		
		function getNextSendDate($digestId, $digestFreq, $forceNewSendDate=false){
			$dbUtils = MstFactory::getDBUtils();
			$currDigest = $this->getDigest($digestId);
			$digestFreqChanged = $currDigest->digest_freq !== $digestFreq;
			if($digestId <= 0 || $digestFreqChanged || is_null($currDigest->next_send_date) || $forceNewSendDate){
				// new digest or digest frequency was changed
				switch($digestFreq){
					case MstConsts::DIGEST_NO_DIGEST:
						return 'NULL';
						break;
					case MstConsts::DIGEST_DAILY:
						return $dbUtils->getDateTimeTomorrowNextMidnight();
						break;
					case MstConsts::DIGEST_WEEKLY:
						return $dbUtils->getDateTimeNextSundayMidnight();
						break;
					case MstConsts::DIGEST_MONTHLY:
						return $dbUtils->getDateTimeThisMonthLastDayMidnight();
						break;
				}
			}else{
				// existing digest, digest frequency did not change -> no change necessary
				return $currDigest->next_send_date;
			}
		}
		
		function getDigest2ArticleDigests($listId){
			$data = array();
			$data['list_id'] = $listId;
			$data['user_id'] = MstConsts::DIGEST_USER_ID_MEANING_DIGEST_TO_ARTICLE; // means "non-user" digest --> which is an "archive 2 article" digest
			$data['is_joomla_user'] = 0;
			
			return $this->getData(true, $data['user_id'], $data['is_joomla_user'], $data['list_id']);
		}
		
		function storeDigest2ArticleDigest($listId, $autoUpdateSendDate=true){
			$log = MstFactory::getLogger();
			
			$data = array();			
			$data['list_id'] = $listId;
			$data['user_id'] = MstConsts::DIGEST_USER_ID_MEANING_DIGEST_TO_ARTICLE; // means "non-user" digest --> which is an "archive 2 article" digest
			$data['is_joomla_user'] = 0;			

			$existingDigests = $this->getDigest2ArticleDigests($listId);
			if(count($existingDigests)>0){ // already existing digest 2 article entry
				$log->debug('storeDigest2ArticleDigest: existing digests, take first of '.print_r($existingDigests, true));
				$data['id'] = $existingDigests[0]->id;
				$data['digest_freq'] = $existingDigests[0]->digest_freq;
				$data['last_send_date'] = $existingDigests[0]->last_send_date;
				$data['next_send_date'] = $existingDigests[0]->next_send_date;
			}else{
				$log->debug('storeDigest2ArticleDigest: current non existing article 2 digest, create one');
				$data['id'] = 0; // new entry
				$data['digest_freq'] = MstConsts::DIGEST_DAILY;
				$data['last_send_date'] = null;
				$data['next_send_date'] = null;
			}
						
			$log->debug('storeDigest2ArticleDigest: '.print_r($data, true));
			return $this->store($data, $autoUpdateSendDate);
		}
		
		function removeDigest2ArticleDigestIfExisting($listId){
			$log = MstFactory::getLogger();			
			$existingDigests = $existingDigests = $this->getDigest2ArticleDigests($listId);
			if(count($existingDigests)>0){ // already existing digest 2 article entry
				$log->debug('removeDigest2ArticleDigestIfExisting: existing digests, delete them: '.print_r($existingDigests, true));
				foreach($existingDigests AS $existingDigest){
					$log->debug('removeDigest2ArticleDigestIfExisting: delete digest ID '.$existingDigest->id);
					$this->delete(array($existingDigest->id));
				}
			}else{
				$log->debug('removeDigest2ArticleDigestIfExisting: non existing, nothing to delete');				
			}			
		}
		
		function store($data, $autoUpdateSendDate=true)
		{
			$log = MstFactory::getLogger();
            $row  =& $this->getTable();
            //$log->debug('digests model -> store -> table: '.print_r($row, true));
			
			if($autoUpdateSendDate){
				$data['next_send_date'] = $this->getNextSendDate($data['id'], $data['digest_freq']);
			}
							
			if (!$row->bind($data)) {
				JError::raiseError(500, $this->_db->getErrorMsg() );
				return false;
			}
		
			$row->id = (int) $row->id;
		
			if (!$row->check()) {
				$this->setError($row->getError());
				return false;
			}
		
			if (!$row->store()) {
				JError::raiseError(500, $this->_db->getErrorMsg() );
				return false;
			}
		
			return $row->id;
		}
		
		function delete($cid = array())
		{
			$result = false;

			if (count( $cid )){
				
				for($i=0;$i<count($cid);$i++){
					$digestId = $cid[$i];
					
					$query = 'DELETE FROM #__mailster_digest_queue'
							. ' WHERE digest_id = \'' . $digestId . '\'';
					$this->_db->setQuery( $query );
					$errorMsg = '';
                    try {
                        $result = $this->_db->query();
                    } catch (DatabaseException $e) {
                        $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                    }
					if(!$result) {
						$this->setError($errorMsg);
						return false;
					}

					$query = 'DELETE FROM #__mailster_digests'
							. ' WHERE id = \'' . $digestId . '\'';
					$this->_db->setQuery( $query );
					$errorMsg = '';
                    try {
                        $result = $this->_db->query();
                    } catch (DatabaseException $e) {
                        $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                    }
					if(!$result) {
						$this->setError($errorMsg);
						return false;
					}
				}
			}

			return true;
		}

        function deleteDigestQueueEntriesOfMails($mailIds)
        {
            $result = false;

            if (count($mailIds)){
                $mailIdsStr = implode( ',', $mailIds);
                $query = 'DELETE FROM #__mailster_digest_queue'
                    . ' WHERE mail_id IN ('. $mailIdsStr .')';
                $this->_db->setQuery( $query );
                $errorMsg = '';
                try {
                    $result = $this->_db->query();
                } catch (DatabaseException $e) {
                    $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                }
                if(!$result) {
                    $this->setError($errorMsg);
                    return false;
                }
            }

            return true;
        }
		
	}//Class end
?>
