<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	 
	defined('_JEXEC') or die('Restricted access');

	jimport('joomla.application.component.model');

	/**
	 * Mailing List Groups Model
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterModelGroups extends MailsterModel
	{
		var $_data = null;
		var $_total = null;
		var $_pagination = null;
		
		function __construct()
		{
			parent::__construct();
		}

		function getData($indexedByGroupId = false)
		{
			if (empty($this->_data))
			{
			 	$query = $this->_buildQuery();
				$data = $this->_getList($query, 0, 0);
				$data = $this->_additionals($data);
				
				if($indexedByGroupId){
					$groups = array();
					for($i=0;$i<count($data);$i++){
						$group = $data[$i];
						$groups[$group->id] = $group;
					}
					$data = $groups;
				}
				$this->_data = $data;				
			}
			
			return $this->_data;
		}
		
		function getGroupsExpectProvidedGroupIds($groupIds){
			$groupModel = MstFactory::getModel('group');
			$where = ( count( $groupIds ) ? ' WHERE id NOT IN ('.implode( ', ', $groupIds ).')' : '' );
			$query = 'SELECT *'
			. ' FROM #__mailster_groups'
			. $where;
			$rawGroups = $this->_getList($query, 0, 0);
			$groups = array();
			foreach($rawGroups AS $rawGroup){
				$groupModel->setId($rawGroup->id);
				$groups[] = $groupModel->getData();
			}
			return $groups;
		}

		function getTotal()
		{
			if (empty($this->_total))
			{
				$query = $this->_buildQuery();
				$this->_total = $this->_getListCount($query);
			}

			return $this->_total;
		}

		function getPagination()
		{
			if (empty($this->_pagination)){
				jimport('joomla.html.pagination');
				$this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
			}
			return $this->_pagination;
		}

        function getMailsterGroupsUsingJoomlaGroups(){
            $query = 'SELECT * FROM #__mailster_groups'
                    . ' WHERE is_joomla_group=\'1\'';
            $mstGroupsUsingJGroups = $this->_getList($query, 0, 0);
            return $mstGroupsUsingJGroups;
        }
		
		function getNonUsedJoomlaGroups(){
			$query = 'SELECT id, title AS name FROM #__usergroups'
					. ' WHERE id NOT IN ('
						. ' SELECT joomla_group_id FROM #__mailster_groups'
						. ' WHERE is_joomla_group=\'1\''
					. ') ORDER BY name';
			$nonUsedJoomlaGroups = $this->_getList($query, 0, 0);
			return $nonUsedJoomlaGroups;
		}

		function _buildQuery()
		{

            $query = '	SELECT g.id, g.name, is_joomla_group, joomla_group_id, Count(gu.group_id) as memberCount FROM #__mailster_groups AS g
                            LEFT JOIN #__mailster_group_users AS gu
                            ON (gu.group_id = g.id)  WHERE g.is_joomla_group=\'0\' GROUP BY g.id
                         UNION
                            SELECT g.id, jug.title AS name, is_joomla_group, joomla_group_id, Count(uum.group_id) as memberCount FROM #__mailster_groups AS g
                            LEFT JOIN #__user_usergroup_map AS uum
                            ON (g.joomla_group_id = uum.group_id)
                            LEFT JOIN #__usergroups AS jug
                            ON (g.joomla_group_id = jug.id) WHERE g.is_joomla_group=\'1\' GROUP BY g.id
                        ORDER by name';
			
			return $query;
		}

	
		function _additionals($rows)
		{
			/* nothing for now */
			return $rows;
		}

		
		/**
		 * Method to remove a group from the list
		 */
		function delete($cid = array())
		{
			$result = false;

			if (count( $cid ))
			{
				$cids = implode( ',', $cid );
				$query = 'DELETE FROM #__mailster_groups'
						. ' WHERE id IN ('. $cids .')';

				$this->_db->setQuery( $query );
				$errorMsg = '';
                try {
                    $result = $this->_db->query();
                } catch (DatabaseException $e) {
                    $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                }
				if(!$result) {
					$this->setError($errorMsg);
					return false;
				}
				
				$query = 'DELETE FROM #__mailster_group_users'
						. ' WHERE group_id IN ('. $cids .')';

				$this->_db->setQuery( $query );
				$errorMsg = '';
                try {
                    $result = $this->_db->query();
                } catch (DatabaseException $e) {
                    $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                }
				if(!$result) {
					$this->setError($errorMsg);
					return false;
				}
				
				$query = 'DELETE FROM #__mailster_list_groups'
						. ' WHERE group_id IN ('. $cids .')';

				$this->_db->setQuery( $query );
				$errorMsg = '';
                try {
                    $result = $this->_db->query();
                } catch (DatabaseException $e) {
                    $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                }
				if(!$result) {
					$this->setError($errorMsg);
					return false;
				}
			}

			return true;
		}
	}//Class end
?>
