<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	 
	defined('_JEXEC') or die('Restricted access');

	jimport('joomla.application.component.model');

	/**
	 * Mailing List Group Users Model
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterModelGroupUsers extends MailsterModel
	{
		var $_data = null;
		var $_groupData = null;
		var $_nonMemberData = null;
		var $_pagination = null;
		var $_groupID = null;

		function __construct(){
			parent::__construct();
		}

		function getData($groupID)
		{
			$log = MstFactory::getLogger();
			$this->_groupID = $groupID;
			if (empty($this->_data))
			{
				$query = $this->_buildQuery();
				$this->_data = $this->_getList($query, 0, 0);
				//$log->debug('groupusers->getData() query: '.$query);
			}
			return $this->_data;			
		}		
		
		function getAllGroupsUsersOfJoomlaUserData()
		{
			$orderby = ' ORDER BY gu.user_id';
			$where = ' WHERE gu.is_joomla_user = \'1\'';			
			$query = $this->_buildSelectClause();
			$query = $query	. $where . $orderby;			
			return $this->_getList($query, 0, 0);
		}

		/**
		 * Method to get general group data
		 */
		function getGroupData($groupID)
		{
			$this->_groupID = $groupID;
			if (empty($this->_groupData))
			{
				$query = $this->_buildGroupQuery();
				$this->_groupData = $this->_getList($query, 0, 0);
			}
			return $this->_groupData;
			
		}
		
		/**
		 * Method to get all users that are not not in the group
		 */
		function getNonMemberData($groupID)
		{
			$this->_groupID = $groupID;
			if (empty($this->_nonMemberData))
			{
				$query = $this->_buildNonMemberQuery();
				$this->_nonMemberData = $this->_getList($query, 0, 0);
			}
			return $this->_nonMemberData;
			
		}

		function getPagination()
		{
			if (empty($this->_pagination))
			{
				jimport('joomla.html.pagination');
				$this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
			}

			return $this->_pagination;
		}
		
		
		/**
		 * Build the Non Member Data query
		 */
		function _buildNonMemberQuery()
		{				
			$query = 'SELECT u.id, u.name, u.email, u.notes, FORMAT(0,0) AS is_joomla_user'
				. ' FROM #__mailster_users u'
				. ' WHERE u.id NOT IN'
				. ' 	(SELECT gu.user_id'
				. ' 	FROM #__mailster_group_users gu'
				. ' 	WHERE gu.group_id = \'' . $this->_groupID . '\''
				. ' 	AND gu.is_joomla_user = \'0\' )';
			$query = $query
				. ' UNION'
				. ' SELECT ju.id, ju.name, ju.email, " ", FORMAT(1,0)'
				. ' FROM #__users ju'
				. ' WHERE ju.id NOT IN'
				. '		(SELECT gu.user_id'
				. ' 	FROM #__mailster_group_users gu'
				. ' 	WHERE gu.group_id = \'' . $this->_groupID . '\''
				. '		AND gu.is_joomla_user = \'1\' )'
				. ' ORDER BY name, email';
			return $query;
		}
		
		/**
		 * Build the Group Data query
		 */
		function _buildGroupQuery()
		{				
			$query = 'SELECT g.*'
						. ' FROM #__mailster_groups g '
						. ' WHERE g.id=\'' . $this->_groupID . '\'';
						;	
			return $query;
		}

		function _buildQuery()
		{
			$groupModel = MstFactory::getModel('group');
			$groupModel->setId($this->_groupID);
			$group = $groupModel->getData();

			if($group->is_joomla_group > 0){ // Joomla 2.5 and 3.0+
				$query = 'SELECT '.$this->_groupID.' AS group_id, ju.id AS user_id, \'1\' AS is_joomla_user, ju.name AS name, ju.email AS email, " " AS notes'
						. ' FROM #__users ju'
						. ' WHERE id IN ('
							. ' SELECT user_id'
							. ' FROM #__user_usergroup_map'
							. ' WHERE group_id IN ('
								. ' SELECT joomla_group_id'
								. ' FROM #__mailster_groups'
								. ' WHERE id = \'' . $this->_groupID . '\''
							. ' )'
						. ' ) ';
			}else{			
				$where		= $this->_buildContentWhere();
				$orderby	= $this->_buildContentOrderBy();
				
				$query = $this->_buildSelectClause();		
				$query = $query	. $where;
				$query = $query	. $orderby;
			}
			
			return $query;
		}
		
		function _buildSelectClause(){			
			$query = 'SELECT gu.*,'
					. ' CASE WHEN gu.is_joomla_user = \'1\''
					. ' THEN ju.name ELSE u.name END AS name,'
					. ' CASE WHEN gu.is_joomla_user = \'1\''
					. ' THEN ju.email ELSE u.email END AS email,'
					. ' CASE WHEN gu.is_joomla_user = \'1\''
					. ' THEN " " ELSE u.notes END AS notes'
					. ' FROM #__mailster_group_users gu '
					. ' LEFT JOIN #__mailster_users u ON (gu.user_id = u.id)'
					. ' LEFT JOIN #__users ju ON (gu.user_id = ju.id)';
			return $query;
		}

		function _buildContentOrderBy()
		{
			$orderby 	= ' ORDER BY name, email, gu.is_joomla_user';			
			return $orderby;
		}
			
		public function getTable($type = 'mailster_group_users', $prefix = '', $config = array()){
			JTable::addIncludePath(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster'.DS.'tables');
			return JTable::getInstance($type, $prefix, $config);
		}
		
		function store($data)
		{		
			$log		= MstFactory::getLogger();
			$user		= JFactory::getUser();
			$config 	= JFactory::getConfig();

            if(!property_exists($data, 'group_id')
                || !property_exists($data, 'user_id')
                || !property_exists($data, 'is_joomla_user')
                || $data->user_id <= 0
            ){
                $log->warning('groupusers->store() data not setup okay, will exit because: '.print_r($data, true));
                return false;
            }
						
			// Delete from table if existing
			$deleteSuccess = $this->delete($data->group_id, array($data->user_id), array($data->is_joomla_user));
						
			$row  = $this->getTable();
			
			if (!$row->bind($data)) {
				JError::raiseError(500, $this->_db->getErrorMsg() );
				return false;
			}

			if (!$row->check()) {
				$this->setError($row->getError());
				return false;
			}

			if (!$row->store()) {
				JError::raiseError(500, $this->_db->getErrorMsg() );
				return false;
			}
			return true;
		}
				
		/**
		 * Get all mailing lists that use this group as recipient group
		 */
		function getListsWithGroup($groupID)
		{			
			$query = 'SELECT l.*'
						. ' FROM #__mailster_lists l '
						. ' WHERE l.id IN ('
						. '		SELECT lg.list_id'
						. '		FROM #__mailster_list_groups lg '
						. '		WHERE lg.group_id=\'' . $groupID. '\''
						. ' )';	
			$lists = $this->_getList($query, 0, 0);
			return $lists;
		}
		
		function _buildContentWhere()
		{	
			$where = array();			
			$where[] = ' gu.group_id = \'' . $this->_groupID . '\'';
			$where 		= ( count( $where ) ? ' WHERE ' . implode( ' AND ', $where ) : '' );			
			return $where;
		}
		

		/**
		 * Method to remove Users from the group
		 */
		function delete($group_id, $user_ids = array(), $is_joomla_user_flags = array())
		{
			$result = false;
			$userCount = count( $user_ids );
			if ($userCount)
			{
				for($i=0; $i < $userCount; $i++)
				{
					$query = 'DELETE FROM #__mailster_group_users'
							. ' WHERE user_id =\''. $user_ids[$i] . '\''
							. ' AND group_id =\''. $group_id . '\''
							. ' AND is_joomla_user =\''. $is_joomla_user_flags[$i] . '\'';
					$this->_db->setQuery( $query );

					$errorMsg = '';
                    try {
                        $result = $this->_db->query();
                    } catch (DatabaseException $e) {
                        $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                    }
					if(!$result) {
						$this->setError($errorMsg);
						return false;
					}
				}
			}

			return true;
		}		
		
	}//Class end
?>
