<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	 
	defined('_JEXEC') or die('Restricted access');

	jimport('joomla.application.component.model');

	/**
	 * Log Entries Model
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterModelLog extends MailsterModel
	{
		var $_data = null;
		var $_total = null;
		var $_pagination = null;

		function __construct(){
			parent::__construct();
			$app = JFactory::getApplication();
			$limit = $app->getUserStateFromRequest('global.list.limit', 'limit', $app->getCfg('list_limit'));
			$limitstart = $app->getUserStateFromRequest(JRequest::getCmd('option','com_e2a').'limitstart','limitstart',0);
				
			$limit = $limit + 0; // make it numerical
			$limitstart = $limitstart + 0; // make it numerical
				
			if($limit == 0){
				$limitstart = 0;
			}
	
			$this->setState('limit',$limit);
			$this->setState('limitstart',$limitstart);
		}
		
		function getData($overrideLimits=false)
		{
			if (empty($this->_data))
			{
				$query = $this->_buildQuery();
				$limitstart = $this->getState('limitstart');
				$limit = $this->getState('limit');
	
				if(!$overrideLimits){
					$this->_data = $this->_getList($query, $limitstart, $limit);
				}else{
					$this->_data = $this->_getList($query, 0, 0);
				}
			}
			return $this->_data;	
		}
			
		function getPagination()
		{
			if (empty($this->_pagination))
			{
				jimport('joomla.html.pagination');
				$this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
			}

			return $this->_pagination;
		}
		
		function getTotal()
		{
			if (empty($this->_total)){
				$query = $this->_buildQuery();
				$this->_total = $this->_getListCount($query);
			}
			return $this->_total;
		}

		function _buildQuery()
		{
			$where		= $this->_buildContentWhere();
			$orderby	= $this->_buildContentOrderBy();
			
			$query = 'SELECT l.*'
						. ' FROM #__mailster_log l'
						. $where
						. $orderby;						
			return $query;
		}

		function _buildContentOrderBy()
		{
			$orderby 	= ' ORDER BY l.log_time ASC, l.id ASC';
			return $orderby;
		}

		function _buildContentWhere()
		{	
			$where = array();
			$where 		= ( count( $where ) ? ' WHERE ' . implode( ' AND ', $where ) : '' );	
			return $where;
		}

		function clearLog()
		{
			$query = 'DELETE FROM #__mailster_log';
			$this->_db->setQuery( $query );
			$errorMsg = '';
            try {
                $result = $this->_db->query();
            } catch (DatabaseException $e) {
                $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
            }
			if(!$result) {
				$this->setError($errorMsg);
				return false;
			}
			return true;
		}
		
		function delete($cid = array())
		{	
			if (count( $cid )){
				
				for($i=0;$i<count($cid);$i++){
					$query = 'DELETE FROM #__mailster_log'
							. ' WHERE id = \'' . $cid[$i] . '\'';
					$this->_db->setQuery( $query );
					$errorMsg = '';
                    try {
                        $result = $this->_db->query();
                    } catch (DatabaseException $e) {
                        $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                    }
					if(!$result) {
						$this->setError($errorMsg);
						return false;
					}
				}
			}

			return true;
		}
	}//Class end
?>
