<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	defined('_JEXEC') or die('Restricted access');

	jimport('joomla.application.component.model');

	/**
	 * Mail Model
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterModelMail extends MailsterModel
	{
		var $_id = null;
		var $_data = null;
		var $_filterByFrontendMailArchiveACL = false;

		function __construct(){
			parent::__construct();
			$array = JRequest::getVar('cid',  0, '', 'array');
			$this->setId((int)$array[0]);
		}

		function setId($id)
		{
			$this->_id	    = $id;
			$this->_data	= null;
		}		

		function &getData($id=false, $filterByFrontendMailArchiveACL=false)
		{
			if($id){
				$this->setId($id);
				$this->_filterByFrontendMailArchiveACL = $filterByFrontendMailArchiveACL;
			}
			if (!$this->_loadData()){
				$this->_initData();
			}
			return $this->_data;
		}

		function _loadData()
		{
			$log = MstFactory::getLogger();
			if (empty($this->_data))
			{
				$frontendACLFilterQueryPart = '';
				if($this->_filterByFrontendMailArchiveACL){
					$user = JFactory::getUser();
					$mListUtils = MstFactory::getMailingListUtils();
					$listIdsAllowedToBeShown = $mListUtils->getAllMailingListsUserCanAccessMailsInFrontend($user->id);
					if(count($listIdsAllowedToBeShown)){
						$frontendACLFilterQueryPart = ' AND list_id IN ('.implode(',', $listIdsAllowedToBeShown).')';
						$log->debug('mailmodel->_loadData: filterByFrontendMailArchiveACL add condition: "'.$frontendACLFilterQueryPart.'"');
					}
				}
				
				$query = 'SELECT *'
						. ' FROM #__mailster_mails'
						. ' WHERE id = '.$this->_id
						. $frontendACLFilterQueryPart
						;
				$this->_db->setQuery($query);
				$this->_data = $this->_db->loadObject();

				return (boolean) $this->_data;
			}
			return true;
		}
		
		function _initData()
		{
			if (empty($this->_data))
			{
				$mail = new stdClass();
				$mail->id						= 0;
				$mail->list_id					= null;
				$mail->thread_id				= null;
				$mail->hashkey					= null;
				$mail->message_id				= null;
				$mail->in_reply_to				= null;
				$mail->references_to			= null;
				$mail->receive_timestamp		= null;
				$mail->from_name				= null;
				$mail->from_email				= null;
                $mail->sender_user_id			= null;
                $mail->sender_is_joomla_user	= null;
				$mail->orig_to_recips			= null;
				$mail->orig_cc_recips			= null;
				$mail->subject					= null;
				$mail->body						= null;
				$mail->html						= null;
				$mail->has_attachments			= 0;
				$mail->attachments				= null;
				$mail->fwd_errors				= 0;
				$mail->fwd_completed			= 0;
				$mail->fwd_completed_timestamp	= 0;
				$mail->blocked_mail				= 0;
				$mail->bounced_mail				= 0;
				$mail->no_content				= 0;
				$mail->has_send_report			= 0;
				$mail->size_in_bytes			= -1;
							
				$this->_data		= $mail;
				return (boolean) $this->_data;
			}
			return true;
		}
			
		public function getTable($type = 'mailster_mails', $prefix = '', $config = array()){
			return JTable::getInstance($type, $prefix, $config);
		}

		function store($data)
		{		
			$user		= & JFactory::getUser();
			$config 	= & JFactory::getConfig();
			
			$row  =& $this->getTable();
			
			if (!$row->bind($data)) {
				JError::raiseError(500, $this->_db->getErrorMsg() );
				return false;
			}
			
			$row->id = (int) $row->id;

			if (!$row->check()) {
				$this->setError($row->getError());
				return false;
			}
			if (!$row->store()) {
				JError::raiseError(500, $this->_db->getErrorMsg() );
				return false;
			}
			return $row->id;
		}
		
		
	}
?>
