<?php
/**
 * @package Joomla
 * @subpackage Mailster
 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
 * @license GNU/GPL, see license.txt
 * Mailster is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.
 *
 * Mailster is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mailster; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * or see http://www.gnu.org/licenses/.
 */


defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

/**
 * Mailing List Mails Model
 *
 * @package Joomla
 * @subpackage Mailster
 */
class MailsterModelMails extends MailsterModel
{

	var $_id = 0;
	var $_data = null;
	var $_limit_offset = null;
	var $_total = null;
	var $_pagination = null;	
	var $_order_by = 'rfirst';	
	var $_filterByFrontendMailArchiveACL = false;
	var $_filterByCoreACL = false;

	function __construct(){
		parent::__construct();
		$app = JFactory::getApplication();
		$limit = $app->getUserStateFromRequest('global.list.limit', 'limit', $app->getCfg('list_limit'));
		$limitstart = $app->getUserStateFromRequest(JRequest::getCmd('option','com_mailster').'limitstart','limitstart',0);
			
		$limit = $limit + 0; // make it numerical
		$limitstart = $limitstart + 0; // make it numerical
			
		if($limit == 0){
			$limitstart = 0;
		}

		$this->setState('limit',$limit);
		$this->setState('limitstart',$limitstart);
	}

	function getData($listId, $overrideLimits=false, $orderBy='rfirst', $filterByFrontendMailArchiveACL=false, $filterByCoreACL=false)
	{
		if (empty($this->_data) || $this->_id != $listId){
			$this->_id = $listId;
			$this->_order_by = $orderBy;
			$this->_filterByFrontendMailArchiveACL = $filterByFrontendMailArchiveACL;
			$this->_filterByCoreACL = $filterByCoreACL;
			
			$query = $this->_buildQuery($listId);

			$limitstart = $this->getState('limitstart');
			$limit = $this->getState('limit');

			if(!$overrideLimits){
				$this->_data = $this->_getList($query, $limitstart, $limit);
				$this->_limit_offset = $limitstart;
			}else{
				$this->_data = $this->_getList($query, 0, 0);
				$this->_limit_offset = $limitstart;
			}
		}
		return $this->_data;
	}
	
	function getDataForBackendMailArchive($listId){
		return $this->getData($listId, false, 'rfirst', false, true); // filter by core ACL
	}

	/**
	 * Method to get blocked mails
	 */
	function getBlockedMailData($listId, $overrideLimits=false, $filterByCoreACL=false)
	{
		if($this->_id != $listId){
			$this->_id = $listId;
			$this->_data = null;
			$this->_filterByCoreACL = $filterByCoreACL;
		}
			
		$firstLast = $this->getFirstLastDate();
		if(!is_null($firstLast)){
			$query = $this->_buildQuery($listId, true, null, $firstLast['first'], $firstLast['last']);
		}else{
			$query = $this->_buildQuery($listId, true, null);
		}
		
		$blockedMails = $this->_getList($query, 0, 0);
		return $blockedMails;
	}

	/**
	 * Method to get bounced mails
	 */
	function getBouncedMailData($listId, $overrideLimits=false, $filterByCoreACL=false)
	{
		if($this->_id != $listId){
			$this->_id = $listId;
			$this->_data = null;
			$this->_filterByCoreACL = $filterByCoreACL;
		}
		$firstLast = $this->getFirstLastDate();
		if(!is_null($firstLast)){
			$query = $this->_buildQuery($listId, null, true, $firstLast['first'], $firstLast['last']);
		}else{
			$query = $this->_buildQuery($listId, null, true);
		}
		
		$bouncedMails = $this->_getList($query, 0, 0);		
		return $bouncedMails;
	}

	function getFirstLastDate(){
		$firstLast = null;
		if(!empty($this->_data)){
			$firstLast = array();
			$first = $this->_data[0];
			$last = $this->_data[count($this->_data)-1];
			$firstTstmp = $first->receive_timestamp; // MySQL format
			$firstParsed = strtotime($firstTstmp); // timestamp
			$lastTstmp = $last->receive_timestamp; // MySQL format
			$lastParsed = strtotime($lastTstmp); // timestamp

			$firstMail = $firstTstmp; // MySQL format
			$lastMail = $lastTstmp; // MySQL format

			if($lastParsed < $firstParsed){ // determine smaller timestamp
				$firstMail = $lastTstmp; // MySQL format
				$lastMail = $firstTstmp; // MySQL format
			}
			$firstLast['first'] = $firstMail; // MySQL format
			$firstLast['last']	= $lastMail; // MySQL format
			
			if($this->_limit_offset == 0){
				// This means the forwarded emails are shown from the beginning
				// To include bounced/blocked emails that are more recent,
				// we have to null the last date timestamp.
				$firstLast['last'] = null;
			}
		}
		return $firstLast;
	}

	function getTotal()
	{
		if (empty($this->_total)){
			$query = $this->_buildQuery($this->_id);
			$this->_total = $this->_getListCount($query);
		}

		return $this->_total;
	}

	function getPagination()
	{
		if (empty($this->_pagination)){
			jimport('joomla.html.pagination');
			$this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
		}

		return $this->_pagination;
	}
	
	function _buildQuery($listId, $isBlocked=null, $isBounced=null, $firstDate=null, $lastDate=null)
	{
		$where		= $this->_buildContentWhere($listId, $isBlocked, $isBounced, $firstDate, $lastDate);
		$orderby	= $this->_buildContentOrderBy();
			
		$query = 'SELECT m.*, l.name'
					. ' FROM #__mailster_mails AS m'
					. ' LEFT JOIN #__mailster_lists AS l'
					. ' ON (m.list_id = l.id)'
					. $where
					. $orderby;
		return $query;
	}

	function _buildContentOrderBy()
	{
		if($this->_order_by === 'rfirst'){
			$orderby = ' ORDER BY m.receive_timestamp DESC, m.fwd_completed_timestamp DESC';
		}else{
			$orderby = ' ORDER BY m.receive_timestamp ASC, m.fwd_completed_timestamp ASC';
		}
		return $orderby;
	}

	function _buildContentWhere($listId, $isBlocked=null, $isBounced=null, $firstDate=null, $lastDate=null)
	{
		$log = MstFactory::getLogger();
		$where = array();
		
		if($listId > 0){
			$where[] = ' m.list_id = \'' . $listId . '\'';
		}
		
		/*if($this->_filter2subscribed_lists_user_id){			
			$recip = MstFactory::getRecipients();
			$subscribedListIdsOfUser = $recip->getListsUserIsMemberOf($this->_filter2subscribed_lists_user_id, 1);
			$where[] = ' m.list_id IN ( '.implode(', ', $subscribedListIdsOfUser).' )';
		}*/
		
		if($this->_filterByFrontendMailArchiveACL){
			$listIdsAllowedToBeShown = array();
			$mailingListUtils = MstFactory::getMailingListUtils();
			$mailingLists = $mailingListUtils->getAllMailingLists();
			$recip = MstFactory::getRecipients();
			$user = JFactory::getUser();
			$userIsLoggedIn = false;
			if($user->id > 0){
				$userIsLoggedIn = true;
			}
			foreach($mailingLists AS $mList){
				if($mList->front_archive_access == MstConsts::FRONT_ARCHIVE_ACCESS_ALL_USERS){
					$listIdsAllowedToBeShown[] = $mList->id;
					$log->debug('_filterByFrontendMailArchiveACL -> add list ID '.$mList->id.' as all users are allowed');
				}elseif($mList->front_archive_access == MstConsts::FRONT_ARCHIVE_ACCESS_LOGGED_IN_USERS && $userIsLoggedIn){
					$listIdsAllowedToBeShown[] = $mList->id;
					$log->debug('_filterByFrontendMailArchiveACL -> add list ID '.$mList->id.' as all logged-in users are allowed');
				}elseif($mList->front_archive_access == MstConsts::FRONT_ARCHIVE_ACCESS_LOGGED_IN_SUBSCRIBERS_OF_MAILING_LIST && $userIsLoggedIn){
					$subscribedListIdsOfUser = $recip->getListsUserIsMemberOf($user->id, 1);
					for($i=0;$i<count($subscribedListIdsOfUser);$i++){
						if($subscribedListIdsOfUser[$i] == $mList->id){
							$listIdsAllowedToBeShown[] = $mList->id;
							$log->debug('_filterByFrontendMailArchiveACL -> add list ID '.$mList->id.' as user ID '.$user->id.' is a subscriber as required');
							break;
						}
					}
                }elseif($mList->front_archive_access == MstConsts::FRONT_ARCHIVE_ACCESS_NOBODY){
                    continue; // nope, this list is not allowed...
                }
			}
			$log->debug('mails_model: _filterByFrontendMailArchiveACL -> listIdsAllowedToBeShown: '.print_r($listIdsAllowedToBeShown, true));
			if(count($listIdsAllowedToBeShown) > 0){
				$where[] = ' m.list_id IN ( '.implode(', ', $listIdsAllowedToBeShown).' )';
			}else{
				$where[] = ' m.list_id < 0'; // add impossible condition
			}
		}
		
		if($this->_filterByCoreACL){
			$listIdsAllowedToBeShown = array();
			$mailingListUtils = MstFactory::getMailingListUtils();
			$mailingLists = $mailingListUtils->getAllMailingLists();
			foreach($mailingLists AS $mList){
				if(MstFactory::getAuthorization()->isAllowed('core.edit', $mList->id)){
					$listIdsAllowedToBeShown[] = $mList->id;
				}
			}
			//$log->debug('mails_model: _filterByCoreACL -> listIdsAllowedToBeShown: '.print_r($listIdsAllowedToBeShown, true));
			if(count($listIdsAllowedToBeShown) > 0){
				$where[] = ' m.list_id IN ( '.implode(', ', $listIdsAllowedToBeShown).' )';
			}else{
				$where[] = ' m.list_id < 0'; // add impossible condition
			}
		}
				
		if(!is_null($firstDate)){
			$where[] = ' m.receive_timestamp >= \'' . $firstDate . '\'';
		}
		if(!is_null($lastDate)){
			$where[] = ' m.receive_timestamp <= \'' . $lastDate . '\'';
		}
		if(!is_null($isBlocked)){
			if($isBlocked){
				$where[] = ' m.blocked_mail > \'0\'';
			}else{
				$where[] = ' m.blocked_mail = \'0\'';
			}
		}
		if(!is_null($isBounced)){
			$bounced = $isBounced ? 1 : 0;
			$where[] = ' m.bounced_mail = \'' . $bounced . '\'';
		}
		if(is_null($isBlocked) && is_null($isBounced)){
			$where[] = ' ((m.bounced_mail IS NULL AND m.blocked_mail IS NULL) OR (m.bounced_mail = \'0\' AND m.blocked_mail = \'0\'))';
		}
		
		$filterSearch = JRequest::getString('filter_search', '');
		if(strlen(trim($filterSearch)) > 0){
			$where[] = '( '
						. ' m.subject LIKE \'%'.$filterSearch.'%\''
						. ' OR m.body LIKE \'%'.$filterSearch.'%\''
						. ' OR m.html LIKE \'%'.$filterSearch.'%\''
						. ' OR m.from_name LIKE \'%'.$filterSearch.'%\''
						. ' OR m.from_email LIKE \'%'.$filterSearch.'%\''
						. ' OR EXISTS ('
					        . ' SELECT 1'
					        . ' FROM #__mailster_attachments attach'
					        . ' WHERE attach.mail_id = m.id'
					        . ' AND attach.filename LIKE \'%'.$filterSearch.'%\''
							. ')'
						. ')';
		}
		
		$where 		= ( count( $where ) ? ' WHERE ' . implode( ' AND ', $where ) : '' );
		
		return $where;
	}

    function getEmailsOlderThanDays($nrDays, $listId=0, $isBlocked=false, $isBounced=false, $returnIdOnly=true, $limit=false){
        $where = array();
        $where[] = ' m.receive_timestamp < DATE_SUB(CURDATE(), INTERVAL '.$nrDays.' DAY)';
        if($listId > 0){
            $where[] = ' m.list_id = \'' . $listId . '\'';
        }
        if($isBlocked){
            $where[] = ' m.blocked_mail = \'1\'';
        }
        if($isBounced){
            $where[] = ' m.bounced_mail = \'1\'';
        }
        $where 	 = ( count( $where ) ? ' WHERE ' . implode( ' AND ', $where ) : '' );

        $select = $returnIdOnly ? 'm.id' : 'm.*';
        $query = 'SELECT '.$select
                    . ' FROM #__mailster_mails AS m'
                    . $where;
        if($limit){
            $query .= ' LIMIT 0,'.$limit;
        }

        $oldEmails = $this->_getList($query, 0, 0);
        return $oldEmails;
    }

    function moveEmailsToOfflineArchive($mailIds){
        $log = MstFactory::getLogger();
        $log->debug('moveEmailsToOfflineArchive for '.count($mailIds).' mails');

        if(count($mailIds) <= 0){
            $log->debug('moveEmailsToOfflineArchive: No mail IDs provided, return/exit');
            return;
        }

        $mailIdsStr = implode(',',$mailIds);
        $log->debug('moveEmailsToOfflineArchive IDs: '.print_r($mailIds, true));


        $query = 'INSERT INTO #__mailster_oa_attachments'
                    . ' SELECT * FROM #__mailster_attachments a'
                    . ' WHERE a.mail_id IN ('.$mailIdsStr.')'
                    . ' ON DUPLICATE KEY UPDATE id=a.id;';
        $this->_db->setQuery( $query );
        $errorMsg = '';
        $result = false;
        try {
            $result = $this->_db->query();
        } catch (DatabaseException $e) {
            $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
        }
        $deleteOldAttachments = true;
        if(!$result) {
            $log->error('moveEmailsToOfflineArchive: Error moving attachments: '.$errorMsg);
            $deleteOldAttachments = false;
        }

        $query = 'INSERT INTO #__mailster_oa_mails'
            . ' SELECT * FROM #__mailster_mails m'
            . ' WHERE m.id IN ('.$mailIdsStr.')'
            . ' ON DUPLICATE KEY UPDATE id=m.id;';
        $this->_db->setQuery( $query );
        $errorMsg = '';
        $result = false;
        try {
            $result = $this->_db->query();
        } catch (DatabaseException $e) {
            $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
        }
        $deleteOldMails = true;
        if(!$result) {
            $log->error('moveEmailsToOfflineArchive: Error moving emails: '.$errorMsg);
            $deleteOldMails = false;
        }

        if($deleteOldAttachments && $deleteOldMails){
            $log->debug('moveEmailsToOfflineArchive: No error occurred copying attachments and emails, therefore now delete the old entries');

            $query = 'DELETE FROM #__mailster_attachments'
                . ' WHERE mail_id IN ('.$mailIdsStr.')';
            $this->_db->setQuery( $query );
            $errorMsg = '';
            $result = false;
            try {
                $result = $this->_db->query();
            } catch (DatabaseException $e) {
                $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
            }
            if(!$result) {
                $log->error('moveEmailsToOfflineArchive: Error deleting old attachments: '.$errorMsg);
            }

            $query = 'DELETE FROM #__mailster_mails'
                . ' WHERE id IN ('.$mailIdsStr.')';
            $this->_db->setQuery( $query );
            $errorMsg = '';
            $result = false;
            try {
                $result = $this->_db->query();
            } catch (DatabaseException $e) {
                $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
            }
            if(!$result) {
                $log->error('moveEmailsToOfflineArchive: Error deleting old emails: '.$errorMsg);
            }
            $log->debug('moveEmailsToOfflineArchive: Completed offline archiving');
        }else{
            $log->warning('moveEmailsToOfflineArchive: At least one error occured copying attachments and emails, therefore do NOT delete the old entries for mail with IDs: '.$mailIdsStr);
        }
    }

	function delete($cid = array())
	{
		$result = false;

		if (count( $cid )){
            /** @var MailsterModelQueue $queueModel */
            $queueModel = MstFactory::getModel('queue');
            $queueModel->deleteQueueEntriesOfMails($cid);

            /** @var MailsterModelDigest $digestModel */
            $digestModel = MstFactory::getModel('digest');
            $digestModel->deleteDigestQueueEntriesOfMails($cid);

            $sendEvents = MstFactory::getSendEvents();
            $sendEvents->deleteSendReportOfMails($cid);


            $attachUtils = MstFactory::getAttachmentsUtils();
            for($i=0; $i<count($cid); $i++){
                $attachUtils->deleteAttachmentsOfMail($cid[$i]);
            }

			$cids = implode( ',', $cid );
			$query = 'DELETE FROM #__mailster_mails'
			. ' WHERE id IN ('. $cids .')';

			$this->_db->setQuery( $query );
			$errorMsg = '';
            try {
                $result = $this->_db->query();
            } catch (DatabaseException $e) {
                $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
            }
			if(!$result) {
				$this->setError($errorMsg);
				return false;
			}
		}

		return true;
	}
}//Class end
?>
