<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	 
	defined('_JEXEC') or die('Restricted access');

	jimport('joomla.application.component.model');

	/**
	 * Queued Mails Model
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterModelQueue extends MailsterModel
	{
		var $_data = null;
		var $_total = null;
		var $_pagination = null;

		function __construct(){
			parent::__construct();
			$app = JFactory::getApplication();
			$limit = $app->getUserStateFromRequest('global.list.limit', 'limit', $app->getCfg('list_limit'));
			$limitstart = $app->getUserStateFromRequest(JRequest::getCmd('option','com_mailster').'limitstart','limitstart',0);
				
			$limit = $limit + 0; // make it numerical
			$limitstart = $limitstart + 0; // make it numerical
				
			if($limit == 0){
				$limitstart = 0;
			}
	
			$this->setState('limit',$limit);
			$this->setState('limitstart',$limitstart);
		}

		
		function getData($overrideLimits=false, $limitedCols=false, $mailId=false)
		{
			if (empty($this->_data))
			{
				$query = $this->_buildQuery($limitedCols, $mailId);
				$limitstart = $this->getState('limitstart');
				$limit = $this->getState('limit');
	
				if(!$overrideLimits){
					$this->_data = $this->_getList($query, $limitstart, $limit);
				}else{
					$this->_data = $this->_getList($query, 0, 0);
				}
			}
			return $this->_data;	
		}
				
		function getPagination()
		{
			if (empty($this->_pagination))
			{
				jimport('joomla.html.pagination');
				$this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
			}

			return $this->_pagination;
		}
				
		function getTotal()
		{
			if (empty($this->_total)){
				$query = $this->_buildQuery(true);
				$this->_total = $this->_getListCount($query);
			}
			return $this->_total;
		}

		function _buildQuery($limitedCols=false, $mailId=false)
		{	
			$where		= $this->_buildContentWhere($mailId);
			$orderby	= $this->_buildContentOrderBy();
			
			if($limitedCols){
				$query = 'SELECT m.mail_id, m.name, m.email, m.is_locked, ma.id, ma.subject';	
			}else{
				$query = 'SELECT m.*, ma.*';
			}
			
			$query = $query	. ' FROM #__mailster_queued_mails m	LEFT JOIN #__mailster_mails ma ON (m.mail_id = ma.id)'
							. $where
							. $orderby;	
						
			return $query;
		}

		function _buildContentOrderBy()
		{
			$orderby 	= ' ORDER BY m.mail_id ASC, m.email ASC';
			return $orderby;
		}

		function _buildContentWhere($mailId=false)
		{	
			$where = array();
			if($mailId){
				$where[] = 'm.mail_id = \''.$mailId.'\'';
			}
			$where 		= ( count( $where ) ? ' WHERE ' . implode( ' AND ', $where ) : '' );	
			return $where;
		}

		
		function clearQueue()
		{
			$query = 'DELETE FROM #__mailster_queued_mails';
			$this->_db->setQuery( $query );
			$errorMsg = '';
            try {
                $result = $this->_db->query();
            } catch (DatabaseException $e) {
                $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
            }
			if(!$result) {
				$this->setError($errorMsg);
				return false;
			}
			return true;
		}
		
		function delete($cid = array())
		{
			$result = false;

			if (count( $cid )){
				
				for($i=0;$i<count($cid);$i++){
					$queueEntry = explode(':',$cid[$i]);
					$mailId = $queueEntry[0];
					$email = $queueEntry[1];
					$query = 'DELETE FROM #__mailster_queued_mails'
							. ' WHERE mail_id = \'' . $mailId . '\''
							. ' AND email = ' . $this->_db->quote($email);
					$this->_db->setQuery( $query );
					$errorMsg = '';
                    try {
                        $result = $this->_db->query();
                    } catch (DatabaseException $e) {
                        $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                    }
					if(!$result) {
						$this->setError($errorMsg);
						return false;
					}
				}
			}

			return true;
		}

        function deleteQueueEntriesOfMails($mailIds){
            $result = false;

            if (count($mailIds)){
                $mailIdsStr = implode( ',', $mailIds);
                $query = 'DELETE FROM #__mailster_queued_mails'
                    . ' WHERE mail_id IN ('. $mailIdsStr .')';
                $this->_db->setQuery( $query );
                $errorMsg = '';
                try {
                    $result = $this->_db->query();
                } catch (DatabaseException $e) {
                    $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                }
                if(!$result) {
                    $this->setError($errorMsg);
                    return false;
                }
            }

            return true;
        }
	}//Class end
?>
