<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	 
	defined('_JEXEC') or die('Restricted access');

	jimport('joomla.application.component.model');

	/**
	 * Servers Model
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterModelServers extends MailsterModel
	{
		var $_data = null;
		var $_total = null;
		var $_pagination = null;

		function __construct(){
			parent::__construct();
		}

        function getInboxServers($onlyPublished = false){
            $query = 'SELECT a.*'
                . ' FROM #__mailster_servers a'
                . ' WHERE server_type = '.MstConsts::SERVER_TYPE_MAIL_INBOX;
            if($onlyPublished){
                $query .= ' AND published = \'1\'';
            }
            return $this->_getList($query, 0, 0);
        }

        function getSMTPServers($onlyPublished = false){
            $query = 'SELECT a.*'
                . ' FROM #__mailster_servers a'
                . ' WHERE server_type = '.MstConsts::SERVER_TYPE_SMTP;
            if($onlyPublished){
                $query .= ' AND published = \'1\'';
            }
            return $this->_getList($query, 0, 0);
        }

        function getProviderTypeSettings($providerType){
            $query = 'SELECT *'
                .' FROM #__mailster_servers'
                .' WHERE provider_type =\''.$providerType.'\'';
            return $this->_getList($query, 0, 0);
        }

		function getData()
		{
			if (empty($this->_data))
			{
				$query = $this->_buildQuery();
				$this->_data = $this->_getList($query, 0, 0);
				$this->_data = $this->_additionals($this->_data);
			}
			return $this->_data;
		}

		function getTotal()
		{
			if (empty($this->_total))
			{
				$query = $this->_buildQuery();
				$this->_total = $this->_getListCount($query);
			}

			return $this->_total;
		}

		function getPagination()
		{
			if (empty($this->_pagination))
			{
				jimport('joomla.html.pagination');
				$this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
			}

			return $this->_pagination;
		}

		function _buildQuery()
		{
			$where		= $this->_buildContentWhere();
			$orderBy	= $this->_buildContentOrderBy();

			$query = ' SELECT a.*,'
                        .' (SELECT COUNT(*) FROM #__mailster_lists l WHERE l.server_inb_id = a.id) AS used_in_list_inb_count,'
                        .' (SELECT COUNT(*) FROM #__mailster_lists l WHERE l.server_out_id = a.id) AS used_in_list_out_count'
                        .' FROM #__mailster_servers a'
						. $where
						. $orderBy
						;
			return $query;
		}

		function _buildContentOrderBy()
		{
            $orderBy = ' ORDER BY a.id';
			return $orderBy;
		}

		function _buildContentWhere()
		{		
			$where = array();					
			$where = ( count( $where ) ? ' WHERE ' . implode( ' AND ', $where ) : '' );
			return $where;
		}

		/**
		 * Additional tasks
		 */
		function _additionals($rows)
		{
			/* nothing for now */
			return $rows;
		}

        function publish($cid = array(), $value = 1){

            if($value != 1){ // when unpublishing we need to check first if that setting is used in a mailing list!
                $mailingListUtils = MstFactory::getMailingListUtils();
                for($i=0;$i<count($cid);$i++){
                    $listsUsingServer = $mailingListUtils->getMailingListsUsingServerId($cid[$i]);
                    if(count($listsUsingServer)){
                        $this->setError(JText::sprintf('COM_MAILSTER_SERVER_ID_X_IS_USED_BY_MAILING_LIST', $cid[$i]));
                        return false;
                    }
                }
            }

            /** @var mailster_servers $table */
            $table = JTable::getInstance('mailster_servers', '', array());
            // Attempt to change the state of the records.
            if (!$table->publish($cid, $value)) {
                $this->setError($table->getError());
                return false;
            }
            return true;
        }
		
		function delete($cid = array())
		{
			$result = false;
            $mailingListUtils = MstFactory::getMailingListUtils();

            for($i=0;$i<count($cid);$i++){
                $listsUsingServer = $mailingListUtils->getMailingListsUsingServerId($cid[$i]);
                if(count($listsUsingServer)){
                    $this->setError(JText::sprintf('COM_MAILSTER_SERVER_X_IS_USED_BY_MAILING_LIST', $cid[$i]));
                    return false;
                }
            }
				
			if (count($cid)){
				$cids = implode( ',', $cid );
				$query = 'DELETE FROM #__mailster_servers'
						. ' WHERE id IN ('. $cids .') AND provider_type = \'0\''; // only allow user-defined providers to be removed

				$this->_db->setQuery( $query );
				$errorMsg = '';
                try {
                    $result = $this->_db->query();
                } catch (DatabaseException $e) {
                    $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                }
				if(!$result) {
					$this->setError($errorMsg);
					return false;
				}

			}

			return true;
		}
	}
?>