<?php
/**
 * @package Joomla
 * @subpackage Mailster
 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
 * @license GNU/GPL, see license.txt
 * Mailster is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.
 *
 * Mailster is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mailster; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * or see http://www.gnu.org/licenses/.
 */


defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

/**
 * Mailing List Thread Model
 *
 * @package Joomla
 * @subpackage Mailster
 */
class MailsterModelThread extends MailsterModel
{

	var $_id = 0;
	var $_data = null;
	var $_total = null;
	var $_pagination = null;
	var $_filterByFrontendMailArchiveACL = false;
	
	function __construct()
	{
		parent::__construct();
		$app = JFactory::getApplication();
		$limit = $app->getUserStateFromRequest('global.list.limit', 'limit', $app->getCfg('list_limit'), 'int');
		$limitstart = JRequest::getVar('limitstart', 0, '', 'int');
		$this->setState('limit',$limit);
		$this->setState('limitstart',$limitstart);
	}

	function getData($threadId, $overrideLimits=false, $filterByFrontendMailArchiveACL=false)
	{
		if (empty($this->_data) || $this->_id != $threadId){
			$this->_id = $threadId;
			$this->_filterByFrontendMailArchiveACL = $filterByFrontendMailArchiveACL; 
			
			$query = $this->_buildQuery($threadId);
			
			$limitstart = $this->getState('limitstart');
			$limit = $this->getState('limit');
			
			if(!$overrideLimits){
				$this->_data = $this->_getList($query, $limitstart, $limit);
			}else{
				$this->_data = $this->_getList($query, 0, 0);
			}
			
		}
		return $this->_data;
	}
	
	function getThreadSubject($threadId){
		$query = 'SELECT m.subject '
				. ' FROM #__mailster_mails AS m'
				. ' WHERE m.thread_id = \'' . $threadId . '\''
				. ' ORDER BY m.receive_timestamp';
		$subjects = $this->_getList($query, 0, 1);
		return count($subjects)>0 ? $subjects[0]->subject : '';
	}

	function getTotal()
	{
		if (empty($this->_total)){
			$query = $this->_buildQuery($this->_id);
			$this->_total = $this->_getListCount($query);
		}

		return $this->_total;
	}

	function getPagination()
	{
		if (empty($this->_pagination)){
			jimport('joomla.html.pagination');
			$this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
		}

		return $this->_pagination;
	}

	function _buildQuery($threadId)
	{
		$where		= $this->_buildContentWhere($threadId);
		$orderby	= $this->_buildContentOrderBy();
		
		$query = 'SELECT * '
					. ' FROM #__mailster_mails AS m'				
					. $where
					. $orderby;
		return $query;
	}
	
	function _buildContentOrderBy()
	{
		$orderby 	= ' ORDER BY m.receive_timestamp DESC';
		return $orderby;
	}
	
	function _buildContentWhere($threadId, $filterOutBlockedAndBouncedMails=true)
	{
		$log = MstFactory::getLogger();
		$where = array();
		if($threadId > 0){
			$where[] = ' m.thread_id = \'' . $threadId . '\'';
		}
		if($filterOutBlockedAndBouncedMails){
			$where[] = ' ((m.bounced_mail IS NULL AND m.blocked_mail IS NULL) OR (m.bounced_mail = \'0\' AND m.blocked_mail = \'0\'))';
		}
		if($this->_filterByFrontendMailArchiveACL){
			$user = JFactory::getUser();
			$mListUtils = MstFactory::getMailingListUtils();
			$listIdsAllowedToBeShown = $mListUtils->getAllMailingListsUserCanAccessMailsInFrontend($user->id);
			if(count($listIdsAllowedToBeShown)){
				$queryPart = 'm.list_id IN ('.implode(',', $listIdsAllowedToBeShown).')';
				$log->debug('threadmodel->_buildContentWhere: filterByFrontendMailArchiveACL add condition: "'.$queryPart.'"');
				$where[] = $queryPart;
			}
		}
		
		$where 		= ( count( $where ) ? ' WHERE ' . implode( ' AND ', $where ) : '' );
	
		return $where;
	}
	

}//Class end
?>
