<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	 
	defined('_JEXEC') or die('Restricted access');

	jimport('joomla.application.component.model');

	/**
	 * Mailing List User Groups Model
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterModelUserGroups extends MailsterModel
	{
		var $_data = null;
		var $_pagination = null;
		var $_userID = null;
		var $_is_joomla_user = null;

		function __construct(){
			parent::__construct();
		}

		/**
		 * Method to get user groups item data
		 */
		function getData($userID, $is_joomla_user)
		{
			$groupModel = MstFactory::getModel('group');
			$this->_userID = $userID;
			$this->_is_joomla_user = $is_joomla_user;
			
			if (empty($this->_data))
			{
				$query = $this->_buildQuery();				
				$groups = $this->_getList($query, 0, 0);
				$this->_data = array();
				foreach($groups AS $group){
					$groupModel->setId($group->id);
					$this->_data[] = $groupModel->getData();
				}				
			}
			return $this->_data;			
		}

		function getPagination(){
			if (empty($this->_pagination)){
				jimport('joomla.html.pagination');
				$this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
			}

			return $this->_pagination;
		}
		
		function _buildQuery(){
			$where		= $this->_buildContentWhere();
			$orderby	= $this->_buildContentOrderBy();			
		
			$query = 'SELECT g.*'
						. ' FROM #__mailster_groups g ';					
			$query = $query	. $where;
			$query = $query	. $orderby;
			return $query;
		}
		
		function _buildContentOrderBy(){
			$orderby 	= ' ORDER BY g.name, g.id';			
			return $orderby;
		}
			
		public function getTable($type = 'mailster_group_users', $prefix = '', $config = array()){
			return JTable::getInstance($type, $prefix, $config);
		}
		
		function isUserInGroup($userId, $isJoomlaUser, $groupId){
			$userModel = MstFactory::getModel('user');
			return $userModel->isUserInGroup($userId, $isJoomlaUser, $groupId);
		}
		
		function store($groupIds, $userID=null, $is_joomla_user=null)
		{		
			$user		= JFactory::getUser();
			$config 	= JFactory::getConfig();

			if(is_null($userID)){
				$userID = $this->_userID;
			}
			if(is_null($is_joomla_user)){
				$is_joomla_user = $this->_is_joomla_user;	
			}

			$log = MstFactory::getLogger();
			//$log->debug('MailsterModelUserGroups:store for groups: '.print_r($groupIds, true).' for user ID '.$userID.', is_joomla_user '.$is_joomla_user);
			//$log->debug('MailsterModelUserGroups:store First delete current table entries..');
			
			// Delete from table if existing
			$this->delete($groupIds, $userID, $is_joomla_user);
			
			//$log->debug('MailsterModelUserGroups:store Now store new table entries');
			for($i=0;$i<count($groupIds);$i++){

				$data = new stdClass();
				$data->group_id = $groupIds[$i];
				$data->user_id = $userID;
				$data->is_joomla_user = $is_joomla_user;
				
				$row  = $this->getTable();
				
				if (!$row->bind($data)) {
					JError::raiseError(500, $this->_db->getErrorMsg() );
					return false;
				}
	
				if (!$row->check()) {
					$this->setError($row->getError());
					return false;
				}
	
				if (!$row->store()) {
					JError::raiseError(500, $this->_db->getErrorMsg() );
					return false;
				}
			}
			return true;
		}
						
		function _buildContentWhere()
		{	
			$where = array();			
			$where[] = ' g.group_id IN ('
							. ' SELECT DISTINCT group_id FROM #__mailster_group_users'
							. ' WHERE user_id = ' . $this->_userID
							. ' AND is_joomla_user = ' . $this->_is_joomla_user
							. ')';
			$where 		= ( count( $where ) ? ' WHERE ' . implode( ' AND ', $where ) : '' );			
			return $where;
		}
		
		/**
		* Get all mailing lists that use one of the groups
		*/
		function getListsWithGroups($groupIds)
		{
            if(count($groupIds)){
                $ids = implode( ',', $groupIds );
                $query = 'SELECT l.*'
                        . ' FROM #__mailster_lists l '
                        . ' WHERE l.id IN ('
                        . '		SELECT DISTINCT lg.list_id'
                        . '		FROM #__mailster_list_groups lg '
                        . '		WHERE lg.group_id IN ('.$ids.')'
                        . ' )';
                $lists = $this->_getList($query, 0, 0);
            }else{
                $lists = array();
            }
			return $lists;
		}
		

		/**
		 * Method to remove groups from user
		 */		
		function delete($groupIds=array(), $userID=null, $is_joomla_user=null)
		{
			$log = MstFactory::getLogger();
			$log->debug('usergroups Model->delete, groupIds: '.print_r($groupIds, true).', userId: '.$userID. ', is_joomla_user: '.$is_joomla_user);
			if(is_null($userID)){
				$userID = $this->_userID;
			}
			if(is_null($is_joomla_user)){
				$is_joomla_user = $this->_is_joomla_user;	
			}
			
			$result = false;
			$groupCount = count( $groupIds );
			
			for($i=0; $i < $groupCount; $i++){
				$query = 'DELETE FROM #__mailster_group_users'
						. ' WHERE user_id =\''. $userID . '\''
						. ' AND group_id =\''. $groupIds[$i] . '\''
						. ' AND is_joomla_user =\''. $is_joomla_user . '\'';
				$this->_db->setQuery( $query );
				$errorMsg = '';
                try {
                    $result = $this->_db->query();
                } catch (DatabaseException $e) {
                    $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                }
				if(!$result) {
					$log->error('usergroups Model->delete error: '.$errorMsg);
					$this->setError($errorMsg);
					return false;
				}
			}

			return true;
		}
				
		
	}//Class end
?>
