<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	 
	defined('_JEXEC') or die('Restricted access');

	jimport('joomla.application.component.model');

	/**
	 * Mailing List Users Model
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterModelUsers extends MailsterModel
	{
		var $_data = null;
		var $_total = null;
		var $_pagination = null;

		function __construct(){
			parent::__construct();
		}

		function getData()
		{
			if (empty($this->_data))
			{
				$query = $this->_buildQuery();
				$this->_data = $this->_getList($query, 0, 0);
				$this->_data = $this->_additionals($this->_data);
			}
			return $this->_data;
		}

		function getTotal()
		{
			if (empty($this->_total))
			{
				$query = $this->_buildQuery();
				$this->_total = $this->_getListCount($query);
			}

			return $this->_total;
		}

		function getPagination()
		{
			if (empty($this->_pagination))
			{
				jimport('joomla.html.pagination');
				$this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
			}

			return $this->_pagination;
		}

		function _buildQuery()
		{
			$where		= $this->_buildContentWhere();
			$orderby	= $this->_buildContentOrderBy();

			$query = 'SELECT a.*'
						. ' FROM #__mailster_users AS a'
						. $where
						. $orderby
						;
			return $query;
		}

		function _buildContentOrderBy()
		{
			$orderby 	= ' ORDER BY a.name, a.email';			
			return $orderby;
		}

		function _buildContentWhere()
		{		
			$where = array();					
			$where 		= ( count( $where ) ? ' WHERE ' . implode( ' AND ', $where ) : '' );
			return $where;
		}

		/**
		 * Additional tasks
		 */
		function _additionals($rows)
		{
			/* nothing for now */
			return $rows;
		}
		
		function delete($cid = array())
		{
			$result = false;
			$mstRecipients = MstFactory::getRecipients();
		
			$allLists2Update = array();
			$isJUser = 0;
			for($i=0;$i<count($cid);$i++){	
				$lists = $mstRecipients->getListsUserIsMemberOf($cid[$i], $isJUser);
				$allLists2Update = array_merge($allLists2Update, $lists);
				array_unique($allLists2Update);
			}
				
			if (count( $cid ))
			{
				$cids = implode( ',', $cid );
					
				$query = 'DELETE FROM #__mailster_users'
						. ' WHERE id IN ('. $cids .')';

				$this->_db->setQuery( $query );
				$errorMsg = '';
                try {
                    $result = $this->_db->query();
                } catch (DatabaseException $e) {
                    $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                }
				if(!$result) {
					$this->setError($errorMsg);
					return false;
				}
				
				$query = 'DELETE FROM #__mailster_group_users'
						. ' WHERE user_id IN ('. $cids .')'
						. ' AND is_joomla_user = \'0\'';

				$this->_db->setQuery( $query );
				$errorMsg = '';
                try {
                    $result = $this->_db->query();
                } catch (DatabaseException $e) {
                    $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                }
				if(!$result) {
					$this->setError($errorMsg);
					return false;
				}
			
				$query = 'DELETE FROM #__mailster_list_members'
						. ' WHERE user_id IN ('. $cids .')'
						. ' AND is_joomla_user = \'0\'';

				$this->_db->setQuery( $query );
				$errorMsg = '';
                try {
                    $result = $this->_db->query();
                } catch (DatabaseException $e) {
                    $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                }
				if(!$result) {
					$this->setError($errorMsg);
					return false;
				}
												
				$mstRecipients->recipientsUpdatedInLists($allLists2Update);	// update cache states
								
			}

			return true;
		}
	}
?>
