<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	defined('_JEXEC') or die('Restricted access');

	/**
	 * Lists Model class
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class mailster_lists extends JTable
	{
		/**
		 * Primary Key
		 * @var int
		 */
		var $id 					= null;
		var $asset_id				= null;
		var $name					= null;	
		var $admin_mail				= null;	
		var $list_mail				= null;	
		var $subject_prefix			= null;
		var $mail_in_user			= null;	
		var $mail_in_pw				= null;
		var $mail_out_user			= null;	
		var $mail_out_pw			= null;
        var $server_inb_id 			= null;
        var $server_out_id			= null;
        var $published 				= null;
		var $active 				= null;
		var $use_cms_mailer 		= null;
		var $public_registration	= null;
		var $sending_public			= null;
		var $sending_recipients		= null;	
		var $sending_admin			= null;
		var $sending_group			= null;	
		var $sending_group_id		= null;	
		var $allow_subscribe		= null;
		var $allow_unsubscribe		= null;
		var $reply_to_sender		= null;
		var $copy_to_sender			= null;
		var $disable_mail_footer	= null;
		var $custom_header_plain	= null;
		var $custom_header_html		= null;
		var $custom_footer_plain	= null;
		var $custom_footer_html		= null;
		var $mail_format_conv		= null;
		var $mail_format_altbody	= null;
		var $alibi_to_mail			= null;
		var $addressing_mode		= null;
		var $mail_from_mode			= null;
		var $name_from_mode			= null;
		var $archive_mode			= null;
		var $archive2article		= null;
		var $archive2article_author	= null;
		var $archive2article_cat	= null;
		var $archive2article_state	= null;
        var $archive_offline    	= null;
		var $bounce_mode			= null;
		var $bounce_mail			= null;
		var $bcc_count				= null;	
		var $incl_orig_headers		= null;	
		var $max_send_attempts		= null;
		var $filter_mails			= null;
        var $allow_bulk_precedence	= null;
		var $clean_up_subject		= null;
		var $lock_id				= null;
		var $is_locked				= null;
		var $last_lock				= null;
		var $last_check				= null;
		var $last_mail_retrieved	= null;
		var $last_mail_sent			= null;
		var $cstate					= null;
		var $mail_size_limit		= null;
		var $notify_not_fwd_sender 	= null;
		var $save_send_reports		= null;
		var $subscribe_mode			= null;
		var $unsubscribe_mode		= null;
		var $welcome_msg			= null;
		var $welcome_msg_admin		= null;
		var $goodbye_msg			= null;
		var $goodbye_msg_admin		= null;
		var $allow_digests			= null;
		var $front_archive_access	= null;
		
		function mailster_lists(&$db) {
			parent::__construct('#__mailster_lists', 'id', $db);
		}
		
		/**
		 * Overloaded bind function
		 *
		 * @param       array           named array
		 * @return      null|string     null is operation was satisfactory, otherwise returns an error
		 * @see JTable:bind
		 * @since 1.5
		 */
		public function bind($array, $ignore = '')
		{
			$log = MstFactory::getLogger();
			
			if (isset($array['params']) && is_array($array['params'])){
				// Convert the params field to a string.
				$parameter = new JRegistry;
				$parameter->loadArray($array['params']);
				$array['params'] = (string)$parameter;
			}	

			
			/*
			 * [Old ACL]
			 * core.createlist
			 * core.editlist
			 * core.editlist.subscriber
			 * core.editsubscriber
			 * core.deletelist
			 */
			if (isset($array['jform']) && isset($array['jform']['rules']) && is_array($array['jform']['rules'])){
				//$log->debug('binding the rules 1st: '.print_r($array['jform']['rules'], true));
				foreach ($array['jform']['rules'] as $innerarraykey => $innerarrayvalue) {
					foreach ($array['jform']['rules'][$innerarraykey] as $key => $value) {
						if ($value == '') {
							unset($array['jform']['rules'][$innerarraykey][$key]);
						}
					}
				}				
				//$log->debug('binding the rules 2nd: '.print_r($array['jform']['rules'], true));
				$rules = new JAccessRules($array['jform']['rules']);
				$this->setRules($rules);
			}
			return parent::bind($array, $ignore);
		}
		
		/**
		 * Method to compute the default name of the asset.
		 * The default name is in the form `table_name.id`
		 * where id is the value of the primary key of the table.
		 *
		 * @return	string
		 * @since	1.6
		 */
		protected function _getAssetName()
		{
			return 'com_mailster.list.'.(int) $this->id;
		}
		
		/**
		 * Method to return the title to use for the asset table.
		 *
		 * @return	string
		 * @since	1.6
		 */
		protected function _getAssetTitle()
		{
			return $this->name;
		}
		
		/**
		 * Get the parent asset id for the record
		 *
		 * @return	int
		 * @since	1.6
		 */

		protected function _getAssetParentId(JTable $table = null, $id = null)
		{
			$log = MstFactory::getLogger();
			$asset = JTable::getInstance('Asset');
			$asset->loadByName('com_mailster');
			$parent_id = $asset->id;
			$log->debug('Parent ID after loadByName: '.$parent_id);
			return $parent_id;
		}
		
		function check(){			
			$this->name = strip_tags(trim($this->name));

			if ( $this->name == '' ) {
				$this->_error = JText::_( 'COM_MAILSTER_ADD_NAME' );
				JError::raiseWarning('NO MAILING LIST NAME SPECIFIED', $this->_error );
				return false;
			}


            if($this->id <= 0 && (count(MstFactory::getMailingListUtils()->getAllMailingLists(false))+1 > MstFactory::getV()->getFtSetting(MstVersionMgmt::MST_FT_ID_MLT)) ){
                JFactory::getApplication()->enqueueMessage(JText::_( 'COM_MAILSTER_TOO_MANY_MAILING_LISTS_PRODUCT_LIMIT' ), 'warning');
                return false;
            }
			
			return true;
		}
	}
?>
