<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	defined( '_JEXEC' ) or die( 'Restricted access' );
	
		
	if(!defined('DS')){
		define('DS',DIRECTORY_SEPARATOR); // For Joomla 3.0+
	}
	
	/**
	 * This script is heavily based on the one Nicholas K. Dionysopoulos is using
	 * in his direction giving Joomla component Akeeba Backup.
	 * 
	 * Thanks Nicholas for writing great software like that and inspring others
	 * to follow you.
	 * 
	 * - Holger Brandt
	 */	
	class com_mailsterInstallerScript
	{
		public $parent = null;
		
		protected $_extension = '';
		protected $_install_sql_path = '';
        protected $_uninstall_sql_path = '';
		protected $_script_install = '';
		protected $_script_update = '';
		protected $_script_uninstall = '';
        protected $_release_version_installed = '';
        protected $_release_version_current = '';
        protected $_release_edition_current = '';
        protected $_min_joomla_version = '';
        protected $_current_joomla_version = '';
		
		public function __construct() {
			$this->_extension			= 'com_mailster';
			$this->_install_sql_path	= 'install.utf8.sql';
            $this->_uninstall_sql_path	= 'uninstall.sql';
			$this->_script_install		= 'install.mailster.php';
			$this->_script_update		= 'install.mailster.php';
			$this->_script_uninstall	= 'uninstall.mailster.php';
            $this->_release_version_current = false;
            $this->_release_version_installed = false;
            $this->_release_edition_current = false;
            $this->_min_joomla_version = false;
            $jversion = new JVersion();
            $this->_current_joomla_version = $jversion->getShortVersion();
		}
		
		function preflight($type, $parent){
            $manifest = false;

            if($parent){
                $manifest = $parent->get("manifest");
                if($manifest){
                    if(property_exists($manifest, 'version')){
                        $this->_release_version_current = $manifest->version; // version of Mailster to be installed / updated to
                    }
                }
                $this->_min_joomla_version = $manifest->attributes()->version;  // Manifest file minimum Joomla version
            }


            if($this->_current_joomla_version && $this->_min_joomla_version){
                if(version_compare($this->_current_joomla_version, $this->_min_joomla_version, 'lt' ) ) {   // abort if the current Joomla release is older
                    Jerror::raiseWarning(null, 'Cannot install Mailster in a Joomla release prior to '.$this->_min_joomla_version);
                    return false;
                }
            }

            if($type == 'update') {  // abort if the component being installed is not newer than the currently installed version
                $this->_release_version_installed = $this->getParamFromManifestCache('version');
                if($this->_release_version_current && $this->_release_version_installed){
                    if (version_compare($this->_release_version_current, $this->_release_version_installed, 'lt' ) ) {
                        Jerror::raiseWarning(null, 'Incorrect version sequence. Cannot downgrade from version ' . $this->_release_version_installed. ' to version '.$this->_release_version_current);
                        return false;
                    }
                }

                $inclPath = JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster'.DS.'mailster'.DS.'includes.php';
                require_once($inclPath);
                $inclPath = JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster'.DS.'mailster'.DS.'utils'.DS.'DBUtils.php';
                require_once($inclPath);
                if(class_exists('MstDBUtils') && class_exists('MstFactory')){
                    // have to make sure class is included there before method gets called...
                    $this->checkAndFixDBCollations(); // auto fix collation issue (relevant for Joomla 3.5 and later...)
                }
            }
		}
		
	 	function postflight($type, $parent){
	 	}
		
		function install($parent) {
			// Copy the install/uninstall scripts
			$this->_copyLegacyScripts($parent);
			// Load the installation script
			$this->parent = $parent->getParent();
		//	$this->_scriptLoader($this->_script_install);
			return $this->installAndUpdateScript('install', $parent);
		}
	
		function update($parent) {
			// Copy the install/uninstall scripts
			$this->_copyLegacyScripts($parent);
			// Joomla! 1.6/1.7 workaround for not running SQL on updates
			$this->_workaroundApplyInstallationSQL($parent);
			// Load the udpate script
			$this->parent = $parent->getParent();
		//	$this->_scriptLoader($this->_script_update);
			return $this->installAndUpdateScript('update', $parent);
		}
		
		function uninstall($parent) {
			// Load the uninstallation script
			$this->parent = $parent->getParent();
            // Drop components table
            $this->_workaroundApplyUninstallationSQL($parent);
		//	$this->_scriptLoader($this->_script_uninstall);
            return $this->uninstallScript('uninstall', $parent);
		}
		
		function installAndUpdateScript($type, $parent){
			
				global $mailster_install_running;
				$mailster_install_running = 1;

                $this->_release_edition_current = 'Mailster'; // edition/name of Mailster to be installed / updated to

                if(!$this->_current_joomla_version){
                    $this->_current_joomla_version = JVERSION;
                }
			
				$inclPath = JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster'.DS.'mailster'.DS.'includes.php';	
				require_once($inclPath);
		
				jimport('joomla.filesystem.folder');	
				jimport('joomla.installer.helper');
				jimport('joomla.language.language');			
				       
				$lang = JFactory::getLanguage();
				$lang->load('com_mailster', JPATH_ADMINISTRATOR, null, true); // force reload of language files
				
				$inclPath = JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster'.DS.'mailster'.DS.'utils'.DS.'DBUtils.php';	
				require_once($inclPath);
				$dbUtils = new MstDBUtils();
				
				$log = MstFactory::getLogger();
				$log->info('START POST INSTALL OPERATIONS', MstConsts::LOGENTRY_INSTALLER);
                $log->info('Install type: '.$type.($type == 'update' ? (' from v'.($this->_release_version_installed ? $this->_release_version_installed : 'UNKOWN').' to v'.($this->_release_version_current ? $this->_release_version_current : 'UNKOWN').' '.($this->_release_edition_current ? $this->_release_edition_current : 'UNKOWN')) : ''));
                $log->info('Joomla: '.$this->_current_joomla_version.', Min. req. Joomla version: '.($this->_min_joomla_version ? $this->_min_joomla_version : 'UNKOWN'), MstConsts::LOGENTRY_INSTALLER);

				$lang = JFactory::getLanguage();
				$lang->load('com_mailster', JPATH_ADMINISTRATOR);
				
				$installer = new JInstaller();
				$installer->setOverwrite(true);
				$pluginPath = JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster'.DS.'extensions'.DS;
				$plugins = array( 	'plg_mailster.zip'				=> array('name'=>'Mail Plugin', 'isPlugin'=>true, 'element'=>'mailster', 'folder'=>'system', 'mandatory'=>true, 'defaultPublish'=>true),
				               		'plg_mailster_subscr.zip'		=> array('name'=>'Subscriber Plugin', 'isPlugin'=>true, 'element'=>'mailstersubscriber', 'folder'=>'content', 'mandatory'=>true, 'defaultPublish'=>true),
									'plg_mailster_profile.zip'		=> array('name'=>'Profile Subscriber Plugin', 'isPlugin'=>true, 'element'=>'mailsterprofile', 'folder'=>'system', 'mandatory'=>true, 'defaultPublish'=>true),
									'plg_mailster_search.zip'		=> array('name'=>'Search Plugin', 'isPlugin'=>true, 'element'=>'mailstersearch', 'folder'=>'search', 'mandatory'=>false, 'defaultPublish'=>false),
									'plg_mailster_cb.zip'			=> array('name'=>'CB Plugin', 'isPlugin'=>true, 'element'=>'mailstercb', 'folder'=>'system', 'mandatory'=>false, 'defaultPublish'=>false),
									'mod_mailster_subscr.zip'		=> array('name'=>'Subscriber Module', 'isPlugin'=>false, 'element'=>'', 'folder'=>'', 'mandatory'=>true, 'defaultPublish'=>false)
				                 );
				$props = parse_ini_file(JPATH_ADMINISTRATOR . DS . "components" . DS. "com_mailster" . DS . "version.properties");
				$version = $props['major'] . '.' . $props['minor'] . '.' .$props['bugfix'];
				$versionStr = 'v.' . $version . '';
				
				if($props['beta'] !== '-'){
					$versionStr .= ' ('.$props['beta'].')'; 
				}
				
				$msgcolor = "#B0FFB0";
			 	$msgtext  = "Mailster " . $versionStr . ' ' . JText::_( 'COM_MAILSTER_INSTALLED_SUCCESSFUL' );
				$imgSrc = 'components/com_mailster/assets/images/16-tick.png';
									
				?>					
				<center>
					<table cellpadding="4" cellspacing="0" border="0" width="100%" class="adminlist table table-striped">
						<tr>
							<td valign="top" width="20%">
					    		<a href="http://www.brandt-oss.com" target="_blank"><img src="<?php echo 'components/com_mailster/assets/images/logo_boss.png'; ?>" height="80" alt="Brandt OSS Logo" align="left"></a>
							</td>
							<td valign="top" width="50%">
					       	 	<strong>a project hosted at <a href="http://www.brandt-oss.com" target="_blank">Brandt OSS (Open Source Software)</a></strong>
							</td>
							<td valign="top" rowspan="2" width="30%">
								<a href="index.php?option=com_mailster" ><img src="<?php echo 'components/com_mailster/assets/images/biglogo.png'; ?>" height="200" alt="Mailster Logo" align="right"></a>
							</td>
						</tr>	
						<tr>
                            <td valign="top" width="20%">
					    		<a href="http://www.brandt-solutions.de" target="_blank"><img src="<?php echo 'components/com_mailster/assets/images/logo_hbit.png'; ?>" height="80" alt="Brandt IT Solutions Logo" align="left"></a>
							</td>
                            <td valign="top" width="50%">
					       	 	<strong>a product of <a href="http://www.brandt-solutions.de" target="_blank">Brandt IT Solutions</a></strong>
							</td>
						</tr>	
					</table>
					<table bgcolor="<?php echo $msgcolor; ?>" width ="100%">
						<tr style="height:30px">
					    	<td width="30px"><img src="<?php echo $imgSrc; ?>" height="20px" width="20px"></td>
					    	<td><font size="2"><b><?php echo $msgtext; ?></b></font></td>
						</tr>
						<?php
	
						
						// Deactivate all mailing lists as part of the update process
						$res = array();
						$inclPath = JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster'.DS.'mailster'.DS.'mail'.DS.'MailingListUtils.php';	
						require_once($inclPath);
						$mailingListUtils = new MstMailingListUtils();
						$activeMailingLists = $mailingListUtils->getActiveMailingLists();
						$log->info(count($activeMailingLists).' active mailing lists', MstConsts::LOGENTRY_INSTALLER);
						foreach($activeMailingLists AS $currMailList){
							$log->info('ID: '.$currMailList->id.' ('.$currMailList->name.')', MstConsts::LOGENTRY_INSTALLER);
						}
						$deactivationOk = $mailingListUtils->deactivateAllMailingLists();
					
						$msg = ($deactivationOk ? JText::_( 'COM_MAILSTER_MAILING_LISTS_SUCCESSFULLY_DEACTIVATED' ) : JText::_( 'COM_MAILSTER_COULD_NOT_DEACTIVATE_THE_MAILING_LISTS' )); 
						$msgcolor = ($deactivationOk ? '#B0FFB0' : '#FFB0B0'); 
						$this->addMsgRow($msg, $msgcolor);
									
						
						$listsTbl 	= '#__mailster_lists';
						$logTbl		= '#__mailster_log';
						$mailsTbl 	= '#__mailster_mails';
                        $oa_mailsTbl 	= '#__mailster_oa_mails';
						$usersTbl 	= '#__mailster_users';
						$groupsTbl 	= '#__mailster_groups';
						$queueTbl 	= '#__mailster_queued_mails';
						$attachsTbl = '#__mailster_attachments';
                        $serversTbl	= '#__mailster_servers';
						$subscrTbl 	= '#__mailster_subscriptions';
						
						
						/* ----- Process Updates ----- */
						// Version 0.2.0
						$res = array();
								
						$res[] = $dbUtils->renameCol($listsTbl, 	'public_list', 		'public_registration');
						$res[] = $dbUtils->renameCol($listsTbl, 	'public_members', 	'public_senders');	
						$res[] = $dbUtils->renameCol($mailsTbl, 	'timestamp', 		'receive_timestamp');	
						
	// as of 0.2.5:		$res[] = $dbUtils->addColIfNotExists($listsTbl,	'recipients_send_only', 'TINYINT(1) NULL DEFAULT 0',	'public_registration');          
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'reply_to_sender', 		'TINYINT(1) NULL',				'subject_prefix');				
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'copy_to_sender', 		'TINYINT(1) NULL',				'reply_to_sender');				
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'disable_mail_footer', 	'TINYINT(1) NULL',				'copy_to_sender');
	// as of 0.2.5:		$res[] = $dbUtils->addColIfNotExists($listsTbl,	'custom_header', 		'TEXT NULL',					'mail_out_use_sec_auth');
	// as of 0.2.5:		$res[] = $dbUtils->addColIfNotExists($listsTbl,	'custom_footer', 		'TEXT NULL',					'custom_header');	
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'alibi_to_mail', 		'VARCHAR(45) NULL',				'disable_mail_footer');		
	// as of 0.2.5:		$res[] = $dbUtils->addColIfNotExists($listsTbl,	'use_bcc',		 		'TINYINT(1) NULL DEFAULT 1',	'alibi_to_mail');		
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'bcc_count',	 		'INT NULL DEFAULT 10',			'alibi_to_mail');	
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'max_send_attempts',	'INT NULL DEFAULT 5',			'bcc_count');
								
						$res[] = $dbUtils->addColIfNotExists($mailsTbl,	'blocked_mail',	 		'TINYINT(1) NULL',				'fwd_completed_timestamp');			
						$res[] = $dbUtils->addColIfNotExists($mailsTbl,	'bounced_mail', 		'TINYINT(1) NULL',				'blocked_mail');
						
						
						$ok = $this->checkUpdateSuccess('0.2.0', $res);						
											
						// Version 0.2.1
						$res = array();
						
						$res[] = $dbUtils->addColIfNotExists($usersTbl,	'notes', 		'VARCHAR(45) NULL',		'email');
											
						$ok = $this->checkUpdateSuccess('0.2.1', $res);		
						
						// Version 0.2.5 / 0.3.0
						$res = array();	
						
						$res[] = $dbUtils->renameCol($listsTbl, 	'use_bcc', 			'addressing_mode');	
						$res[] = $dbUtils->renameCol($listsTbl, 	'custom_header', 	'custom_header_plain');	
						$res[] = $dbUtils->renameCol($listsTbl, 	'custom_footer', 	'custom_footer_plain');	
						$res[] = $dbUtils->renameCol($listsTbl, 	'mail_adress',	 	'list_mail');	
						
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'addressing_mode', 		'TINYINT(1) NULL DEFAULT 1',	'alibi_to_mail');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'filter_mails', 		'TINYINT(1) NULL DEFAULT 0',	'max_send_attempts');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'clean_up_subject', 	'TINYINT(1) NULL DEFAULT 1',	'filter_mails');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'sending_public',		'TINYINT(1) NULL DEFAULT 1',	'public_registration');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'sending_recipients',	'TINYINT(1) NULL DEFAULT 0',	'sending_public');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'sending_admin',		'TINYINT(1) NULL DEFAULT 0',	'sending_recipients');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'sending_group',		'TINYINT(1) NULL DEFAULT 0',	'sending_admin');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'sending_group_id',		'INT NULL',						'sending_group');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'lock_id',				'INT NULL',						'clean_up_subject');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'is_locked',			'TINYINT(1) NULL DEFAULT 0',	'lock_id');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'last_lock',			'TIMESTAMP NULL',				'is_locked');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'last_check',			'TIMESTAMP NULL',				'last_lock');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'cstate',				'INT NULL DEFAULT 0',			'last_check');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'custom_header_plain',	'TEXT NULL',					'cstate');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'custom_footer_plain',	'TEXT NULL',					'custom_header_plain');					
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'custom_header_html',	'TEXT NULL',					'custom_footer_plain');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'custom_footer_html',	'TEXT NULL',					'custom_header_html');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'mail_format_conv',		'INT NULL DEFAULT 1',			'custom_footer_html');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'mail_format_altbody',	'TINYINT(1) NULL DEFAULT 1',	'mail_format_conv');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'archive_mode',			'INT NULL DEFAULT 0',			'addressing_mode');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'bounce_mode',			'INT NULL DEFAULT 0',			'archive_mode');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,	'bounce_mail',			'VARCHAR(255) NULL',			'bounce_mode');
											
						$res[] = $dbUtils->addColIfNotExists($mailsTbl,	'html', 			'TEXT NULL',					'body');					
						$res[] = $dbUtils->addColIfNotExists($mailsTbl,	'message_id', 		'VARCHAR(255) NULL',			'list_id');
						$res[] = $dbUtils->addColIfNotExists($mailsTbl,	'in_reply_to', 		'VARCHAR(255) NULL',			'message_id');
						$res[] = $dbUtils->addColIfNotExists($mailsTbl,	'references_to',	'VARCHAR(255) NULL',			'in_reply_to');
						$res[] = $dbUtils->addColIfNotExists($mailsTbl,	'hashkey', 			'VARCHAR(45) NULL',				'list_id');
						$res[] = $dbUtils->addColIfNotExists($mailsTbl,	'thread_id', 		'VARCHAR(45) NOT NULL',			'list_id');	
						$res[] = $dbUtils->addColIfNotExists($mailsTbl,	'no_content', 		'TINYINT(1) NULL DEFAULT 0',	'bounced_mail');						
						
						$res[] = $dbUtils->changeColType($listsTbl, 'admin_mail', 		'VARCHAR(255)');
						$res[] = $dbUtils->changeColType($listsTbl, 'list_mail', 		'VARCHAR(255)');
						$res[] = $dbUtils->changeColType($listsTbl, 'mail_in_user', 	'VARCHAR(255)');
						$res[] = $dbUtils->changeColType($listsTbl, 'mail_out_user', 	'VARCHAR(255)');
						$res[] = $dbUtils->changeColType($listsTbl, 'alibi_to_mail', 	'VARCHAR(255)');					
						$res[] = $dbUtils->changeColType($usersTbl, 'email', 			'VARCHAR(255)');
						$res[] = $dbUtils->changeColType($usersTbl, 'name', 			'VARCHAR(255)');
						$res[] = $dbUtils->changeColType($usersTbl, 'notes', 			'VARCHAR(255)');					
						$res[] = $dbUtils->changeColType($queueTbl, 'email', 			'VARCHAR(255)');
						$res[] = $dbUtils->changeColType($queueTbl, 'name', 			'VARCHAR(255)');
						
						$res[] = $dbUtils->deleteColIfExists($listsTbl, 'enable_webform');
						
						$attachConv = $this->convertAttachmentStorage($mailsTbl, $attachsTbl);
						$senderConv = $this->convertSenderControl($listsTbl);
						$res[] = $attachConv;
						$res[] = $senderConv;
						
						if($attachConv > 0){
							$log->info('Converting Attachment Storage successful', MstConsts::LOGENTRY_INSTALLER);
							$delRes = $dbUtils->deleteCol($mailsTbl, 'attachments');
							$res[] = $delRes ? 1 : -1;
							$log->info('Dropping attachments row: ' 			. ($res ? 'ok' : 'failed'), MstConsts::LOGENTRY_INSTALLER);
						}	
										
						if($senderConv > 0){
							$log->info('Converting Sender Control successful', MstConsts::LOGENTRY_INSTALLER);
							$delRes1 = $dbUtils->deleteCol($listsTbl, 'recipients_send_only');
							$delRes2 = $dbUtils->deleteCol($listsTbl, 'public_senders');
							$res[] = $delRes1 ? 1 : -1;
							$res[] = $delRes2 ? 1 : -1;
							$log->info('Dropping recipients_send_only row: ' . ($delRes1 ? 'ok' : 'failed'), MstConsts::LOGENTRY_INSTALLER);
							$log->info('Dropping public_senders row: ' 		. ($delRes2 ? 'ok' : 'failed'), MstConsts::LOGENTRY_INSTALLER);
						}		
					
						$ok = $this->checkUpdateSuccess('0.3.0', $res);	
						
						
						// Version 0.3.1
						$res = array();	
						
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'mail_from_mode',		'TINYINT(1) NULL DEFAULT 0',	'addressing_mode');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'name_from_mode',		'TINYINT(1) NULL DEFAULT 0',	'mail_from_mode');					
						
						$res[] = $dbUtils->addColIfNotExists($queueTbl,		'lock_id',				'INT NULL DEFAULT 0',			'error_count');
						$res[] = $dbUtils->addColIfNotExists($queueTbl,		'is_locked',			'TINYINT(1) NULL DEFAULT 0',	'lock_id');
						$res[] = $dbUtils->addColIfNotExists($queueTbl,		'last_lock',			'TIMESTAMP NULL',				'is_locked');
											
						$res[] = $dbUtils->addColIfNotExists($attachsTbl,	'params',				'VARCHAR(255) NULL',			'subtype');
					
						$ok = $this->checkUpdateSuccess('0.3.1', $res);
						
						
						
						// Version 0.3.2
						$res = array();
						
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'mail_size_limit',			'INT NULL DEFAULT 0',			'cstate');
// as of 0.5.0:		    $res[] = $dbUtils->addColIfNotExists($listsTbl,		'throttle_hour',			'TINYINT(2) NULL DEFAULT 0',	'last_check');
// as of 0.5.0:		    $res[] = $dbUtils->addColIfNotExists($listsTbl,		'throttle_hour_cr',			'INT NULL DEFAULT 0',			'throttle_hour');
// as of 0.5.0:		    $res[] = $dbUtils->addColIfNotExists($listsTbl,		'throttle_hour_limit',		'INT NULL DEFAULT 0',			'throttle_hour_cr');*/
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'notify_not_fwd_sender',	'TINYINT(1) NULL DEFAULT 1',	'mail_size_limit');
					
						$ok = $this->checkUpdateSuccess('0.3.2', $res);
	
						
						
						// Version 0.3.7
						$res = array();
						
						$res[] = $dbUtils->renameCol($listsTbl, 			'allow_registration', 		'allow_subscribe');	
						
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'allow_unsubscribe',		'TINYINT(1) NULL DEFAULT 1',	'allow_subscribe');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'save_send_reports',		'INT NULL DEFAULT 7',			'notify_not_fwd_sender');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'subscribe_mode',			'INT NULL DEFAULT 1',			'save_send_reports');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'unsubscribe_mode',			'INT NULL DEFAULT 1',			'subscribe_mode');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'welcome_msg',				'INT NULL DEFAULT 1',			'unsubscribe_mode');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'goodbye_msg',				'INT NULL DEFAULT 1',			'welcome_msg');
						
						$res[] = $dbUtils->addColIfNotExists($mailsTbl,		'has_send_report',	 		'TINYINT(1) NULL DEFAULT 0',	'no_content');
						$res[] = $dbUtils->addColIfNotExists($mailsTbl,		'size_in_bytes',			'INT NULL DEFAULT -1',			'has_send_report');
						
						$ok = $this->checkUpdateSuccess('0.3.7', $res);
						
						
						// Version 0.3.8
						$res = array();
						$res[] = $dbUtils->addColIfNotExists($subscrTbl,	'add2group',			'INT NOT NULL DEFAULT 0',		'user_id');
						
						$ok = $this->checkUpdateSuccess('0.3.8', $res);
						
						
						// Version 1.0.0
						$res = array();
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'welcome_msg_admin',	 'INT NULL DEFAULT 0',			'welcome_msg');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'goodbye_msg_admin',	 'INT NULL DEFAULT 0',			'goodbye_msg');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'allow_digests',		 'TINYINT(1) NULL DEFAULT 0',	'goodbye_msg_admin');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'front_archive_access',	 'TINYINT(1) NULL DEFAULT 0',	'allow_digests');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'incl_orig_headers',	 'TINYINT(1) NOT NULL DEFAULT 0','bcc_count');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'last_mail_retrieved', 	 'TIMESTAMP NULL',				'last_check');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'last_mail_sent',		 'TIMESTAMP NULL',				'last_mail_retrieved');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'archive2article',		 'TINYINT(1) NOT NULL DEFAULT 0','archive_mode');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'archive2article_author','INT NOT NULL DEFAULT 0',		'archive2article');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'archive2article_cat',	 'INT NOT NULL DEFAULT 0',		'archive2article_author');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'archive2article_state', 'TINYINT(3) NOT NULL DEFAULT 1','archive2article_cat');
                        $res[] = $dbUtils->addColIfNotExists($listsTbl,		'archive_offline',       'TINYINT(1) NOT NULL DEFAULT 0','archive2article_state');
						$res[] = $dbUtils->addColIfNotExists($listsTbl,		'asset_id',	 			 'INT NOT NULL DEFAULT 0',	'id');
						$res[] = $dbUtils->addColIfNotExists($groupsTbl,	'is_joomla_group',		 'TINYINT(1) NOT NULL DEFAULT 0','name');
						$res[] = $dbUtils->addColIfNotExists($groupsTbl,	'joomla_group_id',		 'INT NOT NULL DEFAULT 0',		'is_joomla_group');
						$res[] = $dbUtils->addColIfNotExists($subscrTbl,	'digest_freq',			 'TINYINT(1) NOT NULL DEFAULT 0','add2group');
						$res[] = $dbUtils->addColIfNotExists($mailsTbl,		'orig_to_recips',		 'TEXT NULL',					'from_email');
						$res[] = $dbUtils->addColIfNotExists($mailsTbl,		'orig_cc_recips',	 	 'TEXT NULL',					'orig_to_recips');

						$res = array_merge($res, $this->changeLogFiles());

						$res = array_merge($res, $this->updateRecipientCacheStructure());

                        $res = array_merge($res, $this->migrateSendThrottling($listsTbl));
										
						$ok = $this->checkUpdateSuccess('1.0.0', $res);


                        // Version 1.1.0
                        $res = array();
                        $res[] = $dbUtils->addColIfNotExists($listsTbl,	    'allow_bulk_precedence',	'TINYINT(1) NOT NULL DEFAULT 0',	'filter_mails');
                        $res[] = $dbUtils->addColIfNotExists($mailsTbl,	    'sender_user_id',		    'INT NOT NULL DEFAULT 0',			'from_email');
                        $res[] = $dbUtils->addColIfNotExists($mailsTbl, 	'sender_is_joomla_user',	'TINYINT(1) NOT NULL DEFAULT 0',	'sender_user_id');
                        $res[] = $dbUtils->addColIfNotExists($oa_mailsTbl,	'sender_user_id',		    'INT NOT NULL DEFAULT 0',			'from_email');
                        $res[] = $dbUtils->addColIfNotExists($oa_mailsTbl,	'sender_is_joomla_user',	'TINYINT(1) NOT NULL DEFAULT 0',	'sender_user_id');

                        $res[] = $dbUtils->changeColType($mailsTbl,     'body', 'MEDIUMTEXT NULL');
                        $res[] = $dbUtils->changeColType($mailsTbl,     'html', 'MEDIUMTEXT NULL');
                        $res[] = $dbUtils->changeColType($oa_mailsTbl,  'body', 'MEDIUMTEXT NULL');
                        $res[] = $dbUtils->changeColType($oa_mailsTbl,  'html', 'MEDIUMTEXT NULL');

                        $ok = $this->checkUpdateSuccess('1.1.0', $res);


                        // Version 1.1.2
                        $res = array();
                        $res[] = $dbUtils->renameCol($listsTbl, 'use_joomla_mailer', 'use_cms_mailer');

                        $ok = $this->checkUpdateSuccess('1.1.2', $res);



                        // Version 1.2.3
                        $res = array();
                        $res[] = $dbUtils->addColIfNotExists($listsTbl,	    'server_inb_id',		    'INT NOT NULL DEFAULT 0',			'use_cms_mailer');
                        $res[] = $dbUtils->addColIfNotExists($listsTbl,	    'server_out_id',		    'INT NOT NULL DEFAULT 0',			'server_inb_id');
                        $res = array_merge($res, $this->migrateMailServerStructure($listsTbl, $serversTbl)); // server_inb_id and server_out_id need to be there before!

                        $ok = $this->checkUpdateSuccess('1.2.3', $res);


						/* ----- General Updates ----- */
						$res = array();
                        // Create Indexes that are not existent
						$res = array_merge($res, $this->createIndexes());
                        // Create and update default email provider server settings
                        $res = array_merge($res, $this->createAndUpdateDefaultEmailProviderSettings());
                        // Create Indexes that are not existent
                        $res = array_merge($res, $this->checkAndFixDBCollations());
						$ok = $this->checkUpdateSuccess('General updates', $res, true, false);

                        /* ----- Uninstall plugins that are not intended for the installed product edition  ----- */
                        if($this->_release_edition_current && ($this->_release_edition_current === "Mailster" || $this->_release_edition_current === "Mailster Essential")){
                            $nonFreeNoneEssentialPlugins = array(
                                'mailstersearch'	=> array('name'=>'Mailster Search Plugin', 'element'=>'mailstersearch', 'folder'=>'search', 'type'=>'plugin'),
                                'mailstercb'		=> array('name'=>'Mailster CB Plugin', 'element'=>'mailstercb', 'folder'=>'system', 'type'=>'plugin')
                            );
                            foreach($nonFreeNoneEssentialPlugins as $plugin => $pluginParams ){
                                $pluginName = $pluginParams['name'];
                                $pluginType = $pluginParams['type'];
                                $isInstalled = false;

                                if($pluginType === 'plugin'){
                                    $pluginElement = $pluginParams['element'];
                                    $pluginFolder = $pluginParams['folder'];
                                    $pluginId = $this->getPluginId($pluginFolder, $pluginElement);
                                    if($pluginId){
                                        $isInstalled = true;
                                        $log->info('Uninstall '.$pluginType.' "'.$pluginName.'" because of product version being: '.$this->_release_edition_current);
                                        $success = $this->deinstallPlugin($pluginElement, $pluginFolder);
                                    }
                                }elseif($pluginType === 'module'){
                                    $moduleName = $pluginParams['module'];
                                    $isInstalled = true; // assume it is installed
                                    $log->info('Uninstall '.$pluginType.' "'.$pluginName.'" because of product version being: '.$this->_release_edition_current);
                                    $success = $this->deinstallModules($moduleName);
                                }

                                if($isInstalled){
                                    if($success){
                                        $log->info('Successfully uninstalled '.$pluginType.' "'.$pluginName.'"');
                                    }else{
                                        $log->error('FAILED to uninstall '.$pluginType.' "'.$pluginName.'"');
                                    }
                                }
                            }
                        }
						?>
					</table>
				<table width="100%">
	
				<?php 
				foreach( $plugins as $plugin => $pluginParams ):
				
					if(!is_file($pluginPath.$plugin)&& !$pluginParams['mandatory']){
						$log->info('Skip installation of non mandatory and non existing plugin: '.$pluginParams['name']);
						continue; // non-existing and non-mandatory plugins are just skipped
					}
				
					$package = JInstallerHelper::unpack($pluginPath.$plugin);
					
					$pluginName = $pluginParams['name'];
					$pluginElement = $pluginParams['element'];
					$pluginFolder = $pluginParams['folder'];
					$isPlugin = $pluginParams['isPlugin'];
                    $defaultPublish = $pluginParams['defaultPublish'];

                    $setPluginPublishedStatusTo = $defaultPublish ? 1 : 0; // default setting is according to plugin parameter array
                    if($isPlugin == true){
                        $id = $this->getPluginId($pluginFolder, $pluginElement);
                        if($id > 0){ // plugin already installed?
                            $row = $this->getPluginObj($id); // get current status from DB
                            $setPluginPublishedStatusTo = $row->enabled;
                            $log->info('Plugin '.$pluginParams['name'].' already installed, current status: '.$setPluginPublishedStatusTo);
                        }else{
                            $log->info('Plugin '.$pluginParams['name'].' not yet installed, will set status after installation to: '.$setPluginPublishedStatusTo);
                        }
                    }
					
					if( $installer->install( $package['dir'] ) ){
						$msgcolor = "#B0FFB0";
					 	$msgtext  = $pluginName . ' ' . JText::_( 'COM_MAILSTER_INSTALLED_SUCCESSFUL' );
				 		$imgSrc = 'components/com_mailster/assets/images/16-tick.png';
						$log->info('Installation of plugin '.$pluginParams['name'].' successful');
					}
					else{
						$msgcolor = "#FFB0B0";
					 	$msgtext  = JText::_( 'COM_MAILSTER_INSTALL_ERROR' ) . ' ' . $pluginName;
				 		$imgSrc = 'components/com_mailster/assets/images/16-publish_x.png';
						$log->error('Installation of plugin '.$pluginParams['name'].' failed');
					}
					
				 ?>
					
				<tr bgcolor="<?php echo $msgcolor; ?>" style="height:30px">
			    	<td width="30px"><img src="<?php echo $imgSrc; ?>" height="20px" width="20px"></td>
			    	<td width="350px"><font size="2"><b><?php echo $msgtext; ?></b></font></td>
				<?php
					JInstallerHelper::cleanupInstall( $pluginPath.$plugin, $package['dir'] ); 
					if($isPlugin == true){
                        $id = $this->getPluginId($pluginFolder, $pluginElement);
                        if ($id){
                            $row = $this->getPluginObj($id);
                            if($row->load($id)){ // load it
                                $row->enabled = $setPluginPublishedStatusTo; // might get (re-)enabled right away
                                if( $row->store() ){ // save it
                                    $msgcolor = "#C0FFC0";
                                    $msgtext  = ($setPluginPublishedStatusTo > 0) ? JText::_( 'COM_MAILSTER_PLUGIN_PUBLISHED_SUCCESSFUL' ) : '';
                                }else{
                                    $msgcolor = "#FFC0C0";
                                    $msgtext  = JText::_( 'COM_MAILSTER_PLUGIN_PUBLISH_ERROR' ). ': <br />' . $row->getError();
                                    $log->error('Failed to publish plugin '.$pluginParams['name']);
                                }
                                ?>
                                <td bgcolor="<?php echo $msgcolor; ?>" width="300px" style="text-align:center;"><font size="2"><b><?php echo $msgtext; ?></b></font></td>
                                <td>&nbsp;</td>
                                <?php
                            }
                        }
					}else{?>
							<td>&nbsp;</td>
							<td>&nbsp;</td>
							<?php 	
					}
					?>					
					</tr>
					<?php 
				endforeach; 
	
				// re-activate mailing lists
				$mailingListReactivationFailed = false;
				foreach($activeMailingLists AS $currMailList){
					$activateSuccess = $mailingListUtils->activateMailingList($currMailList->id);
					$log->info('Re-activation of list ID: '.$currMailList->id.' ('.$currMailList->name.') '.($activateSuccess?'successful':'failed'), MstConsts::LOGENTRY_INSTALLER);
					if(!$activateSuccess){
						$mailingListReactivationFailed = true;
					}
				}
				$msg = ($mailingListReactivationFailed ? JText::_( 'COM_MAILSTER_MAILING_LISTS_REACTIVATION_FAILED' ) : JText::_( 'COM_MAILSTER_MAILING_LISTS_SUCCESSFULLY_REACTIVATED' )); 
				$msgcolor = ($mailingListReactivationFailed ? '#FFB0B0' : '#B0FFB0');
				$this->addMsgRow($msg, $msgcolor, false, 2);
				if($deactivationOk && $mailingListReactivationFailed){
					$this->addMsgRow(JText::_( 'COM_MAILSTER_DO_NOT_FORGET_TO_REACTIVTE_MAILING_LISTS' ), '#FFFF00', true);
				}
		?>
				</table>
			</center>
		<?php
		
			$log->info('FINISHED POST INSTALL OPERATIONS', MstConsts::LOGENTRY_INSTALLER);		
			
			global $mailster_install_running;
			$mailster_install_running = 0;
			
			return true; // install success
		}


        function uninstallScript($type, $parent){
            jimport( 'joomla.filesystem.folder' );
            jimport('joomla.installer.helper');

            $lang = JFactory::getLanguage();
            $lang->load('com_mailster', JPATH_ADMINISTRATOR);

            $installer = new JInstaller();
            $installer->setOverwrite(true);
            $plugins = array(
                'mailster'			=> array('name'=>'Mailster Mail Plugin', 'element'=>'mailster', 'folder'=>'system', 'type'=>'plugin'),
                'mailstersubscriber'=> array('name'=>'Mailster Subscriber Plugin', 'element'=>'mailstersubscriber', 'folder'=>'content', 'type'=>'plugin'),
                'mailsterprofile'	=> array('name'=>'Mailster Profile Plugin', 'element'=>'mailsterprofile', 'folder'=>'system', 'type'=>'plugin'),
                'mailstersearch'	=> array('name'=>'Mailster Search Plugin', 'element'=>'mailstersearch', 'folder'=>'search', 'type'=>'plugin'),
                'mailstercb'		=> array('name'=>'Mailster CB Plugin', 'element'=>'mailstercb', 'folder'=>'system', 'type'=>'plugin'),
                'subscribermodule'	=> array('name'=>'Mailster Subscriber Module', 'module'=>'mod_mailster_subscriber', 'type'=>'module')
            );
            $msgcolor = "#B0FFB0";
            $msgtext  = "Mailster Component " . JText::_( 'COM_MAILSTER_UNINSTALLED_SUCCESSFUL' );
            $imgSrc = 'components/com_mailster/assets/images/16-tick.png';
            ?>
            <center>
                <table bgcolor="<?php echo $msgcolor; ?>" width ="100%">
                    <tr style="height:30px">
                        <td width="30px"><img src="<?php echo $imgSrc; ?>" height="20px" width="20px"></td>
                        <td><font size="2"><b><?php echo $msgtext; ?></b></font></td>
                    </tr>
                </table>
                <?php
                foreach( $plugins as $plugin => $pluginParams ):
                    $pluginName = $pluginParams['name'];
                    $type = $pluginParams['type'];

                    $success = false;

                    if($type === 'plugin'){
                        $pluginElement = $pluginParams['element'];
                        $pluginFolder = $pluginParams['folder'];
                        $success = $this->deinstallPlugin($pluginElement, $pluginFolder);
                    }elseif($type === 'module'){
                        $moduleName = $pluginParams['module'];
                        $success = $this->deinstallModules($moduleName);
                    }
                    if( $success ){
                        $msgcolor = "#B0FFB0";
                        $msgtext  = $pluginName . ' ' . JText::_( 'COM_MAILSTER_UNINSTALLED_SUCCESSFUL' );
                        $imgSrc = 'components/com_mailster/assets/images/16-tick.png';
                    }else{
                        $msgcolor = "#FFB0B0";
                        $msgtext  = JText::_( 'COM_MAILSTER_UNINSTALL_ERROR' ) . ' ' . $pluginName;
                        $imgSrc = 'components/com_mailster/assets/images/16-publish_x.png';
                    }
                    ?>
                    <table bgcolor="<?php echo $msgcolor; ?>" width ="100%">
                        <tr style="height:30px">
                            <td width="30px"><img src="<?php echo $imgSrc; ?>" height="20px" width="20px"></td>
                            <td><font size="2"><b><?php echo $msgtext; ?></b></font></td>
                        </tr>
                    </table>
                <?php
                endforeach;
                ?>
            </center>
        <?php
            return true; // uninstall success
        }

        function deinstallPlugin($pluginElement, $pluginFolder){
            jimport( 'joomla.filesystem.folder' );
            jimport('joomla.installer.helper');
            $installer = new JInstaller();
            $installer->setOverwrite(true);
            $db = JFactory::getDBO();

            $query = 'SELECT `extension_id` AS `plg_id`, `client_id` AS `cli_id`'
                . ' FROM `#__extensions`'
                . ' WHERE `type` = '.$db->Quote('plugin')
                . ' AND folder = '.$db->Quote($pluginFolder)
                . ' AND element = '.$db->Quote($pluginElement);

            $db->setQuery($query);
            $success = false;
            if( $db->Query()){
                $pluginObj = $db->loadObject();
                if($pluginObj){
                    $id = $pluginObj->plg_id;
                    $client_id = $pluginObj->cli_id;
                    if($id > 0){
                        $success = $installer->uninstall('plugin', $id, $client_id );
                    }
                }
            }
            return $success;
        }

        function deinstallModules($moduleName){
            jimport( 'joomla.filesystem.folder' );
            jimport('joomla.installer.helper');
            $installer = new JInstaller();
            $installer->setOverwrite(true);

            $db = JFactory::getDBO();
                $query = 'SELECT `extension_id` AS `mod_id`, `client_id` AS `cli_id`'
                    . ' FROM `#__extensions`'
                    . ' WHERE `type` = '.$db->Quote('module')
                    . ' AND element = '.$db->Quote($moduleName);

            $success = false;
            $db->setQuery($query);
            if( $db->Query()){
                $modules = $db->loadObjectList();
                for($i=0;$i<count($modules);$i++){
                    $module = &$modules[$i];
                    $installer = new JInstaller();
                    $success = $installer->uninstall('module', $module->mod_id, $module->cli_id);
                }
            }
            return $success;
        }

        function getPluginId($pluginFolder, $pluginElement){
            $db = JFactory::getDBO();
            $query = 'SELECT `extension_id`' .
                ' FROM `#__extensions`' .
                ' WHERE folder = '.$db->Quote($pluginFolder) .
                ' AND element = '.$db->Quote($pluginElement);
            $row = JTable::getInstance('extension');
            $db->setQuery($query);
            $id = 0;
            if( $db->Query()){
                $id = $db->loadResult();
            }
            return $id;
        }

        function getPluginObj($pluginId){
            $db = JFactory::getDBO();
            $row = JTable::getInstance('extension');
            $row->load($pluginId);
            return $row;
        }

        function checkAndFixDBCollations(){
            $log = MstFactory::getLogger();
            $dbUtils = new MstDBUtils();
            $res = array();

            $cEmailCollation = $dbUtils->getCollation('#__users', 'email');
            $cNameCollation = $dbUtils->getCollation('#__users', 'name');
            $cGroupNameCollation = $dbUtils->getCollation('#__usergroups', 'title');
            $mEmailCollation = $dbUtils->getCollation('#__mailster_users', 'email');
            $mNameCollation = $dbUtils->getCollation('#__mailster_users', 'name');
            $mGroupNameCollation = $dbUtils->getCollation('#__mailster_groups', 'name');

            $collationOk = ( ($cEmailCollation === $mEmailCollation) && ($cNameCollation === $mNameCollation) && ($cGroupNameCollation === $mGroupNameCollation) );

            if($collationOk){
                $log->info('checkAndFixDBCollations() No need to fix DB Collations, all good', MstConsts::LOGENTRY_INSTALLER);
            }else{
                $log->error('checkAndFixDBCollations() DB Collation not okay, we have to fix it', MstConsts::LOGENTRY_INSTALLER);
            }

            if($cEmailCollation !== $mEmailCollation){
                $log->error('checkAndFixDBCollations() Collation not okay: users->email('.$cEmailCollation.') VS mailster_users->email('.$mEmailCollation.')', MstConsts::LOGENTRY_INSTALLER);
                $chgCollationRes = $dbUtils->alterCollation('#__mailster_users', 'email', $cEmailCollation);
                if($chgCollationRes >= 0){
                    $log->info('checkAndFixDBCollations() Changed mailster_users->email collation to Joomla cores collation: '.$cEmailCollation, MstConsts::LOGENTRY_INSTALLER);
                    $res[] = 1;
                }else{
                    $log->error('checkAndFixDBCollations() Failed to change mailster_users->email collation to Joomla cores collation: '.$cEmailCollation, MstConsts::LOGENTRY_INSTALLER);
                    $res[] = -1;
                }
                $mEmailCollation = $dbUtils->getCollation('#__mailster_users', 'email');
                $log->info('checkAndFixDBCollations() mailster_users->email collation afterwards: '.$mEmailCollation, MstConsts::LOGENTRY_INSTALLER);
            }else{
                $res[] = 0;
            }
            if($cNameCollation !== $mNameCollation){
                $log->error('checkAndFixDBCollations() Collation not okay: users->name('.$cNameCollation.') VS mailster_users->name('.$mNameCollation.')', MstConsts::LOGENTRY_INSTALLER);
                $chgCollationRes = $dbUtils->alterCollation('#__mailster_users', 'name', $cNameCollation);
                if($chgCollationRes >= 0){
                    $log->info('checkAndFixDBCollations() Changed mailster_users->name collation to Joomla cores collation: '.$cNameCollation, MstConsts::LOGENTRY_INSTALLER);
                    $res[] = 1;
                }else{
                    $log->error('checkAndFixDBCollations() Failed to change mailster_users->name collation to Joomla cores collation: '.$cNameCollation, MstConsts::LOGENTRY_INSTALLER);
                    $res[] = -1;
                }
                $mNameCollation = $dbUtils->getCollation('#__mailster_users', 'name');
                $log->info('checkAndFixDBCollations() mailster_users->name collation afterwards: '.$mNameCollation, MstConsts::LOGENTRY_INSTALLER);
            }else{
                $res[] = 0;
            }
            if($cGroupNameCollation !== $mGroupNameCollation){
                $log->error('checkAndFixDBCollations() Collation not okay: usergroups->title('.$cGroupNameCollation.') VS mailster_groups->name('.$mGroupNameCollation.')', MstConsts::LOGENTRY_INSTALLER);
                $chgCollationRes = $dbUtils->alterCollation('#__mailster_groups', 'name', $cGroupNameCollation);
                if($chgCollationRes >= 0){
                    $log->info('checkAndFixDBCollations() Changed mailster_groups->name collation to Joomla cores collation: '.$cGroupNameCollation, MstConsts::LOGENTRY_INSTALLER);
                    $res[] = 1;
                }else{
                    $log->error('checkAndFixDBCollations() Failed to change mailster_groups->name collation to Joomla cores collation: '.$cGroupNameCollation, MstConsts::LOGENTRY_INSTALLER);
                    $res[] = -1;
                }
                $mGroupNameCollation = $dbUtils->getCollation('#__mailster_groups', 'name');
                $log->info('checkAndFixDBCollations() mailster_groups->name collation afterwards: '.$mGroupNameCollation, MstConsts::LOGENTRY_INSTALLER);
            }else{
                $res[] = 0;
            }

            return $res;
        }

        function createAndUpdateDefaultEmailProviderSettings(){
            require_once(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster'.DS.'mailster'.DS.'mail'.DS.'ServerSettings.php');
            /*
                 const SERVER_PROVIDER_TYPE_USER_SPECIFIC = 0;
                 const SERVER_PROVIDER_AOL_INBOX = 1;
                 const SERVER_PROVIDER_AOL_SENDER = 2;
                 const SERVER_PROVIDER_GMAIL_INBOX = 3;
                 const SERVER_PROVIDER_GMAIL_SENDER = 4;
                 const SERVER_PROVIDER_ONE_COM_INBOX = 5;
                 const SERVER_PROVIDER_ONE_COM_SENDER = 6;
                 const SERVER_PROVIDER_OUTLOOK_INBOX = 7;
                 const SERVER_PROVIDER_OUTLOOK_SENDER = 8;
                 const SERVER_PROVIDER_YAHOO_INBOX = 9;
                 const SERVER_PROVIDER_YAHOO_SENDER = 10;
             */
            $res = array();
            $res[] = $this->createUpdateEmailProviderSetting(1, MstServerSettings::getAOLInbox());
            $res[] = $this->createUpdateEmailProviderSetting(2, MstServerSettings::getAOLSMTP());
            $res[] = $this->createUpdateEmailProviderSetting(3, MstServerSettings::getGoogleMailInbox());
            $res[] = $this->createUpdateEmailProviderSetting(4, MstServerSettings::getGoogleMailSMTP());
            $res[] = $this->createUpdateEmailProviderSetting(5, MstServerSettings::getOneComInbox());
            $res[] = $this->createUpdateEmailProviderSetting(6, MstServerSettings::getOneComSMTP());
            $res[] = $this->createUpdateEmailProviderSetting(7, MstServerSettings::getOutlookInbox());
            $res[] = $this->createUpdateEmailProviderSetting(8, MstServerSettings::getOutlookSMTP());
            $res[] = $this->createUpdateEmailProviderSetting(9, MstServerSettings::getYahooInbox());
            $res[] = $this->createUpdateEmailProviderSetting(10, MstServerSettings::getYahooSMTP());
            return $res;
        }

        function createUpdateEmailProviderSetting($providerType, $defaultSettings){
            $log = MstFactory::getLogger();
            require_once(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster'.DS.'tables'.DS.'mailster_servers.php');
            require_once(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster'.DS.'models'.DS.'server.php');
            require_once(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster'.DS.'models'.DS.'servers.php');
            $serverModel = new MailsterModelServer();
            $serversModel = new MailsterModelServers();
            $serverSetting = $serversModel->getProviderTypeSettings($providerType);
            $needToStoreSettings = false;

            if(is_array($serverSetting)){
                if(count($serverSetting)){
                    $serverSetting = $serverSetting[0];
                }
            }

            if(!$serverSetting || ($serverSetting->id > 0 && $serverSetting->user_edited < 1)){
                $needToStoreSettings = true;
            }

            if(!$serverSetting){
                $log->info('Create default mail provider settings for provider type '.$providerType.' using default settings: '.print_r($defaultSettings, true), MstConsts::LOGENTRY_INSTALLER);
                // need to create provider's email settings
                $serverSetting = new stdClass();
                $serverSetting->id = 0;
                $serverSetting->published = 1;
                $serverSetting->provider_type = $providerType;
                $serverSetting->user_edited = 0;
            }

            if($serverSetting->id == 0 || ($serverSetting->id > 0 && $serverSetting->user_edited < 1)){
                // was not updated, so we can update existing settings
                $serverSetting->name = $defaultSettings->name;
                $serverSetting->server_type = $defaultSettings->server_type;
                $serverSetting->server_host = $defaultSettings->server_host;
                $serverSetting->server_port = $defaultSettings->server_port;
                $serverSetting->secure_protocol = $defaultSettings->secure_protocol;
                $serverSetting->secure_authentication = $defaultSettings->secure_authentication;
                $serverSetting->protocol = $defaultSettings->protocol;
                $serverSetting->connection_parameter = $defaultSettings->connection_parameter;
                $serverSetting->api_key1 = $defaultSettings->api_key1;
                $serverSetting->api_key2 = $defaultSettings->api_key2;
                $serverSetting->api_endpoint = $defaultSettings->api_endpoint;
            }

            if($needToStoreSettings){
                $log->debug('Store/Update provider settings: '.$serverSetting->name, MstConsts::LOGENTRY_INSTALLER);
                if($serverModel->store($serverSetting, false)){
                    return 1;
                }else{
                    return -1;
                }
            }else{
                $log->debug('Can/should not update settings: '.print_r($serverSetting, true), MstConsts::LOGENTRY_INSTALLER);
                return 0;
            }
        }

		function createIndexes(){		
			$inclPath = JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster'.DS.'mailster'.DS.'utils'.DS.'DBUtils.php';	
			require_once($inclPath);
			$dbUtils = new MstDBUtils();

			$digestsTbl 	= '#__mailster_digests';
			$digestQueueTbl = '#__mailster_digest_queue';
			$groupUsersTbl 	= '#__mailster_group_users';
			$listGroupsTbl 	= '#__mailster_list_groups';
			$listMembersTbl = '#__mailster_list_members';
            $listStatsTbl   = '#__mailster_list_stats';
			$logTbl			= '#__mailster_log';
			$mailsTbl 		= '#__mailster_mails';
			$queuedMailsTbl = '#__mailster_queued_mails';
			$sendReports 	= '#__mailster_send_reports';
            $serversTbl 	= '#__mailster_servers';
			$threadsTbl 	= '#__mailster_threads';
			$usersTbl 		= '#__mailster_users';
			
			$res = array();
			
			$cols = array();
			$cols[] = 'group_id';
			$index = 'group_id';
			$res[] 	= $dbUtils->createIndexIfNotExists($groupUsersTbl, $index, $cols);
			$cols = array();
			$cols[] = 'user_id';
			$cols[] = 'is_joomla_user';
			$index = 'user';
			$res[] 	= $dbUtils->createIndexIfNotExists($groupUsersTbl, $index, $cols);
			
			$cols = array();
			$cols[] = 'list_id';
			$index = 'list_id';
			$res[] 	= $dbUtils->createIndexIfNotExists($listGroupsTbl, $index, $cols);		
			$cols = array();
			$cols[] = 'list_id';
			$cols[] = 'group_id';
			$index = 'list_group';
			$res[] 	= $dbUtils->createIndexIfNotExists($listGroupsTbl, $index, $cols);
			
			$cols = array();
			$cols[] = 'list_id';
			$index = 'list_id';
			$res[] 	= $dbUtils->createIndexIfNotExists($listMembersTbl, $index, $cols);
			$cols = array();
			$cols[] = 'user_id';
			$cols[] = 'is_joomla_user';
			$index = 'user';
			$res[] 	= $dbUtils->createIndexIfNotExists($listMembersTbl, $index, $cols);
			
			$cols = array();
			$cols[] = 'list_id';
			$index = 'list_id';
			$res[] 	= $dbUtils->createIndexIfNotExists($mailsTbl, $index, $cols);
			$cols = array();
			$cols[] = 'blocked_mail';
			$index = 'blocked_mail';
			$res[] 	= $dbUtils->createIndexIfNotExists($mailsTbl, $index, $cols);
			$cols = array();
			$cols[] = 'bounced_mail';
			$index = 'bounced_mail';
			$res[] 	= $dbUtils->createIndexIfNotExists($mailsTbl, $index, $cols);
			$cols = array();
			$cols[] = 'blocked_mail';
			$cols[] = 'bounced_mail';
			$index = 'blocked_bounced';
			$res[] 	= $dbUtils->createIndexIfNotExists($mailsTbl, $index, $cols);
			$cols = array();
			$cols[] = 'fwd_errors';
			$index = 'fwd_errors';
			$res[] 	= $dbUtils->createIndexIfNotExists($mailsTbl, $index, $cols);
			$cols = array();
			$cols[] = 'fwd_completed';
			$index = 'fwd_completed';
			$res[] 	= $dbUtils->createIndexIfNotExists($mailsTbl, $index, $cols);
			$cols = array();
			$cols[] = 'thread_id';
			$index = 'thread_id';
			$res[] 	= $dbUtils->createIndexIfNotExists($mailsTbl, $index, $cols);
			
			$cols = array();
			$cols[] = 'mail_id';
			$index = 'mail_id';
			$res[] 	= $dbUtils->createIndexIfNotExists($queuedMailsTbl, $index, $cols);
			$cols = array();
			$cols[] = 'mail_id';
			$cols[] = 'is_locked';
			$index = 'mail_queued_locked';
			$res[] 	= $dbUtils->createIndexIfNotExists($queuedMailsTbl, $index, $cols);
			$cols = array();
			$cols[] = 'mail_id';
			$cols[] = 'email';
			$index = 'mail_queued';
			$res[] 	= $dbUtils->createIndexIfNotExists($queuedMailsTbl, $index, $cols);
			
			$cols = array();
			$cols[] = 'email';
			$index = 'email';
			$res[] 	= $dbUtils->createIndexIfNotExists($usersTbl, $index, $cols);
					
			$cols = array();
			$cols[] = 'ref_message_id';
			$index = 'ref_message_id';
			$res[] 	= $dbUtils->createIndexIfNotExists($threadsTbl, $index, $cols);
			
			$cols = array();
			$cols[] = 'type';
			$index = 'log_type';
			$res[] 	= $dbUtils->createIndexIfNotExists($logTbl, $index, $cols);		
			$cols = array();
			$cols[] = 'log_time';
			$index = 'log_time';
			$res[] 	= $dbUtils->createIndexIfNotExists($logTbl, $index, $cols);
			
			$cols = array();
			$cols[] = 'list_id';
			$index = 'list_id';
			$res[] 	= $dbUtils->createIndexIfNotExists($digestsTbl, $index, $cols);
			
			$cols = array();
			$cols[] = 'user_id';
			$cols[] = 'is_joomla_user';
			$index = 'digest_user';
			$res[] 	= $dbUtils->createIndexIfNotExists($digestsTbl, $index, $cols);

			$cols = array();
			$cols[] = 'digest_id';
			$index = 'digest_id';
			$res[] 	= $dbUtils->createIndexIfNotExists($digestQueueTbl, $index, $cols);

			$cols = array();
			$cols[] = 'thread_id';
			$index = 'thread_id';
			$res[] 	= $dbUtils->createIndexIfNotExists($digestQueueTbl, $index, $cols);

			$cols = array();
			$cols[] = 'mail_id';
			$index = 'mail_id';
			$res[] 	= $dbUtils->createIndexIfNotExists($sendReports, $index, $cols);

            $cols = array();
            $cols[] = 'list_id';
            $index = 'list_id';
            $res[] 	= $dbUtils->createIndexIfNotExists($listStatsTbl, $index, $cols);

            $cols = array();
            $cols[] = 'server_type';
            $index = 'server_type';
            $res[] 	= $dbUtils->createIndexIfNotExists($serversTbl, $index, $cols);

            $cols = array();
            $cols[] = 'provider_type';
            $index = 'provider_type';
            $res[] 	= $dbUtils->createIndexIfNotExists($serversTbl, $index, $cols);
						
			return $res;
		}
	
		function convertAttachmentStorage($mailsTbl, $attachsTbl){	
			$log = MstFactory::getLogger();
			
			$inclPath = JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster'.DS.'mailster'.DS.'utils'.DS.'DBUtils.php';	
			require_once($inclPath);
			$dbUtils = new MstDBUtils();
				
			$srcFieldExists = $dbUtils->isColExisting($mailsTbl, 'attachments');
			$log->info('Convert Attachment Storage, source field exists: ' . ($srcFieldExists ? 'Yes' : 'No'), MstConsts::LOGENTRY_INSTALLER);
			
			if($srcFieldExists){
				$db = JFactory::getDBO();	
				$query = 'SELECT * FROM ' . $mailsTbl  . ' WHERE has_attachments = \'1\' ';
				$db->setQuery( $query );
				$errorMsg = '';
                try {
                    $result = $db->query();
                } catch (DatabaseException $e) {
                    $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                }
				if (!$result){
					echo $errorMsg;
					return -1;
				}		
				$attachMails = $db->loadObjectList();	
				$log->info('Attachment Storage conversion, found  ' . count($attachMails) . ' that need to be converted', MstConsts::LOGENTRY_INSTALLER);
				for($i=0; $i<count($attachMails); $i++){
					$mail = &$attachMails[$i];
					$attachsCSV = $mail->attachments;
					$attachs = explode(';', $attachsCSV);
					$log->info('Mail ' . $mail->id . ' has ' . count($attachs) . ' to store', MstConsts::LOGENTRY_INSTALLER);
					for($j=0; $j < count($attachs); $j++)
					{
						$fileInfos = explode('|', $attachs[$j]);
						$fileName = rawurldecode($fileInfos[0]);
						$filePath = $fileInfos[1];
						$query = 'INSERT INTO ' 
									. $attachsTbl 
									. '(id,' 
									. ' mail_id,' 
									. ' filename,' 
									. ' filepath,' 
									. ' content_id,' 
									. ' disposition)'
									. ' VALUES'
									. ' (NULL,'
									. '  \'' . $mail->id . '\','
									. ' ' . $db->quote($fileName) . ','
									. ' ' . $db->quote($filePath) . ','
									. ' NULL,'
									. ' \'0\''
									. ')';
						$db->setQuery($query);
						$result = $db->query();
						$attachId = $db->insertid(); 
						$log->info('Converted ' . $fileName . ' of mail ' . $mail->id . ', saved as attachment ' . $attachId, MstConsts::LOGENTRY_INSTALLER);
					}
				}			
				return 1;
			}
			return 0;
		}
		
		function convertSenderControl($listsTbl){
			$log = MstFactory::getLogger();
			
			$inclPath = JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster'.DS.'mailster'.DS.'utils'.DS.'DBUtils.php';	
			require_once($inclPath);
			$dbUtils = new MstDBUtils();
				
			$recipSendOnlyExists = $dbUtils->isColExisting($listsTbl, 'recipients_send_only');
			$publicSendersExists = $dbUtils->isColExisting($listsTbl, 'public_senders');
			$log->info('Need to convert sender control: ' . ($publicSendersExists ? 'Yes' : 'No'), MstConsts::LOGENTRY_INSTALLER);
			
			if($publicSendersExists){
				$db = JFactory::getDBO();	
				$query = 'SELECT * FROM ' . $listsTbl;
				$db->setQuery( $query );
				$errorMsg = '';
                try {
                    $result = $db->query();
                } catch (DatabaseException $e) {
                    $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                }
				if (!$result){
					return -1;
				}		
				$lists = $db->loadObjectList();	
				
				
				$log->info('Found  ' . count($lists) . ' lists that need to be converted', MstConsts::LOGENTRY_INSTALLER);
				for($i=0; $i<count($lists); $i++){
					$mList = &$lists[$i];
					$sending_public = 1;
					$sending_recipients = 0;
					$sending_admin = 0;
					$sending_group = 0;
					if($recipSendOnlyExists){
						if($mList->public_senders == 1){
							// everybody is allowed to send
							$sending_public = 1;
						}elseif(($mList->public_senders == 0) && ($mList->recipients_send_only == 0)){
							// only admins are allowed to send
							$sending_public = 0;
							$sending_admin = 1;					
						}else{
							// only recipients are allowed to send
							$sending_public = 0;
							$sending_recipients = 1;					
						}
					}else{
						if($mList->public_senders == 1){
							$sending_public = 1;
						}else{
							$sending_public = 0;
							$sending_recipients = 1;
						}
					}
					$query = 'UPDATE ' . $listsTbl . ' SET'
							. ' sending_public = \'' . $sending_public . '\','
							. ' sending_recipients = \'' . $sending_recipients . '\','
							. ' sending_admin = \'' . $sending_admin . '\','
							. ' sending_group = \'' . $sending_group . '\''
							. ' WHERE id = \'' . $mList->id . '\'';
						
					$db = JFactory::getDBO();
					$db->setQuery($query);
					$result	 = $db->query();
					$affRows = $db->getAffectedRows();	
					$log->info('Conversion of list ' . $mList->id 
								. ' done (public: ' . $sending_public 
								. ', recipients: ' . $sending_recipients 
								. ', admin: ' . $sending_admin .'), result: ' . $affRows, MstConsts::LOGENTRY_INSTALLER);
				}
				
				$log->info('Converted all lists', MstConsts::LOGENTRY_INSTALLER);						
				return 1;
			}
			return 0;
		}
		
		function changeLogFiles(){
			$log = MstFactory::getLogger();
			$log->debug('changeLogFiles()', MstConsts::LOGENTRY_INSTALLER);
			$logFileFolder = $log->getLogFileFolder();
			$oldInstallLogFile = $logFileFolder.DS.'mailster.install.log';
			$oldGeneralLogFile = $logFileFolder.DS.'mailster.log';

			$res = array();
			
			if(file_exists($oldInstallLogFile) && is_file($oldInstallLogFile)) {
				$log->debug('Old install log file ('.$oldInstallLogFile.') exists', MstConsts::LOGENTRY_INSTALLER);
				if(unlink($oldInstallLogFile)){
					$log->info('Old install log file ('.$oldInstallLogFile.') removed', MstConsts::LOGENTRY_INSTALLER);
					$res[] = 1;
				}else{
					$res[] = -1;
				}
			}else{
				$res[] = 0;
			}
			
			if(file_exists($oldGeneralLogFile) && is_file($oldGeneralLogFile)) {
				$log->debug('Old general log file ('.$oldGeneralLogFile.') exists', MstConsts::LOGENTRY_INSTALLER);
				if(unlink($oldGeneralLogFile)){
					$log->info('Old general log file ('.$oldGeneralLogFile.') removed', MstConsts::LOGENTRY_INSTALLER);
					$res[] = 1;
				}else{
					$res[] = -1;
				}
			}else{
				$res[] = 0;
			}
			
			return $res;
		}
		
		function updateRecipientCacheStructure(){
			$log = MstFactory::getLogger();
			$log->debug('updateRecipientCacheStructure()', MstConsts::LOGENTRY_INSTALLER);
			$res = array();
			$mListUtils = MstFactory::getMailingListUtils();
			$recipUtils = MstFactory::getRecipients();
			$lists = $mListUtils->getAllMailingLists();
			if(count($lists) > 0 ){
				foreach($lists AS $list){
					$recips = $recipUtils->getRecipients($list->id);
					if(count($recips)<=0){
						continue;
					}
					if(property_exists($recips[0], 'user_id')){
						$log->debug('No need to update recipient cache structure', MstConsts::LOGENTRY_INSTALLER);
						$res[] = 0; // no action needed
					}else{
						$log->info('Need to update recipient cache structure', MstConsts::LOGENTRY_INSTALLER);
						$recipUtils->recipientsUpdatedInAllLists(); // invalidate all cached recip info so that new propertys (user_id, is_joomla_user) are included in the future
						$res[] = 1;
					}
				}
			}else{
				$log->debug('No need to update recipient cache, no mailing lists!', MstConsts::LOGENTRY_INSTALLER);
				$res[] = 0; // no action needed
			}
			if(count($res)<=0){ // we could not find out for sure how the recipient cache is structured - there seem to be no recipients (and hence no caches)
				$log->info('Just to be sure update recipient cache structure', MstConsts::LOGENTRY_INSTALLER);
				$recipUtils->recipientsUpdatedInAllLists(); // invalidate all cached recip info so that new propertys (user_id, is_joomla_user) are included in the future
				$res[] = 1;
			}
			return $res;
		}

        function migrateMailServerStructure($listsTbl, $serversTbl){
            $log = MstFactory::getLogger();
            $dbUtils = new MstDBUtils();
            $res = array();
            $log->debug('migrateMailServerStructure()', MstConsts::LOGENTRY_INSTALLER);

            $isMigrationNeeded = $dbUtils->isColExisting($listsTbl, 'mail_in_host');

            if(!$isMigrationNeeded){
                $log->debug('migrateMailServerStructure: No need to migrate mail server structure', MstConsts::LOGENTRY_INSTALLER);
                $res[] = 0; // no action needed
                return $res; // exit
            }

            $db = JFactory::getDBO();
            $query = 'SELECT * FROM ' . $listsTbl;
            $db->setQuery( $query );
            $lists = array();
            try {
                $lists = $db->loadObjectList();
                $log->info('migrateMailServerStructure: '.count($lists).' mailing lists present', MstConsts::LOGENTRY_INSTALLER);
            } catch (DatabaseException $e) {
                $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                $log->error('migrateMailServerStructure: '.$errorMsg. '(query was: '.$query.')', MstConsts::LOGENTRY_INSTALLER);
            }

            $errorHappenedDuringExistingServerMigration = false;
            if(count($lists) <= 0){
                $log->info('No need to create / move email server information in servers table', MstConsts::LOGENTRY_INSTALLER);
            }else{
                $query = 'SELECT mail_in_host, mail_in_port,'
                            .' mail_in_use_secure, mail_in_use_sec_auth,'
                            .' mail_in_protocol, mail_in_params, count(*) AS used_by_lists_count'
                        .' FROM '.$listsTbl
                        .' GROUP BY mail_in_host, mail_in_port,'
                            .' mail_in_use_secure, mail_in_use_sec_auth,'
                            .' mail_in_protocol, mail_in_params';
                $db->setQuery( $query );
                try {
                    $serverConfigs = $db->loadObjectList();
                    $log->info('migrateMailServerStructure: '.count($serverConfigs).' different inbox server configuration present for '.count($lists).' lists', MstConsts::LOGENTRY_INSTALLER);
                } catch (DatabaseException $e) {
                    $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                    $log->error('migrateMailServerStructure: '.$errorMsg. '(query was: '.$query.')', MstConsts::LOGENTRY_INSTALLER);
                    $errorHappenedDuringExistingServerMigration = true;
                }
                $log->info('migrateMailServerStructure: migrating inbox settings: '.print_r($serverConfigs, true), MstConsts::LOGENTRY_INSTALLER);

                foreach($serverConfigs AS $serverConfig){
                    $log->debug('migrateMailServerStructure: Working with server configuration: '.print_r($serverConfig, true), MstConsts::LOGENTRY_INSTALLER);

                    /* Same as in MstConsts::SERVER_TYPE_MAIL_INBOX */
                    $server_type_mail_inbox = 0;

                    $query = 'INSERT INTO '
                        . $serversTbl . ' ('
                        . ' id,'
                        . ' published,'
                        . ' provider_type,'
                        . ' user_edited,'
                        . ' name,'
                        . ' server_type,'
                        . ' server_host,'
                        . ' server_port,'
                        . ' secure_protocol,'
                        . ' secure_authentication,'
                        . ' protocol,'
                        . ' connection_parameter,'
                        . ' api_key1,'
                        . ' api_key2,'
                        . ' api_endpoint)'
                        . ' VALUES'
                        . ' (NULL,'
                        . '  \'1\',' // is published
                        . '  \'0\',' // means user created
                        . '  \'1\',' // means user edited
                        . ' ' . $db->quote($serverConfig->mail_in_host.' ('.strtoupper($serverConfig->mail_in_protocol).')') . ','
                        . '  \'' . $server_type_mail_inbox . '\','
                        . ' ' . $db->quote($serverConfig->mail_in_host) . ','
                        . ' ' . $db->quote($serverConfig->mail_in_port) . ','
                        . ' ' . $db->quote($serverConfig->mail_in_use_secure) . ','
                        . ' ' . $db->quote($serverConfig->mail_in_use_sec_auth) . ','
                        . ' ' . $db->quote($serverConfig->mail_in_protocol) . ','
                        . ' ' . $db->quote($serverConfig->mail_in_params) . ','
                        . ' NULL,'
                        . ' NULL,'
                        . ' NULL'
                        . ')';
                    $db->setQuery($query);
                    try {
                        $result = $db->query();
                        $serverIdInbox = $db->insertid();
                        $log->debug('migrateMailServerStructure: server configuration stored as ID '.$serverIdInbox, MstConsts::LOGENTRY_INSTALLER);

                        $query = 'UPDATE ' . $listsTbl . ' SET'
                            . ' server_inb_id = \'' . $serverIdInbox . '\''
                            . ' WHERE mail_in_host =  '. $db->quote($serverConfig->mail_in_host)
                            . ' AND mail_in_port =  '. $db->quote($serverConfig->mail_in_port)
                            . ' AND mail_in_use_secure =  '. $db->quote($serverConfig->mail_in_use_secure)
                            . ' AND mail_in_use_sec_auth =  '. $db->quote($serverConfig->mail_in_use_sec_auth)
                            . ' AND mail_in_protocol =  '. $db->quote($serverConfig->mail_in_protocol)
                            . ' AND mail_in_params =  '. $db->quote($serverConfig->mail_in_params);

                        $db->setQuery($query);
                        $result	 = $db->query();
                        $affRows = $db->getAffectedRows();
                        $log->debug('migrateMailServerStructure: '.$affRows.' lists updated with server_inb_id = '.$serverIdInbox, MstConsts::LOGENTRY_INSTALLER);

                    } catch (DatabaseException $e) {
                        $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                        $log->error('migrateMailServerStructure: '.$errorMsg . '(query was: '.$query.')', MstConsts::LOGENTRY_INSTALLER);
                        $errorHappenedDuringExistingServerMigration = true;
                    }

                }

                $log->info('migrateMailServerStructure: Completed inbox handling');


                $query = 'SELECT mail_out_host, mail_out_port,'
                    .' mail_out_use_secure, mail_out_use_sec_auth,'
                    .' count(*) AS used_by_lists_count'
                    .' FROM '.$listsTbl
                    .' WHERE use_cms_mailer = \'0\''
                    .' GROUP BY mail_out_host, mail_out_port,'
                    .' mail_out_use_secure, mail_out_use_sec_auth';
                $db->setQuery( $query );
                try {
                    $serverConfigs = $db->loadObjectList();
                    $log->info('migrateMailServerStructure: '.count($serverConfigs).' different outgoing email server configuration present for '.count($lists).' lists (not including lists using CMS mailer)', MstConsts::LOGENTRY_INSTALLER);
                } catch (DatabaseException $e) {
                    $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                    $log->error('migrateMailServerStructure: '.$errorMsg. '(query was: '.$query.')', MstConsts::LOGENTRY_INSTALLER);
                    $errorHappenedDuringExistingServerMigration = true;
                }
                $log->info('migrateMailServerStructure: migrating outgoing email server settings: '.print_r($serverConfigs, true), MstConsts::LOGENTRY_INSTALLER);

                foreach($serverConfigs AS $serverConfig){
                    $log->debug('migrateMailServerStructure: Working with server configuration: '.print_r($serverConfig, true), MstConsts::LOGENTRY_INSTALLER);

                    /* Same as in MstConsts::SERVER_TYPE_SMTP */
                    $server_type_mail_smtp = 1;

                    $query = 'INSERT INTO '
                        . $serversTbl . ' ('
                        . ' id,'
                        . ' published,'
                        . ' provider_type,'
                        . ' user_edited,'
                        . ' name,'
                        . ' server_type,'
                        . ' server_host,'
                        . ' server_port,'
                        . ' secure_protocol,'
                        . ' secure_authentication,'
                        . ' protocol,'
                        . ' connection_parameter,'
                        . ' api_key1,'
                        . ' api_key2,'
                        . ' api_endpoint)'
                        . ' VALUES'
                        . ' (NULL,'
                        . '  \'1\',' // is published
                        . '  \'0\',' // means user created
                        . '  \'1\',' // means user edited
                        . ' ' . $db->quote($serverConfig->mail_out_host.' (SMTP)') . ','
                        . '  \'' . $server_type_mail_smtp . '\','
                        . ' ' . $db->quote($serverConfig->mail_out_host) . ','
                        . ' ' . $db->quote($serverConfig->mail_out_port) . ','
                        . ' ' . $db->quote($serverConfig->mail_out_use_secure) . ','
                        . ' ' . $db->quote($serverConfig->mail_out_use_sec_auth) . ','
                        . ' ' . $db->quote('smtp') . ','
                        . ' NULL,'
                        . ' NULL,'
                        . ' NULL,'
                        . ' NULL'
                        . ')';
                    $db->setQuery($query);
                    try {
                        $result = $db->query();
                        $serverIdOutgoing = $db->insertid();
                        $log->debug('migrateMailServerStructure: server configuration stored as ID '.$serverIdOutgoing, MstConsts::LOGENTRY_INSTALLER);

                        $query = 'UPDATE ' . $listsTbl . ' SET'
                            . ' server_out_id = \'' . $serverIdOutgoing . '\''
                            . ' WHERE mail_out_host =  '. $db->quote($serverConfig->mail_out_host)
                            . ' AND mail_out_port =  '. $db->quote($serverConfig->mail_out_port)
                            . ' AND mail_out_use_secure =  '. $db->quote($serverConfig->mail_out_use_secure)
                            . ' AND mail_out_use_sec_auth =  '. $db->quote($serverConfig->mail_out_use_sec_auth)
                            . ' AND use_cms_mailer = \'0\'';

                        $db->setQuery($query);
                        $result	 = $db->query();
                        $affRows = $db->getAffectedRows();
                        $log->debug('migrateMailServerStructure: '.$affRows.' lists updated with server_out_id = '.$serverIdOutgoing, MstConsts::LOGENTRY_INSTALLER);

                    } catch (DatabaseException $e) {
                        $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                        $log->error('migrateMailServerStructure: '.$errorMsg . '(query was: '.$query.')', MstConsts::LOGENTRY_INSTALLER);
                        $errorHappenedDuringExistingServerMigration = true;
                    }

                }

                $log->info('migrateMailServerStructure: Completed outgoing mail handling');

                $query = 'SELECT * FROM ' . $serversTbl;
                $db->setQuery( $query );
                $servers = array();
                try {
                    $servers = $db->loadObjectList();
                    $log->info('migrateMailServerStructure: '.count($servers).' server configurations now present', MstConsts::LOGENTRY_INSTALLER);
                    $log->info('migrateMailServerStructure: server settings: '.print_r($servers, true), MstConsts::LOGENTRY_INSTALLER);
                } catch (DatabaseException $e) {
                    $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                    $log->error('migrateMailServerStructure: '.$errorMsg. '(query was: '.$query.')', MstConsts::LOGENTRY_INSTALLER);
                    $errorHappenedDuringExistingServerMigration = true;
                }
            }

            // Now that all server settings present are migrated we can remove old columns in lists table...
            if(!$errorHappenedDuringExistingServerMigration){ // only preceed if there were now errors before
                $res[] = $dbUtils->deleteColIfExists($listsTbl, 'mail_in_host');
                $res[] = $dbUtils->deleteColIfExists($listsTbl, 'mail_in_port');
                $res[] = $dbUtils->deleteColIfExists($listsTbl, 'mail_in_use_secure');
                $res[] = $dbUtils->deleteColIfExists($listsTbl, 'mail_in_use_sec_auth');
                $res[] = $dbUtils->deleteColIfExists($listsTbl, 'mail_in_protocol');
                $res[] = $dbUtils->deleteColIfExists($listsTbl, 'mail_in_params');
                $res[] = $dbUtils->deleteColIfExists($listsTbl, 'mail_out_host');
                $res[] = $dbUtils->deleteColIfExists($listsTbl, 'mail_out_port');
                $res[] = $dbUtils->deleteColIfExists($listsTbl, 'mail_out_use_secure');
                $res[] = $dbUtils->deleteColIfExists($listsTbl, 'mail_out_use_sec_auth');
                $log->info('migrateMailServerStructure: result removing lists server settings: '.print_r($res, true), MstConsts::LOGENTRY_INSTALLER);
            }else{
                $res[] = -1;
                $log->error('migrateMailServerStructure: considered failure, do not remove any of existing list server settings for now', MstConsts::LOGENTRY_INSTALLER);
            }

            return $res;
        }

        function migrateSendThrottling($listsTbl){
            $log = MstFactory::getLogger();
            $inclPath = JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster'.DS.'mailster'.DS.'utils'.DS.'DBUtils.php';
            require_once($inclPath);
            $dbUtils = new MstDBUtils();
            $inclPath = JPATH_ADMINISTRATOR.DS.'components'.DS.'com_mailster'.DS.'mailster'.DS.'conf'.DS.'Configuration.php';
            require_once($inclPath);

            $log->debug('migrateSendThrottling()', MstConsts::LOGENTRY_INSTALLER);
            $res = array();

            $listSpecificThrottleHourLimitExists = $dbUtils->isColExisting($listsTbl, 'throttle_hour_limit');
            $log->info('Need to migrate send throttling: ' . ($listSpecificThrottleHourLimitExists ? 'Yes' : 'No'), MstConsts::LOGENTRY_INSTALLER);

            if($listSpecificThrottleHourLimitExists){
                $db = JFactory::getDBO();
                $query = 'SELECT COALESCE(MAX(throttle_hour_limit),0) AS max_throttle_hour_limit FROM ' . $listsTbl;
                $db->setQuery( $query );
                $errorMsg = '';
                try {
                    $result = $db->query();
                } catch (DatabaseException $e) {
                    $errorMsg = 'Error No: ' . $e->getCode() . ', Message: ' . $e->getMessage();
                }
                $maxThrottleHourLimit = 0;
                if (!$result){
                    $log->error('migrateSendThrottling: '.$errorMsg);
                }else{
                    $maxThrottleHourLimit = $db->loadResult();
                }

                if($maxThrottleHourLimit > 0){
                    $log->info('migrateSendThrottling: yes, throttle is higher than zero: '.$maxThrottleHourLimit.', therefore save into global setting');

                    $params = MstConfiguration::getComponentParams();
                    if($params){
                        $params->set('max_mails_per_hour', $maxThrottleHourLimit);
                        MstConfiguration::updateComponentParams($params);
                    }
                    $params = MstConfiguration::getComponentParams();
                    if($params->get('max_mails_per_hour', 0) == $maxThrottleHourLimit){
                        $res[] = 1;
                        $log->info('migrateSendThrottling: global setting updated successfully');
                        $res[] = $dbUtils->deleteColIfExists($listsTbl, 'throttle_hour') ? 1 : -1;
                        $res[] = $dbUtils->deleteColIfExists($listsTbl, 'throttle_hour_cr') ? 1 : -1;
                        $res[] = $dbUtils->deleteColIfExists($listsTbl, 'throttle_hour_limit') ? 1 : -1;
                        $log->info('migrateSendThrottling: Result after removing list specific throttle columns: '.print_r($res, true));
                    }else{
                        $res[] = -1;
                        $log->error('migrateSendThrottling: failed to update global setting');
                    }
                }else{
                    $log->info('migrateSendThrottling: send throttle is zero of existing mailing lists, no action necessary');
                    $res[] = 0; // no action needed
                }
            }

            return $res;
        }
		
		function checkUpdateSuccess($version, $res, $printErrorsOnly=false, $dbUpdate=true){
			$updError = false;
			$updAction = false;
			
			for($i=0; $i < count($res); $i++){
				if($res[$i] < 0){
					$updError = true;
					$msgcolor = "#FFB0B0";
					$msg =  $version . ': error in step ' . ($i+1) . '';
					if($dbUpdate){
						$msg = JText::sprintf( 'COM_MAILSTER_UPDATE_TO_VERSION_X_ERROR_IN_STEP_Y' , $version, ($i+1) );
					}
				}
				if($res[$i] > 0){
					$msgcolor = "#B0FFB0";
					$msg =  JText::sprintf( 'COM_MAILSTER_UPDATE_TO_VERSION_X_STEP_Y_OK' , $version, ($i+1) );
					$updAction = true;
				}
				if(($res[$i] != 0 && !$printErrorsOnly) || ($res[$i]<0 && $printErrorsOnly)){
					$this->addMsgRow($msg, $msgcolor);
				}	
			}
			
			if($dbUpdate){
				if($updError){
					$msgcolor = "#FFB0B0";
					$msg = JText::sprintf( 'COM_MAILSTER_DATABASE_UPDATE_TO_VERSION_X_FAILED_CONTACT_SUPPORT' , $version );
				}else{
					if($updAction){
						$msgcolor = "#B0FFB0";
						$msg = JText::sprintf( 'COM_MAILSTER_DATABASE_UPDATE_TO_VERSION_X_SUCCESSFUL' , $version );
					}else{
						// no update action for this version done, 
						// probably an older version as the last installed version
					}
				}
	
				if($updError || $updAction){
					$this->addMsgRow($msg, $msgcolor);
				}
			}
			
			return !$updError;
		}
		
		function addMsgRow($msg, $msgcolor, $onlyHighlightMsg=false, $emptyColCount = 1){
			if($onlyHighlightMsg){
				echo '<tr style="height:30px"><td>&nbsp;</td><td bgcolor="'. $msgcolor .'" >' . $msg . '</td><td colspan="'.$emptyColCount.'">&nbsp;</td>';
			}else{
				echo '<tr bgcolor="'. $msgcolor .'" style="height:30px"><td>&nbsp;</td><td>' . $msg . '</td><td colspan="'.$emptyColCount.'">&nbsp;</td>';
			}
			echo '</tr>';
		}
		
			
		/**
		 * Joomla! 1.6+ won't run the SQL file on updates. It will also run none of
		 * the update SQL files the first time you update an extension to a version
		 * which has update SQL files. Therefore, we need a workaround.
		 * 
		 */
		private function _workaroundApplyInstallationSQL($parent){
			if(method_exists($parent, 'extension_root')) {
				$sqlfile = $parent->getPath('extension_root').'/'.$this->_install_sql_path;
			} else {
				$sqlfile = $parent->getParent()->getPath('extension_root').'/'.$this->_install_sql_path;
			}
            return $this->_executeScriptSQL($sqlfile);
		}


        private function _workaroundApplyUninstallationSQL($parent){
            if(method_exists($parent, 'extension_root')) {
                $sqlfile = $parent->getPath('extension_root').'/'.$this->_uninstall_sql_path;
            } else {
                $sqlfile = $parent->getParent()->getPath('extension_root').'/'.$this->_uninstall_sql_path;
            }
            return $this->_executeScriptSQL($sqlfile);
        }

        private function _executeScriptSQL($sqlfile){
            $db = JFactory::getDBO();
            $buffer = file_get_contents($sqlfile);
            if ($buffer !== false) {
                jimport('joomla.installer.helper');
                $queries = JInstallerHelper::splitSql($buffer);
                if (count($queries) != 0) {
                    foreach ($queries as $query)
                    {
                        $query = trim($query);
                        if ($query != '' && $query{0} != '#') {
                            $db->setQuery($query);
                            if (!$db->query()) {
                                JError::raiseWarning(1, JText::sprintf('JLIB_INSTALLER_ERROR_SQL_ERROR', $db->stderr(true)));
                                return false;
                            }
                        }
                    }
                }
            }
            return true;
        }
	
		/**
		 * Copy the legacy install/uninstall scripts to the component's back-end
		 */
		private function _copyLegacyScripts($parent){
			$installFile = (string)$parent->getParent()->getManifest()->installfile;
			if ($installFile) {
				$path['src']	= $parent->getParent()->getPath('source') . '/' . $installFile;
				$path['dest']	= $parent->getParent()->getPath('extension_administrator') . '/' . $installFile;
				$parent->getParent()->copyFiles(array ($path));
			}
			
			$uninstallFile = (string)$parent->getParent()->getManifest()->uninstallfile;
			if ($uninstallFile) {
				$path['src']	= $parent->getParent()->getPath('source') . '/' . $uninstallFile;
				$path['dest']	= $parent->getParent()->getPath('extension_administrator') . '/' . $uninstallFile;
				$parent->getParent()->copyFiles(array ($path));
			}
			
			$scriptfile = (string)$parent->getParent()->getManifest()->scriptfile;
			if ($scriptfile) {
				$path['src']	= $parent->getParent()->getPath('source') . '/' . $scriptfile;
				$path['dest']	= $parent->getParent()->getPath('extension_administrator') . '/' . $scriptfile;
				$parent->getParent()->copyFiles(array ($path));
			}
		}

        /*
	     * get a variable from the manifest file (actually, from the manifest cache).
	    */
        function getParamFromManifestCache( $name ) {
            $db = JFactory::getDbo();
            $db->setQuery('SELECT manifest_cache FROM #__extensions WHERE element = "com_mailster"');
            $result = $db->loadResult();
            if($result){
                $manifest = json_decode($result , true );
                if($manifest && array_key_exists($name, $manifest)){
                    return $manifest[ $name ];
                }
            }
            return false;
        }
		
	}
?>
