<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	defined( '_JEXEC' ) or die( 'Restricted access' );

	jimport('joomla.application.component.controller');

	/**
	 * Mailster Component Profile Controller
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterControllerProfile extends MailsterController
	{
		function __construct(){
			parent::__construct();			
		}
		
		function view(){
			/*$authorization = &MstFactory::getAuthorization();
			if( !$authorization->frontendMenuViewAccessOK() ){ return false; }*/			
			$view  = $this->getView( 'profile', 'html'  );
			$view->display();		
		}
		
		function digest(){
			
			$view  = $this->getView( 'profile', 'html'  );
			$view->setLayout('digest');
			$view->displayDigest();
			
		}
		
		function saveDigest(){
			$log = MstFactory::getLogger();
			$user = JFactory::getUser();
			$subscrUtils = MstFactory::getSubscribeUtils();
			$digestModel = MstFactory::getModel('digest');
			$convUtils = MstFactory::getConverterUtils();
			
			$log->debug('saveDigest CALLED');
							
			$listId = JRequest::getInt('listID', 0);
			$digestId = JRequest::getInt('digestID', 0);
			$digestFreq = JRequest::getInt('digest_freq', 0);
			$backLink = base64_decode(JRequest::getString('bl', null));
			
			if($user->id <= 0){
				JFactory::getApplication()->enqueueMessage(JText::_('COM_MAILSTER_YOU_NEED_TO_LOGIN_TO_ACCESS_THIS_SECTION'), 'error');
				return;
			}
			
			$listUtils = MstFactory::getMailingListUtils();
			$mList = $listUtils->getMailingList($listId);
			
			if($mList){
				if($mList->allow_digests){
					if($subscrUtils->isUserSubscribedToMailingList($user->id, 1, $listId)){
						if($digestId > 0){
							$digest = $convUtils->object2Array($digestModel->getDigest($digestId));
						}else{
							$digest = array();
							$digest['id'] = $digestId;
							$digest['list_id'] = $listId;
							$digest['user_id'] = $user->id;
							$digest['is_joomla_user'] = 1;
						}
						$digest['digest_freq'] = $digestFreq;
						$log->debug('Digest: '.print_r($digest, true));
						$digestModel->store($digest);
					}
				}
			}
						
			if(is_null($backLink)){
				$this->view();
			}else{
				$this->setRedirect( $backLink );
			}
		}

		function subscribe(){
			$log = MstFactory::getLogger();
			$user = JFactory::getUser();
			$subscrUtils = MstFactory::getSubscribeUtils();

			$listId = JRequest::getInt('listID', 0);
			$backLink = base64_decode(JRequest::getString('bl', null));
			
			if($user->id <= 0){
				JFactory::getApplication()->enqueueMessage(JText::_('COM_MAILSTER_YOU_NEED_TO_LOGIN_TO_ACCESS_THIS_SECTION'), 'error');
				return;
			}
			$listUtils = MstFactory::getMailingListUtils();
			$mList = $listUtils->getMailingList($listId);
			if($mList){
				if($mList->allow_subscribe){
					if($mList->subscribe_mode != MstConsts::SUBSCRIBE_MODE_DOUBLE_OPT_IN){
						$log->debug('Double Opt-in subscribe mode not activated (frontend)');
						$success = $subscrUtils->subscribeUserId($user->id, true, $listId);
						$subscrUtils->sendWelcomeOrGoodbyeSubscriberMsg($user->name, $user->email, $listId, MstConsts::SUB_TYPE_SUBSCRIBE);
						if($success){
							JFactory::getApplication()->enqueueMessage(JText::_('COM_MAILSTER_SUBSCRIPTION_SUCCESSFUL'));
						}else{
							JFactory::getApplication()->enqueueMessage(JText::_('COM_MAILSTER_SUBSCRIPTION_FAILED'), 'error');
						}
					}else{
						$log->debug('Double Opt-in subscribe mode (frontend)');
						$subscrUtils->subscribeUserWithDoubleOptIn($user->email, $user->name, $listId);
						JFactory::getApplication()->enqueueMessage(JText::_('COM_MAILSTER_SUBSCRIPTION_SUCCESSFUL_CONFIRM_BY_CLICKING_LINK_IN_CONFIRMATION_EMAIL_SENT_TO_YOU'));
					}					
				}
			}

			if(is_null($backLink)){
				$this->view();
			}else{
				$this->setRedirect( $backLink );
			}
		}

		function unsubscribe(){
			$log = MstFactory::getLogger();
			$user = JFactory::getUser();
			$subscrUtils = MstFactory::getSubscribeUtils();

			$listId = JRequest::getInt('listID', 0);
			$backLink = base64_decode(JRequest::getString('bl', null));
			
			if($user->id <= 0){
				JFactory::getApplication()->enqueueMessage(JText::_('COM_MAILSTER_YOU_NEED_TO_LOGIN_TO_ACCESS_THIS_SECTION'), 'error');
				return;
			}
			
			$listUtils = MstFactory::getMailingListUtils();
			$mList = $listUtils->getMailingList($listId);
			if($mList){
				if($mList->allow_unsubscribe){
					if($mList->unsubscribe_mode != MstConsts::UNSUBSCRIBE_MODE_DOUBLE_OPT_IN){
						$log->debug('Double Opt-in unsubscribe mode not activated (frontend)');
						$success = $subscrUtils->unsubscribeUserId($user->id, true, $listId);
						$tmpUser = $subscrUtils->getUserByEmail($user->email);
						$subscrUtils->sendWelcomeOrGoodbyeSubscriberMsg($tmpUser['name'], $user->email, $listId, MstConsts::SUB_TYPE_UNSUBSCRIBE);
						if($success){
							JFactory::getApplication()->enqueueMessage(JText::_('COM_MAILSTER_UNSUBSCRIPTION_SUCCESSFUL'));
						}else{
							JFactory::getApplication()->enqueueMessage(JText::_('COM_MAILSTER_UNSUBSCRIPTION_FAILED'), 'error');
						}
					}else{
						$log->debug('Double Opt-in unsubscribe mode (frontend)');
						$subscrUtils->unsubscribeUserWithDoubleOptIn($user->email, $listId);
						JFactory::getApplication()->enqueueMessage(JText::_('COM_MAILSTER_UNSUBSCRIPTION_SUCCESSFUL_CONFIRM_BY_CLICKING_LINK_IN_CONFIRMATION_EMAIL_SENT_TO_YOU'));
					}					
				}
			}
			if(is_null($backLink)){
				$this->view();
			}else{
				$this->setRedirect( $backLink );
			}
		}
	}
?>
