<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	defined( '_JEXEC' ) or die( 'Restricted access' );

	jimport('joomla.application.component.controller');

	/**
	 * Mailster Component Subscriptions Controller
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterControllerSubscriptions extends MailsterController
	{	
		public function subscribe()
		{
			$view = $this->getView('subscribe', 'html');
			$view->display();
		}

		public function sub3rd(){
			// 3rd Party Subscription
			$log = MstFactory::getLogger();
			$app = JFactory::getApplication();
			$subscrUtils = MstFactory::getSubscribeUtils();
			$listUtils = MstFactory::getMailingListUtils();
			$listId = (int) JRequest::getInt('list', 0);
			
			$encodedReturnUrlOK = JRequest::getString('urlok', '');
			$returnUrlOK = base64_decode($encodedReturnUrlOK);
			$encodedReturnUrlFailed = JRequest::getString('urlfailed', '');
			$returnUrlFailed = base64_decode($encodedReturnUrlFailed);
			
			$name = JRequest::getString('name', '');
			$email = JRequest::getString('email', null);

            $blockedMail = false;
            if(trim(strtolower($email)) === 'sample@email.tst'){
                $blockedMail = true; // TODO IMPLEMENT INTRODUCE FLEXIBLE WAY TO NOT DEAL WITH KNOWN SPAMMER ADDRESSES
            }

            if(!$blockedMail){
                $log->info('sub3rd() 3rd Party Subscription (email: '.$email.', name: '.$name.') to list ID '.$listId);

                $success = false;
                $mList = $listUtils->getMailingList($listId);
                if($mList){
                    if($mList->allow_subscribe){
                        if($email){
                            if($mList->subscribe_mode == MstConsts::SUBSCRIBE_MODE_DOUBLE_OPT_IN){
                                $log->debug('sub3rd() 3rd Party Subscription: Double Opt-In');
                                $success = $subscrUtils->subscribeUserWithDoubleOptIn($email, $name, $listId);
                            }else{
                                $log->debug('sub3rd() 3rd Party Subscription: NO Double Opt-In');
                                $success = $subscrUtils->subscribeUser($name, $email, $listId); // subscribing user...
                                $subscrUtils->sendWelcomeOrGoodbyeSubscriberMsg($name, $email, $listId, MstConsts::SUB_TYPE_SUBSCRIBE);
                            }
                            if($success){
                                $log->debug('sub3rd() 3rd Party Subscription: Subscribe OK');
                                if($returnUrlOK){
                                    $log->debug('sub3rd() 3rd Party Subscription: Redirect to OK url: '.$returnUrlOK);
                                    $app->redirect($returnUrlOK);
                                }else{
                                    $log->debug('sub3rd() 3rd Party Subscription: No OK redirection URL given, do not redirect');
                                }
                            }else{
                                $log->error('sub3rd() 3rd Party Subscription: Subscribe FAILED');
                            }
                        }else{
                            $log->error('sub3rd() 3rd Party Subscription: No email address supplied');
                        }
                    }else{
                        $log->error('sub3rd() 3rd Party Subscription: Subscription is not active/allowed');
                    }
                }else{
                    $log->error('sub3rd() 3rd Party Subscription: No mailing list supplied');
                }

                if(!$success){
                    if($returnUrlFailed){
                        $log->debug('sub3rd() 3rd Party Subscription: Redirect to Failed url: '.$returnUrlFailed);
                        $app->redirect($returnUrlFailed);
                    }else{
                        $log->debug('sub3rd() 3rd Party Subscription: No Failed redirection URL given, do not redirect');
                    }
                }
            }
		}

		public function unsubscribe()
		{	
			$log = MstFactory::getLogger();
			$recips = MstFactory::getRecipients();
			$hashUtils = MstFactory::getHashUtils();
			$subscrUtils = MstFactory::getSubscribeUtils();
			$listUtils = MstFactory::getMailingListUtils();
			$listId = JRequest::getInt('list_id', 0);
			$mailId = JRequest::getInt('mail_id', 0);
			$hashOk = JRequest::getBool('hash_ok', 0);
			$salt 	= JRequest::getInt('salt', rand());
			$hash 	= JRequest::getString('hash', "");
			$email 	= JRequest::getString('email', "");

			$hashOk = ($hashOk && $hashUtils->checkUnsubscribeKeyOfMail($mailId, $salt, $hash));
			
			$view = $this->getView('unsubscribe', 'html');
			$defaultErrorMsg = "";
			$view->assignRef('error_message', $defaultErrorMsg);
			
			$mList = $listUtils->getMailingList($listId);
			$isRecipient = $recips->isRecipient($listId, $email);
			
			if($hashOk){
				if($isRecipient){
					if($mList->unsubscribe_mode != MstConsts::UNSUBSCRIBE_MODE_DOUBLE_OPT_IN){
						$log->debug('Double Opt-in unsubscribe mode not activated (frontend)');
						$success = $subscrUtils->unsubscribeUser($email, $listId);
						$tmpUser = $subscrUtils->getUserByEmail($email);
						$subscrUtils->sendWelcomeOrGoodbyeSubscriberMsg($tmpUser['name'], $email, $listId, MstConsts::SUB_TYPE_UNSUBSCRIBE);
						if($success){
							$view->unsubscriptionOk();
						}else{
							$view->unsubscriptionFailed();
						}
					}else{
						$log->debug('Double Opt-in unsubscribe mode (frontend)');	
						$subscrUtils->unsubscribeUserWithDoubleOptIn($email, $listId); 
						$view->confirmationEmailSent();
					}
				}else{
					$view->assignRef('error_message', JText::_( 'COM_MAILSTER_EMAIL_NOT_SUBSCRIBED' ));
					$view->unsubscriptionFailed();
				}
			}else{
				$view->unsubscriptionFailed();
			}
		}
		
	}
?>
