<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	 
	defined( '_JEXEC' ) or die( 'Restricted access' );

	jimport( 'joomla.application.component.view');
	
	/**
	 * HTML View class for the Profile View
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterViewProfile extends MailsterView
	{
		function display($tpl = null)
		{				
			$menu = JSite::getMenu();
			$menuParams = $menu->getParams( JRequest::getInt( 'Itemid' ) );
			$log = MstFactory::getLogger();
			$subscrUtils = MstFactory::getSubscribeUtils();
			$digestModel = MstFactory::getModel('digest');
			$user = JFactory::getUser();
			if($user->id <= 0){
				JFactory::getApplication()->enqueueMessage(JText::_('COM_MAILSTER_YOU_NEED_TO_LOGIN_TO_ACCESS_THIS_SECTION'), 'error');
			}		

			$pageclass_sfx = $menuParams->get('pageclass_sfx');
			$menuTitle	= $menuParams->get('title');
			$menuTitle = JText::_('COM_MAILSTER_MAILING_LISTS');
			
			$menuParamShowHeading = $menuParams->get('show_page_heading');
			$menuParamPageHeading = $menuParams->get('page_heading');
			$defaultPageHeading = $menuParamShowHeading ? $menuParamPageHeading : $menuTitle;
			
			$log->debug('menuParamShowHeading: '.$menuParamPageHeading);
			$log->debug('menuParamPageHeading: '.$menuParamPageHeading);
			$log->debug('menuTitle: '.$menuTitle);
			
			$lists = $subscrUtils->getMailingLists2RegisterAt(false, true);
			
			for($i=0, $n=count($lists); $i<$n; $i++){
				$lists[$i]->isSubscribed = $subscrUtils->isUserSubscribedToMailingList($user->id, true, $lists[$i]->id);
				$lists[$i]->digest = false;
				if($lists[$i]->isSubscribed){
					if($digestModel->isUserDigestRecipientOfList($user->id, 1, $lists[$i]->id)){
						$digests = $digestModel->getDigestsOfUser($user->id, 1, $lists[$i]->id);
						if($digests && count($digests) > 0){
							for($j=0; $j<count($digests); $j++){
								if($digests[$j]->digest_freq > 0){
									$lists[$i]->digest = $digests[$j];
								}
							}
						}
					}
				}
			}
			
			$log->debug('MailsterViewProfile->display '.print_r($lists, true));
			
			$currUrl = JRoute::_('index.php?&option=com_mailster&view=profile&Itemid='.JRequest::getInt( 'Itemid' ) );
						
			$this->assignRef('currurl', $currUrl);
			$this->assignRef('user', $user);
			$this->assignRef('lists', $lists);
			$this->assignRef('title', $menuTitle);
			$this->assignRef('pageclass_sfx', $pageclass_sfx);
			
			parent::display($tpl);
		}
		
		function displayDigest($tpl = null)
		{
			
			$log = MstFactory::getLogger();
			$menu = JSite::getMenu();
			$menuParams = $menu->getParams( JRequest::getInt( 'Itemid' ) );
			$user = JFactory::getUser();
			$digestModel = MstFactory::getModel('digest');
			
			$pageclass_sfx = $menuParams->get('pageclass_sfx');
			$menuTitle	= $menuParams->get('title');
			$menuTitle = JText::_('COM_MAILSTER_EDIT_DIGEST');
			
			$menuParamShowHeading = $menuParams->get('show_page_heading');
			$menuParamPageHeading = $menuParams->get('page_heading');
			$defaultPageHeading = $menuParamShowHeading ? $menuParamPageHeading : $menuTitle;
			
			$listId = JRequest::getInt('listID', 0);
			$digestId = JRequest::getInt('digestID', 0);
			$backLinkEncoded = JRequest::getString('bl', null);
			
			$digest = false;
			if($digestId > 0){
				$digest = $digestModel->getDigest($digestId);
			}else{
				$digests = $digestModel->getDigestsOfUser($user->id, true, $listId);
				if($digest && count($digest)>0){
					$digest = $digests[0]; 
				}
			}
			
			$digestFreq = 0;
			if($digest){
				$digestFreq = $digest->digest_freq;
			}
				
			if($user->id <= 0){
				JFactory::getApplication()->enqueueMessage(JText::_('COM_MAILSTER_YOU_NEED_TO_LOGIN_TO_ACCESS_THIS_SECTION'), 'error');
			}
			$listUtils = MstFactory::getMailingListUtils();
			$mList = $listUtils->getMailingList($listId);

			$currUrl = JRoute::_('index.php?&option=com_mailster&view=profile&Itemid='.JRequest::getInt( 'Itemid' ) );

			$lists = array();
			$lists['digest_freq'] = $digestModel->getDigestChoiceHtml($digestFreq);

			$this->assignRef('currurl', $currUrl);
			$this->assignRef('backLink', $backLinkEncoded);
			$this->assignRef('lists', $lists);
			$this->assignRef('user', $user);
			$this->assignRef('mList', $mList);
			$this->assignRef('digest', $digest);
			$this->assignRef('title', $menuTitle);
			$this->assignRef('pageclass_sfx', $pageclass_sfx);
		
			parent::display($tpl);
		}

	}
?>
