<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	 
	defined( '_JEXEC' ) or die( 'Restricted access' );

	jimport( 'joomla.application.component.view');

	
	/**
	 * HTML View for subscribing
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterViewSubscribe extends MailsterView
	{
		function display($tpl = null)
		{	
			$log = MstFactory::getLogger();
			$listUtils = MstFactory::getMailingListUtils();
			$hashUtils = MstFactory::getHashUtils();
			$subscribeUtils = MstFactory::getSubscribeUtils();
			$subscribeMode = JRequest::getInt('sm', 0);
			
			$salt 	= JRequest::getInt('s', rand());
			$hash 	= JRequest::getString('h', "");
						
			if($subscribeMode == 0){
				$log->debug('Default subscribe, not yet existing...');
			}elseif($subscribeMode == MstConsts::SUBSCRIBE_MODE_DOUBLE_OPT_IN){
				$subscriptionId	= JRequest::getInt('si', 0);
				$log->debug('Double Opt-In subscribe');
				$subscribeInfo = $subscribeUtils->getSubscribeInfo($subscriptionId);
				if(!is_null($subscribeInfo)){
					$log->debug('Found subscribe info: '.print_r($subscribeInfo, true));
					$saltedKeyHash = $hashUtils->getSubscribeKey($salt, $subscribeInfo->hashkey);
					$hashOk = ($saltedKeyHash==$hash);
				}else{
					$hashOk = false;
				}
				if($hashOk){
					$listId = $subscribeInfo->list_id;
					$subscrUtils = MstFactory::getSubscribeUtils();
					$success = $subscrUtils->subscribeUser($subscribeInfo->name, $subscribeInfo->email, $listId, $subscribeInfo->digest_freq);
					$subscrUtils->sendWelcomeOrGoodbyeSubscriberMsg($subscribeInfo->name, $subscribeInfo->email, $listId, MstConsts::SUB_TYPE_SUBSCRIBE);
					if($success){
						$log->debug('Subscribing successful');
						if($subscribeInfo->add2group){
							$log->debug('User is added to group '.$subscribeInfo->add2group.' after subscribe');
							
							$subscrUser = $subscrUtils->getUserByEmail($subscribeInfo->email, true);
							$subscrUser->group_id = $subscribeInfo->add2group;
							$log->debug('Subscr User:'.print_r($subscrUser, true));
							$groupUserModel = MstFactory::getModel('groupusers');
							$add2GroupSuccess = $groupUserModel->store($subscrUser);
							$log->debug('User added to group: '.($add2GroupSuccess ? 'Yes' : 'No'));
						}
						// ####### TRIGGER NEW EVENT #######
						$mstEvents = MstFactory::getEvents();
						$mstEvents->userSubscribedOnWebsite($subscribeInfo->name, $subscribeInfo->email, $listId);
						// #################################
						$subscrUtils->deleteSubscriptionInfo($subscribeInfo->id);
						parent::display('successful');
						return;
					}else{
						$log->debug('Subscribing failed');
						parent::display('failed');
						return;
					}
				}else{
					$log->debug('Subscribing failed, hash was not correct');
					parent::display('failed');
					return;
				}
			}
			
			$this->assignRef('list_id'    	 	, $listId);
			$this->assignRef('salt'    			, $salt);
			$this->assignRef('hash'				, $hash);
			$this->assignRef('hash_ok' 	 		, $hashOk);
			$this->assignRef('subscribe_mode'	, $subscribeMode);
			
			
			parent::display('failed'); // we fail as we should not end up here
			
		}

	}
?>
