<?php
	/**
	 * @package Joomla
	 * @subpackage Mailster
	 * @copyright (C) 2010 - 2016 Holger Brandt IT Solutions
	 * @license GNU/GPL, see license.txt
	 * Mailster is free software; you can redistribute it and/or
	 * modify it under the terms of the GNU General Public License 2
	 * as published by the Free Software Foundation.
	 * 
	 * Mailster is distributed in the hope that it will be useful,
	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 * GNU General Public License for more details.
	 * 
	 * You should have received a copy of the GNU General Public License
	 * along with Mailster; if not, write to the Free Software
	 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
	 * or see http://www.gnu.org/licenses/.
	 */

	 
	defined( '_JEXEC' ) or die( 'Restricted access' );

	jimport( 'joomla.application.component.view');

	
	/**
	 * HTML View for unsubscribing
	 *
	 * @package Joomla
	 * @subpackage Mailster
	 */
	class MailsterViewUnsubscribe extends MailsterView
	{
		function display($tpl = null)
		{	
			$log = MstFactory::getLogger();
			$recips = MstFactory::getRecipients();
			$listUtils = MstFactory::getMailingListUtils();
			$hashUtils = MstFactory::getHashUtils();
			$subscribeUtils = MstFactory::getSubscribeUtils();
			$unsubscribeMode = JRequest::getInt('sm', 0);
			
			$salt 	= JRequest::getInt('s', rand());
			$hash 	= JRequest::getString('h', "");
			
			$defaultErrorMsg = "";
			$this->assignRef('error_message', $defaultErrorMsg);
			
			$notSubscribed = false;
			
			if($unsubscribeMode == 0){
				$log->debug('Default unsubscribe');
				$mailId	= JRequest::getInt('m', 0);
				$listId = $listUtils->getMailingListIdByMailId($mailId);
				$list = $listUtils->getMailingList($listId);
				$hashOk = $hashUtils->checkUnsubscribeKeyOfMail($mailId, $salt, $hash);
				$this->assignRef('mail_id'	, $mailId);
			}elseif($unsubscribeMode == MstConsts::UNSUBSCRIBE_MODE_DOUBLE_OPT_IN){
				$subscriptionId	= JRequest::getInt('si', 0);
				$log->debug('Double Opt-In unsubscribe with subscriptionId: '.$subscriptionId);
				$subscribeInfo = $subscribeUtils->getSubscribeInfo($subscriptionId);
				if(!is_null($subscribeInfo)){
					$log->debug('Found subscribe info: '.print_r($subscribeInfo, true));
					$saltedKeyHash = $hashUtils->getUnsubscribeKey($salt, $subscribeInfo->hashkey);
					$hashOk = ($saltedKeyHash==$hash);
				}else{
					$hashOk = false;
					$notSubscribed = true;
					$log->debug('Did NOT find subscribe info');
				}
				if($hashOk){
					$listId = $subscribeInfo->list_id;
					$subscrUtils = MstFactory::getSubscribeUtils();
					$isRecipient = $recips->isRecipient($listId, $subscribeInfo->email);
					if($isRecipient){
						$log->debug($subscribeInfo->email.' is a recipient of list '.$listId);
						$success = $subscrUtils->unsubscribeUser($subscribeInfo->email, $listId);
						$tmpUser = $subscrUtils->getUserByEmail($subscribeInfo->email);
						$subscrUtils->sendWelcomeOrGoodbyeSubscriberMsg($tmpUser['name'], $subscribeInfo->email, $listId, MstConsts::SUB_TYPE_UNSUBSCRIBE);
						if($success){
							$log->debug('Unsubscribing successful');
							// ####### TRIGGER NEW EVENT #######
							$mstEvents = MstFactory::getEvents();
							$mstEvents->userUnsubscribedOnWebsite($subscribeInfo->email, $listId);
							// #################################
							$subscrUtils->deleteSubscriptionInfo($subscribeInfo->id);
							parent::display('successful');
							return;
						}else{
							$log->debug('Unsubscribing failed');
							parent::display('failed');
							return;
						}
					}else{
						$log->debug($subscribeInfo->email.' is NOT a recipient of list '.$listId);
						$notSubscribed = true;
					}
				}else{
					$log->debug('Hash not ok');
				}
				if($notSubscribed){
					$log->debug('Unsubscribing failed, hash was not correct');
					$this->assignRef('error_message', JText::_( 'COM_MAILSTER_EMAIL_NOT_SUBSCRIBED' ));
					parent::display('failed');
					return;
				}
			}
			$this->assignRef('list'     		, $list);
			$this->assignRef('list_id'    	 	, $listId);
			$this->assignRef('salt'    			, $salt);
			$this->assignRef('hash'				, $hash);
			$this->assignRef('hash_ok' 	 		, $hashOk);
			$this->assignRef('unsubscribe_mode'	, $unsubscribeMode);
			
			parent::display($tpl);
		}

		function unsubscriptionOk(){
			parent::display('successful');
		}

		function unsubscriptionFailed(){
			parent::display('failed');
		}

		function confirmationEmailSent(){
			parent::display('confirm_email_sent');
		}

	}
?>
